
/*--

  \chapter{Electrocardiognosis specification}

  \FileTitle{
  {\tt ecgnosis.h} --- overall header
  }
  \CopyrightNotice{

    Copyright (c) 1991--1996 Alexandru Dan Corlan MD (http://dan.corlan.net)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

  }

  \PackagePurpose{
  An interactive and/or batch application that processes ECG and pressional 
  long-term (hours, days) recordings.
  }

  
  \section{Main}
  
  \PackagePurpose{
  The main function of the Electrocardiognosis program and it's
  application-wide derivatives.
  }
  
  \begin{code}
  --*/


/*--
  \end{code}

  \subparagraph{System-wide constants}

  \begin{code}
  --*/

#define CCSIN24H 2073600000                                     /* [DEF] */ 


/*--
  \end{code}

  \subparagraph{Debug message logging}

  \begin{code}
  --*/

extern FILE *ecgnosislog;  					/* [VAR] */ 

/*--
  \end{code}

  \section{Viewer}

  \PackagePurpose{
  This package contains the user interface of the Electrocardiognosis
  application. 
  }

  \FunctionInterraction{
  The viewer contains, loosely speaking, one module for each type of diagram
  or interraction mode of the Electrocardiognosis application.
  The main function in each module is an ``interraction'' function, such as
  \functiontag{td\_interract} or \functiontag{scat\_interract} which
  first repaints the whole screen in a suitable manner and than performs
  the specific interraction and the resulted actions on the data file.
  }


  \dataparagraph{The dwins that make up the display}

  \Definition{
  These variables keep the numbers of all the dwins that make up the
  displays of the Electrocardiognosis program.
  }
  \SemanticStructure{
  The values of the various dwins, as allocated with \functiontag{newdwin}
  sometimes at startup.
  }
  \ConsistencyRules{


  }

  \begin{code}
  --*/


extern short upd;      /* Upper DWIN */
extern short lod;      /* Lower DWIN */
extern short umctrld;  /* Upper-Medium Control DWIN */
extern short lctrld;   /* Lower Control DWIN */
extern short rjumps;   /* Right side buttons */
extern short fulld;    /* Full display dwin */
extern short prod;     /* Place for progression indicator */
extern short scatd;    /* The scatter diagram */
extern short scatpd;   /* The scatter diagram position */
extern short topd;     /* The uppermost line of the screen */


/*--
  \end{code}

  \dataparagraph{Interraction command symbols}

  \begin{code}
  --*/

#define INTERRACTION_EXIT 0
#define GO_POINCARE 1
#define GO_RECORDER 2
#define GO_TIME_DOMAIN 3
#define GO_SEQUENCE 4
#define GO_SRV 5
#define GO_FMANAGER 6
#define GO_PMANAGER 7
#define GO_MORPHO 8
#define GO_REPORT 9

/*--
  \end{code}

  \subsection{Colors management}

  \dataparagraph{The color definition dwins}

  \begin{code}
  --*/


extern short colpadd;
extern short redd;
extern short greend;
extern short blued;


/*--
  \end{code}

  \dataparagraph{Color management widget dimensions}

  \Definition{
  Constants that define the dimensions and placement on the
  screen of the color management widgets.
  }

  \begin{code}
  --*/

#define COLU 20
#define COLPXORG 540
#define COLPYORG 25
#define COLCYORG 165


/*--
  \end{code}

  \dataparagraph{General use color definitions}

  \begin{code}
  --*/

extern short reds[];
extern short greens[];
extern short blues[];

/*--
  \end{code}

  \subsection{Strip and Time Diagrams Viewers}

  \begin{code}
  --*/


/*--
  \end{code}

  \subsection{Patient Data Viewer}

  \PackagePurpose{
  This package provides editing of the patient information (epidemiological
  and clinical) and also editing of site and users information.
  }
  \FunctionInterraction{
  There are similar subpackages for each of the types of data
  taken into account: site where processing takes place, users
  of the program and data pertaining to the patient.

  All the subpackages are initialized by the {\tt egviewdt()} function call.
  Data is stored in static variables. There are three important
  functions in each of the subpackages: {\tt read\_patient\_data},
  {\tt edit\_patient\_data} and {\tt write\_new\_patient\_data} --- for
  the data on the patient. 
  }

  \begin{code}
  --*/


/*--
  \end{code}

  \subsection{Heart Rate Variability Viewer}

  \PackagePurpose{
  Implements the user interaction for heart rate variability, including
  also user interaction for record editing (for HRV purposes).
  }

  \begin{code}
  --*/


/*--
  \end{code}

  \section{Interpreter}

  \PackagePurpose{
  Executes [batch] processing of signal recordings under the control
  of a Lisp-like script.
  }

  \begin{code}
  --*/

short batch_qrs_detect(PERFILE *p, char *qrsfn); 		/* [FUN] */ 

/*--
  \end{code}

  \section{Diagrams}

  \PackagePurpose{
  Contains the executing functions for each type of analyses performed
  by Electrocardiognosis --- the result of each type of analyses being
  a ``diagram''.
  }

  \FunctionInterraction{
  Subpackages with similar functionality exist for each type of diagram in
  part.
  }

  \begin{code}
  --*/

extern short use_figure_environments;

/*--
  \end{code}

  \subsection{Strips}

  \PackagePurpose{
  Performs any processings on simple ECG strips.
  }

  \FunctionInterraction{
  The various parameter seting functions should be called first, then
  one of the \verb|put_strip| functions which will produce the required
  type of output.
  }

  \begin{code}
  --*/


/*--
  \end{code}

  \dataparagraph{Paper speed symbols} 

  \begin{code}
  --*/

#define S10mmps   100
#define S25mmps   250
#define S50mmps   500
#define S100mmps 1000
#define S200mmps 2000
#define S2p5mmps   25


/*--
  \end{code}

  \dataparagraph{Paper lineage formats}

  \begin{code}
  --*/


#define LINED_GRID  2
#define DOTTED_GRID 1
#define NO_GRID     0


/*--
  \end{code}

  \funparagraph{put\_strip\_to\_eps}

  \PurposeReturn{
  Prints in the argument eps file the strips from the patient file begining
  with the given start position, in encapsulated postscript format.
  }

  \begin{code}
  --*/

short put_strip_to_eps(FILE *f, PERFILE *p, long t0);


/*--
  \end{code}

  \funparagraph{put\_strip\_to\_latex\_list}
  \PurposeReturn{
  This function adds a strip to the current list of diagrams in the
  latex report, doing everything necessary, such as labels and titles.
  }

  \begin{code}
  --*/

void put_strip_to_latex_list(PERFILE *p, long m0);

/*--
  \end{code}

  \funparagraph{File name functions}

  \begin{code}
  --*/

FILE *the_epsf();

/*--
  \end{code}

  \subsection{Time diagrams}

  \PackagePurpose{
  Functions and data structures specific to the time diagrams.
  }

  \dataparagraph{Registry of all time-diagram types}

  \Definition{
  Each of the time-diagrams is either user-defined or predefined. During a
  session, some new diagrams are added. Diagrams are designated internally
  by indexes in a vector [of diagrams]. This data structure
  identifies the predefined diagrams and describes the vector.
  }


  \subparagraph{Identifiers of the diagrams}

  \begin{code}
  --*/

#define F_DIAGRAM        1
#define RR_DIAGRAM       2
#define ST1_DIAGRAM      3
#define ST2_DIAGRAM      4
#define J_DIAGRAM        5
#define BL_DIAGRAM       6
#define STMOD_DIAGRAM    7
#define TPEAK_DIAGRAM    8
#define QTPEAK_DIAGRAM   9
#define TSIM_DIAGRAM    10
#define VGRAD_DIAGRAM   11
#define ST1MED_DIAGRAM  12
#define ST2MED_DIAGRAM  13


/*--
  \end{code}

  \subparagraph{Dimensional constants}

  \begin{code}
  --*/


#define MAX_NOF_TDS 100
#define MAX_TD_NAME  10


/*--
  \end{code}

  \subparagraph{Vector element data type}

  \begin{code}
  --*/

typedef struct timediag{
  char tdname[MAX_TD_NAME+2];  /* making space for the null ending */
  void *formula;               /* used only when language gets in force */
  short u_base;
  short u_incr;
  short d_base;
  short d_incr;
} TIMEDIAG;


/*--
  \end{code}

  \subparagraph{The vector and its fill pointer}

  \begin{code}
  --*/


extern TIMEDIAG tds[MAX_NOF_TDS];
extern short nof_tds;


/*--
  \end{code}

  \funparagraph{Constructors and selectors for td types}

  \begin{code}
  --*/

char *td_name(short k);

void set_td_resolution(long res);  /* in ccs */
void set_td_graphic_step(double t);  /* in bp */       		/* [FUN] */ 
short set_td_scale_factor(short k, float fact);
void set_td_min_scale_value(short k, float val);
void set_td_scale_label_format(short k, char *ff);

/*--
  \end{code}

  \subsection{Scattergrams}

  \dataparagraph{Current scattergram}

  \begin{code}
  --*/

extern long scat_dim;

extern short **scat;
extern short **dscat;


/*--
  \end{code}

  \dataparagraph{Boundary descriptions}  % a name for the data structure

  \begin{code}
  --*/

#define MAX_BOUNDS 10

extern short ver_bounds[];
extern short ver_bounds_fill;

extern short hor_bounds[];
extern short hor_bounds_fill;


/*--
  \end{code}

  \dataparagraph{SCATMES --- scattergram measurements}

  \Definition{
  This data structure will contain the results of different computations
  made on any sector(s) of the scattergram.
  }
  \SemanticStructure{
  For every sector, as numbered in figure~\ref{fig:vectnum}, an instance
  of the \typetag{scatmes} structure is used. The fill pointer
  in this vector of records is reprezented by $F_H \times F_V$.

  Each of the result slots in the result structure is accompanied
  by a ``guardian'' slot, with the name ended in ``g'' which tells
  whether the value of the slot was not yet computed, was computed but
  resulted in an error, or was computed and the slot holds
  the correct result.

  The meanings of the slots are:

  \begin{description}
  \item[volume] is the total number of points found in a sector;
  \item[surface] is the total number of points with mutually different
  coordinates found in a sector;
  \item[xmed] is the arithmetic mean of the $x$ coordinate of
  all the points in the sector;
  \item[ymed] is the arithmetic mean of the $y$ coordinate of
  all the points in the sector;
  \item[xdisp] is the dispersion on the $x$ coordinate, i.e.
  the standard deviation of the $x$ coordinates of all
  the points in that sector;
  \item[ydisp] is the dispertion on the $y$ coordinate, i.e.
  the standard deviation of the $y$ coordinates of all
  the points in the sector;
  \item[corr] is the corration coeficient for association
  between the $x$ and $y$ coordinates of all the points in the
  sector;
  \item[oimed] is the arithmetic mean of all the points, as
  projected on the first axis (45$^0$);
  \item[oiimed] is the arithmetic mean of all the points, as
  projected on the seconf axis (135$^0$);
  \item[dispi] is the dispersion of the distribution obtained
  by projecting all the points on the first axis (45$^0$);
  \item[dispii] is the dispersion of the distribution obtained
  by prjecting all the points on the second axis (135$^0$).
  \item[lsup] is the uppermost line in the sector which still contains
  one point;
  \item[linf] is the lowermost line in the sector which still contains
  one point;
  \item[lleft] is the leftmost line in the sector which still contains
  one point;
  \item[lright] is the rightmost line in the sector which still contains
  one point.

  \end{descrition}
  }
  \ConsistencyRules{


  The guardians (the slots ending in ``g'') will contain one of the
  three symbols, \constanttag{SM\_ERROR}, \constanttag{SM\_NOTCO} or
  \constanttag{SM\_DONE}.
  }

  \begin{code}
  --*/

#define SM_ERROR -1     /* error when computing */
#define SM_NOTCO  0     /* not computed */
#define SM_DONE   1     /* computed, done */

struct scatmes {
  long volume;
  short volg;
  
  long surface;
  short surg;
  
  float xmed;
  short xmeg;
  
  float ymed;
  short ymeg;

  short lsup;
  short lsg;

  short linf;
  short lig;

  short lleft;
  short llg;

  short lright;
  short lrg;
  
  double xdisp;
  short xdig;
  
  double ydisp;
  short ydig;

  double corr;
  short corg;

  float oimed;
  short oimg;

  float oiimed;
  short oiimg;
  
  double dispi;
  short dig;
  
  double dispii;
  short diig;
};

extern struct scatmes sm[];


/*--
  \end{code}

  \subsection{Heart Rate Variability Diagrams and Statistics}

  \begin{code}
  --*/


/*--
  \end{code}

  \section{Names}

  \PackagePurpose{
  This package generates names for the many files maintained
  by the Electrocardiognosis application. It also generates other
  names, such as for the tags in the report document.
  }

  \begin{code}
  --*/


/*--
  \end{code}

  \funparagraph{generate\_names\_from}
  \PurposeReturn{
  Given a file name, which is generic for a given recording, this function
  generates the names of various file used by the system.
  }

  \begin{code}
  --*/

void generate_names_from(char *tem);


/*--
  \end{code}

  \funparagraph{set\_eps\_counter}

  \begin{code}
  --*/

void set_eps_counter(short new_counter);


/*--
  \end{code}

  \funparagraph{inc\_eps\_counter}

  \begin{code}
  --*/

void inc_eps_counter();


/*--
  \end{code}

  \funparagraph{latex report file}

  \begin{code}
  --*/

short open_latex_file();
short close_latex_file();
char *the_latex_file_name();
FILE *the_latex_file();
FILE *the_latex_list();

/*--
  \end{code}

  \funparagraph{current eps file}

  \begin{code}
  --*/


short open_eps_file();
short close_eps_file();
FILE *the_eps_file();
char *the_eps_file_name();
short current_eps_count();

FILE *the_report_file();


/*--
  \end{code}

  \funparagraph{label of the last diagram}

  \begin{code}
  --*/


char *the_label_name();


/*--
  \end{code}

  \subsection{Miscellaneous functions}

  \begin{code}
  --*/

char *the_ecg_file_name();

/*--
  \end{code}

  \section{Events}

  \PackagePurpose{
  This package manipulates the events signaled in the data during
  the recording process. Similarly, ``soft events'' are added that
  identify moments established by the user after the recording was
  finished.
  }

  \begin{code}
  --*/



/*--
  \end{code}


  \endoffile

  --*/

