/* 

    Copyright (c) 1991--1996 Alexandru Dan Corlan MD (http://dan.corlan.net)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 * 
 * A program to produce summaries of the RR and ST in 
 * Electrocardiognosis Holter recordings (starting from .qst
 * or from .rir files).
 *
 * started: Oct 1996
 */

#include <stdio.h>
#include <rir.h>
#include <malloc.h>
#include <math.h> 

/* extern void *malloc(int); */
/* extern double sqrt(double); */

typedef struct perper {
  long nrr;           /* number of RR intervals */
  double sumfr;
  double sumfrsq;     /* sum of the squared frequencies, in 1/min*1/min */
  long maxfr;
  long minfr;

  double sumrr;       /* sum of the rr intervals */
  double sumrrsq;     /* sum of the sqared rr intervals */

  double countnn50;     /* count those with longer than 50ms between the RRs */
  double sumrmssd;      /* sum the r-MSSD values */

  long nst1;          /* number of ST1 samples which are usable */
  double sumst1;        /* in uV */
  double sumst1sq;    /* in uV**2 */
  long maxst1;        /* in uV */
  long minst1;        /* in uV */

  long nst2;          /* number of ST2 samples which are usable */
  double sumst2;        /* in uV */
  double sumst2sq;      /* in uV**2 */
  long maxst2;        /* in uV */
  long minst2;        /* in uV */
} PERPER;

/* Global variables */

   /* statistics */

PERPER *sta;         /* all of the interval statistics */
long    stasize;     /* their number */
long    stafill;     /* the current entry in sta */

PERPER fullst;       /* statistics for the full recording */

  /* input/default parameters */

short varhr =   10;   /* allowed variability of the RR (min**-1) */
short minhr =   30;   /* minimum heart rate which can be considered sinusal */
short maxhr =  190;   /* maximum heart rate which can be considered sinusal */
short varst =   55;   /* allowed variability of the ST (uV) */
short maxst = 2000;   /* maximum conceivable ST (2mV) */

long  every =   60;  /* at how many seconds to compute the local statistics */

long recbegin = -1; /* start from the beginning of the recording */
long recend =  -1;   /* end at the end of the recording */

  /* other global variables */

RIR *therir;

short init_perper(PERPER *p)

{
  p->nrr = 0;
  p->sumfr = 0;
  p->sumfrsq = 0;
  p->sumrr = 0;
  p->sumrrsq = 0;
  p->maxfr = 0;
  p->minfr = maxhr+1;

  p->countnn50 = 0;
  p->sumrmssd = 0;

  p->nst1 = 0;
  p->sumst1 = 0;
  p->sumst1sq = 0;
  p->maxst1 = -maxst;
  p->minst1 = maxst;


  p->nst2 = 0;
  p->sumst2 = 0;
  p->sumst2sq = 0;
  p->maxst2 = -maxst;
  p->minst2 = maxst;
}

int init_ecg_sum(char *fn)

{
  long len, i;

  /* .qst file assumed; the parameters of the computation 
   * must have been already set up.
   */

  therir = rir_open(fn, QST_TYPE);
  if(therir==NULL)
    return 1;  /* error */
  fprintf(stderr, "rir_open'ed successfully\n");
  len = rir_duration(therir)/10000; /* duration in seconds */
  fprintf(stderr, "rir_duration assessed: %ld\n", len);
  sta = (PERPER *)malloc(sizeof(PERPER) * (stasize = (len / every) + 1));
  if (sta == NULL)
    return 1; /* error---not enough memory */
  fprintf(stderr, "sta is malloced\n");
  for(i=0; i<stasize; i++)
    init_perper(sta+i);
  init_perper(&fullst);

  fprintf(stderr, "perpers done\n");

  for(i=0; i<10; i++)
    rir_read_next_record(therir);

  fprintf(stderr, "10 records read\n");

  return 0;
}

int use_hr(long place, long hr, long prev_hr)

     /* the value of the heart rate for that place is */
     /* added into the statistics, assumed already filtered */ 
     /* if a previous instantaneous hr was valid, it is */
     /* also added to the statistics (for rMSSD and pNN50) */

{
  long elt;

  elt = (place / 10000) / every;

  sta[elt].nrr ++;
  sta[elt].sumfr += hr;
  sta[elt].sumfrsq += hr * hr;
  sta[elt].sumrr += 60000 / hr;
  sta[elt].sumrrsq += (60000.0 / hr) * (60000.0 / hr);

  if (hr > sta[elt].maxfr)
    sta[elt].maxfr = hr;
  if (hr < sta[elt].minfr)
    sta[elt].minfr = hr;

  if (abs((60000 / hr) - (60000 / prev_hr)) >= 50)
    sta[elt].countnn50 ++;
  sta[elt].sumrmssd += ((60000 / hr) - (60000 / prev_hr)) *
                       ((60000 / hr) - (60000 / prev_hr));


  fullst.nrr ++;
  fullst.sumfr += hr;
  fullst.sumfrsq += hr * hr;
  fullst.sumrr += 60000 / hr;
  fullst.sumrrsq += (60000.0 / hr) * (60000.0 / hr);

  if (hr > fullst.maxfr)
    fullst.maxfr = hr;
  if (hr < fullst.minfr)
    fullst.minfr = hr;

  if (abs((60000 / hr) - (60000 / prev_hr)) >= 50)
    fullst.countnn50 ++;
  fullst.sumrmssd += ((60000 / hr) - (60000 / prev_hr)) *
                     ((60000 / hr) - (60000 / prev_hr));
  
  
}




int use_st1(long place, short st)

     /* the value of the st position for that place is */
     /* added into the statistics, assumed already filtered */


{
  long elt;

  elt = (place / 10000) / every;

  sta[elt].nst1 ++;
  sta[elt].sumst1 += st;
  sta[elt].sumst1sq += st * st;

  if (st > sta[elt].maxst1)
    sta[elt].maxst1 = st;
  if (st < sta[elt].minst1)
    sta[elt].minst1 = st;

  fullst.nst1 ++;
  fullst.sumst1 += st;
  fullst.sumst1sq += st * st;

  if (st > fullst.maxst1)
    fullst.maxst1 = st;
  if (st < fullst.minst1)
    fullst.minst1 = st;
}





int use_st2(long place, short st)

     /* the value of the st position for that place is */
     /* added into the statistics, assumed already filtered */


{
  long elt;

  elt = (place / 10000) / every;

  sta[elt].nst2 ++;
  sta[elt].sumst2 += st;
  sta[elt].sumst2sq += st * st;

  if (st > sta[elt].maxst2)
    sta[elt].maxst2 = st;
  if (st < sta[elt].minst2)
    sta[elt].minst2 = st;

  fullst.nst2 ++;
  fullst.sumst2 += st;
  fullst.sumst2sq += st * st;

  if (st > fullst.maxst2)
    fullst.maxst2 = st;
  if (st < fullst.minst2)
    fullst.minst2 = st;

}

void grab_all_data()

{
  long lllast_place, llast_place, last_place, crt_place;
  long  llast_hr, last_hr, crt_hr;
  short llast_st1, last_st1, crt_st1;
  short llast_st2, last_st2, crt_st2;
  long ctpl;

  while(!rir_read_next_record(therir)) {
    ctpl = rir_meas(therir, RIR_PLACE);
    if(ctpl == crt_place)
      continue;
    lllast_place = llast_place;
    llast_place = last_place;
    last_place = crt_place;
    crt_place = ctpl;

    llast_st1 = last_st1;
    last_st1 = crt_st1;
    crt_st1 = rir_meas(therir, RIR_ST_A);

    llast_st2 = last_st2;
    last_st2 = crt_st2;
    crt_st2 = rir_meas(therir, RIR_ST_B);
    
    llast_hr = last_hr;
    last_hr = crt_hr;
    crt_hr = 600000 / (crt_place - last_place);

    if ((abs(last_hr - llast_hr) < varhr) &&
	(abs(crt_hr - last_hr) < varhr) &&
	(last_hr < maxhr) && (last_hr > minhr))
      use_hr(crt_place, last_hr, llast_hr);
    if ((abs(last_st1 - llast_st1) < varst) &&
	(abs(crt_st1 - last_st1) < varst) &&
	(crt_st1 < maxst) && (crt_st1 > -maxst))
      use_st1(crt_place, last_st1);
    if ((abs(last_st2 - llast_st2) < varst) &&
	(abs(crt_st2 - last_st2) < varst) &&
	(crt_st2 < maxst) && (crt_st2 > -maxst))
      use_st2(crt_place, last_st2);
  }
}

char link[1024] = "";
char name[1024] = "";
char author[1024] = "";
char title[1024] = "";
char stitle[1024] = "";
char comment[4096] = "";

void rep_print_all()

{
  long i, tm;

  printf("\\begin{gnode}{ecgnosis}{%s}\n\n%s\n\n", name, stitle);
  printf("\\slot{recording} \\link{%s}\n", link);
  printf("\\slot{diaclass} ecgsum\n");
  printf("\\slot{params} ( every %d minhr %d maxhr %d varhr %d\n",
         every, minhr, maxhr, varhr);
  printf("                varst %d maxst %d\n", varst, maxst);
  printf("                variables (tm n hour hr-med hr-max hr-min\n");
  printf("                                st-a-med st-a-max st-a-min\n");
  printf("                                st-b-med st-b-max st-b-min\n");
  printf("                                sdnn pnn50 r-mssd )\n");
  printf("                begin %d end %d)\n\n", recbegin, recend);

  printf("\\slot{contents}\n( full-disclosure (\n");
 
  tm = 0;
  for(i=0; i<stasize; i++) {
    printf("  (%d %d ", tm, sta[i].nrr);
    printf("\"%02d:%02d:%02d\" ", tm / 3600, (tm / 60) % 60, tm % 60);
    if (sta[i].nrr > 3)
      printf("%4.0f %4d %4d ", sta[i].sumfr / sta[i].nrr, sta[i].maxfr,
	     sta[i].minfr);
    else
      printf("nil nil nil ");
    if (sta[i].nst1 > 3)
      printf(" %5.2f %5.2f %5.2f ", 
             (double)sta[i].sumst1 / sta[i].nst1 / 1000.0,
	     (double)sta[i].maxst1 / 1000.0, (double)sta[i].minst1 / 1000.0);
    else
      printf(" nil nil nil ");
    if (sta[i].nst2 > 3)
      printf(" %5.2f %5.2f %5.2f ", (double)sta[i].sumst2 / sta[i].nst2 / 1000.0,
	     (double)sta[i].maxst2 / 1000.0, (double)sta[i].minst2 / 1000.0);
    else
      printf(" nil nil nil ");
    if (sta[i].nrr > 3)
      printf("%5.1f %5.1f %5.1f )\n",
	     sqrt((double)(sta[i].nrr * sta[i].sumrrsq
					      - sta[i].sumrr * sta[i].sumrr)/
		  (double)(sta[i].nrr * (sta[i].nrr-1))),
	     (double)(sta[i].countnn50 * 100.0) / sta[i].nrr,
	     sqrt(sta[i].sumrmssd / (long)sta[i].nrr));
    else
      printf(" nil nil nil )\n");
    tm += every;
  }
  printf(" )\n  global \n");

  if (fullst.nrr > 3)
      printf("( nil %d %4.0f %d %d ", fullst.nrr,
             fullst.sumfr / fullst.nrr, fullst.maxfr,
	     fullst.minfr);
    else
      printf(" nil %d nil nil nil ", fullst.nrr);
    if (fullst.nst1 > 3)
      printf(" %5.2f %5.2f %5.2f ", (double)fullst.sumst1 / fullst.nst1 / 1000.0,
	     (double)fullst.maxst1 / 1000.0, (double)fullst.minst1 / 1000.0);
    else
      printf(" nil nil nil ");
    if (fullst.nst2 > 3)
      printf(" %5.2f %5.2f %5.2f ", (double)fullst.sumst2 / fullst.nst2 / 1000.0,
	     (double)fullst.maxst2 / 1000.0, (double)fullst.minst2 / 1000.0);
    else
      printf(" nil nil nil ");
    if (fullst.nrr > 3)
      printf("%5.1f %5.1f %5.1f ) )\n\n",
	     sqrt(((double)fullst.nrr * fullst.sumrrsq
					      - fullst.sumrr * fullst.sumrr)/
		  (double)((double)fullst.nrr * (double)(fullst.nrr-1))),
	     (double)(fullst.countnn50 * 100.0) / fullst.nrr,
	     sqrt(fullst.sumrmssd / fullst.nrr));
    else
      printf(" nil nil nil ) )\n\n");

  printf("\\end{gnode}\n\n");
}

void ascii_print_all()

{
  long tm, i;

  tm = 0;

  printf("--------------------+------------------+------------------+-----------------\n");
  printf("HH:MM:SS%4s%4s%4s| %5s %5s %5s| %5s %5s %5s|%5s %5s %5s\n",
	 "HR", "MAX", "MIN", "ST1", "MAX", "MIN", "ST2", "MAX", "MIN",
	 "SDNN", "pNN50", "rMSSD");
  for(i=0; i<stasize; i++) {
    if(!(i%5))
        printf("--------------------+------------------+------------------+-----------------\n");
    printf("%02d:%02d:%02d", tm / 3600, (tm / 60) % 60, tm % 60);
    if (sta[i].nrr > 3)
      printf("%4.0f%4d%4d|", sta[i].sumfr / sta[i].nrr, sta[i].maxfr,
	     sta[i].minfr);
    else
      printf(" --- --- ---|");
    if (sta[i].nst1 > 3)
      printf(" %5.2f %5.2f %5.2f|", (double)sta[i].sumst1 / sta[i].nst1 / 1000.0,
	     (double)sta[i].maxst1 / 1000.0, (double)sta[i].minst1 / 1000.0);
    else
      printf("   ---   ---   ---|");
    if (sta[i].nst2 > 3)
      printf(" %5.2f %5.2f %5.2f|", (double)sta[i].sumst2 / sta[i].nst2 / 1000.0,
	     (double)sta[i].maxst2 / 1000.0, (double)sta[i].minst2 / 1000.0);
    else
      printf("   ---   ---   ---|");
    if (sta[i].nrr > 3)
      printf("%5.1f %5.1f %5.1f\n",
	     sqrt((double)(sta[i].nrr * sta[i].sumrrsq
					      - sta[i].sumrr * sta[i].sumrr)/
		  ((double)(sta[i].nrr) * (double)(sta[i].nrr-1))),
	     (double)(sta[i].countnn50 * 100.0) / sta[i].nrr,
	     sqrt(sta[i].sumrmssd / (long)sta[i].nrr));
    else
      printf("  ---   ---   ---\n");
    tm += every;
  }
  printf("--------------------+------------------+------------------+-----------------\n\n\n");

  printf("For the full period:\n\n");
  printf("--------------------+------------------+------------------+-----------------\n");
  printf("HH:MM:SS%4s%4s%4s| %5s %5s %5s| %5s %5s %5s|%5s %5s %5s\n",
	 "HR", "MAX", "MIN", "ST1", "MAX", "MIN", "ST2", "MAX", "MIN",
	 "SDNN", "pNN50", "rMSSD");
  printf("--------------------+------------------+------------------+-----------------\n");
    printf("        ");
    if (fullst.nrr > 3)
      printf("%4.0f%4d%4d|", fullst.sumfr / fullst.nrr, fullst.maxfr,
	     fullst.minfr);
    else
      printf(" --- --- ---|");
    if (fullst.nst1 > 3)
      printf(" %5.2f %5.2f %5.2f|", (double)fullst.sumst1 / fullst.nst1 / 1000.0,
	     (double)fullst.maxst1 / 1000.0, (double)fullst.minst1 / 1000.0);
    else
      printf("   ---   ---   ---|");
    if (fullst.nst2 > 3)
      printf(" %5.2f %5.2f %5.2f|", (double)fullst.sumst2 / fullst.nst2 / 1000.0,
	     (double)fullst.maxst2 / 1000.0, (double)fullst.minst2 / 1000.0);
    else
      printf("   ---   ---   ---|");
    if (fullst.nrr > 3)
      printf("%5.1f %5.1f %5.1f\n",
	     sqrt((double)(fullst.nrr * fullst.sumrrsq
					      - fullst.sumrr * fullst.sumrr)/
		  ((double)(fullst.nrr) * (double)(fullst.nrr-1))),
	     (double)(fullst.countnn50 * 100.0) / fullst.nrr,
	     sqrt(fullst.sumrmssd / (long)fullst.nrr));
    else
      printf("  ---   ---    ---\n");
  printf("--------------------+------------------+------------------+-----------------\n");
}

main(short argc, char **argv)

{
  short outype = 2; /* 1=rep, 2=stdout */

  while((argv[1][0]=='-')) {
    if (!strcmp(argv[1], "--every")) {
      every = atoi(argv[2]);
      argv ++; argv ++;
      argc --; argc --;
      continue;
    }
    if (!strcmp(argv[1], "--varhr")) {
      varhr = atoi(argv[2]);
      argv ++; argv ++;
      argc --; argc --;
      continue;
    }
    if (!strcmp(argv[1], "--rep")) {
      outype = 1;
      argv ++;
      argc --; 
    }
    if (!strcmp(argv[1], "--ascii")) {
      outype = 2;
      argv ++; 
      argc --;
    }
    if (!strcmp(argv[1], "--varst")) {
      varst = atoi(argv[2]);
      argv ++; argv ++;
      argc --; argc --;
      continue;
    }
    if (!strcmp(argv[1], "--name")) {
      strncmp(name, argv[2], 1023);
      argv ++; argv ++;
      argc --; argc --;
      continue;
    }
    if (!strcmp(argv[1], "--title")) {
      strncmp(title, argv[2], 1023);
      argv ++; argv ++;
      argc --; argc --;
      continue;
    }
    if (!strcmp(argv[1], "--stitle")) {
      strncmp(stitle, argv[2], 1023);
      argv ++; argv ++;
      argc --; argc --;
      continue;
    }
    if (!strcmp(argv[1], "--link")) {
      strncmp(link, argv[2], 1023);
      argv ++; argv ++;
      argc --; argc --;
      continue;
    }
    if (!strcmp(argv[1], "--comment")) {
      strncmp(comment, argv[2], 4095);
      argv ++; argv ++;
      argc --; argc --;
      continue;
    }
    if (!strcmp(argv[1], "--author")) {
      strncmp(author, argv[2], 1023);
      argv ++; argv ++;
      argc --; argc --;
      continue;
    }
    /*    fprintf(stderr, "Usage: ecgsum [--every nsec] [--varhr min**-1] [--varst uV] filename.qst\n");
    exit(-1);*/
  }

  fprintf(stderr, "Electrocardiognosis recording: %s\n", argv[1]);

  if(!init_ecg_sum(argv[1])) {
    fprintf(stderr, "Size=%d, stasize=%d ...", rir_duration(therir), stasize);
    fprintf(stderr, " nentries=%d ...", therir->n_entries);
    grab_all_data();
    fprintf(stderr, " data grabbed\n\n");
    switch(outype) {
    case 1:
      rep_print_all(); /* to stdout of course */
      break;
    case 2:
      ascii_print_all();
      break;
    default:
      break;
    }
  }
  else
    fprintf(stderr, "Unusable file: %s\n", argv[1]);
  
}
