
/*--

  \FileTitle{ 
  {\tt egdiagtd.c} --- Electrocardiognosis time diagrams implementation
  }
  \CopyrightNotice{

    Copyright (c) 1991--1996 Alexandru Dan Corlan MD (http://dan.corlan.net)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  }

  \subsection{Time diagrams implementation}

  \PackagePurpose{
  Implements the functions that computed, interpret, typeset,
  input and out time diagrams in a variety of formats.
  }

  \begin{code}
  --*/


#include <stdio.h>

#include "gliks.h"
#include "ecgnosis.h"

/*--
  \end{code}

  \subsubsection{Time diagrams' registry}

  \begin{code}
  --*/


TIMEDIAG tds[MAX_NOF_TDS];
short nof_tds = 0;


/*--
  \end{code}

  \funparagraph{Constructor and selector functions}

  \begin{code}
  --*/

char *td_name(short k)

{
  return tds[k].tdname;
}

/*--
  \end{code}

  \funparagraph{Initialisation}

  \begin{code}
  --*/


void init_tds()

{

  strcpy(tds[F_DIAGRAM].tdname, "HRate");
  tds[F_DIAGRAM].u_base = 0;
  tds[F_DIAGRAM].u_incr = 20;
  tds[F_DIAGRAM].d_base = 0;
  tds[F_DIAGRAM].d_incr = 10;

  strcpy(tds[RR_DIAGRAM].tdname, "RR");
  tds[RR_DIAGRAM].u_base = 0;
  tds[RR_DIAGRAM].u_incr = 20;
  tds[RR_DIAGRAM].d_base = 0;
  tds[RR_DIAGRAM].d_incr = 10;

  strcpy(tds[ST1_DIAGRAM].tdname, "ST1");
  tds[ST1_DIAGRAM].u_base = -50;
  tds[ST1_DIAGRAM].u_incr = 10;
  tds[ST1_DIAGRAM].d_base = -50;
  tds[ST1_DIAGRAM].d_incr = 10;


  strcpy(tds[ST2_DIAGRAM].tdname, "ST2");
  tds[ST2_DIAGRAM].u_base = -50;
  tds[ST2_DIAGRAM].u_incr = 10;
  tds[ST2_DIAGRAM].d_base = -50;
  tds[ST2_DIAGRAM].d_incr = 10;

  strcpy(tds[ST1MED_DIAGRAM].tdname, "ST1MED");
  tds[ST1MED_DIAGRAM].u_base = -50;
  tds[ST1MED_DIAGRAM].u_incr = 10;
  tds[ST1MED_DIAGRAM].d_base = -50;
  tds[ST1MED_DIAGRAM].d_incr = 10;


  strcpy(tds[ST2MED_DIAGRAM].tdname, "ST2MED");
  tds[ST2MED_DIAGRAM].u_base = -50;
  tds[ST2MED_DIAGRAM].u_incr = 10;
  tds[ST2MED_DIAGRAM].d_base = -50;
  tds[ST2MED_DIAGRAM].d_incr = 10;

  nof_tds = ST2_DIAGRAM;
}


/*--
  \end{code}

  \subsubsection{Time diagram styles}

  \begin{code}
  --*/



/*--
  \end{code}

  \subsubsection{Time diagram computation and output}

  \begin{code}
  --*/


/*--
  \end{code}

  \dataparagraph{Defining variables}

  \begin{code}
  --*/

#define MAX_SIMULTANEOUS_TDS       2
#define MAX_TD_SIZE            10240

long  td_computation_step = 2500;  /* ccs */ 			/* [VAR] */ 
short td_filter_width = 1;         /* beats or steps */      	/* [VAR] */ 
short td_beat_filtering_f = 0;     /* steps, not beats */ 	/* [VAR] */ 
short *td_filter_vector = NULL; 				/* [VAR] */ 
short td_types[MAX_SIMULTANEOUS_TDS] = {F_DIAGRAM, 0};		/* [VAR] */
short ts_styles[MAX_SIMULTANEOUS_TDS] = {1, 2}; 		/* [VAR] */ 


/*--
  \end{code}

  \dataparagraph{Time diagrams' contents}  % a name for the data structure

  \begin{code}
  --*/

long td_med[MAX_SIMULTANEOUS_TDS][MAX_TD_SIZE]; 		/* [VAR] */ 
long td_max[MAX_SIMULTANEOUS_TDS][MAX_TD_SIZE]; 		/* [VAR] */
long td_min[MAX_SIMULTANEOUS_TDS][MAX_TD_SIZE]; 		/* [VAR] */


/*--
  \end{code}

  \dataparagraph{The time histogram representation}

  \begin{code}
  --*/

#define MAX_HIST_VALS  50 					/* [DEF] */ 
#define MAX_HIST_SIZE  2048 					/* [DEF] */ 
#define TD_HIST_LEVELS 6 					/* [DEF] */ 

char td_hist[MAX_HIST_VALS][MAX_HIST_SIZE]; 			/* [VAR] */ 

short nof_hist_vals = 40; 					/* [VAR] */ 

/*--
  \end{code}

  \dataparagraph{Typographic parameters}

  \begin{code}
  --*/

short td_min_to_display[MAX_SIMULTANEOUS_TDS] = {0, 0};		/* [VAR] */ 
short td_max_to_display[MAX_SIMULTANEOUS_TDS] = {200, 200};	/* [VAR] */ 
short td_display_mode[MAX_SIMULTANEOUS_TDS] = {1, 1}; 		/* [VAR] */ 

char *td_scale_label_format[MAX_SIMULTANEOUS_TDS] =
                     {"%4.1f", "%4.1f"}; 			/* [VAR] */ 
float td_scale_factor[MAX_SIMULTANEOUS_TDS] =
                    { 1.0, 1.0 }; 				/* [VAR] */ 
float td_min_scale_value[MAX_SIMULTANEOUS_TDS] =
                     { 0.0, 0.0 }; 	           		/* [VAR] */ 

short td_width = 432;   /* bp */ 				/* [VAR] */
short td_height = 108;  /* bp */ 				/* [VAR] */ 
short td_val_divisions = 10;    				/* [VAR] */ 
short td_val_labels = 10; 					/* [VAR] */ 
long  td_time_divisions = 600000; /* ccs */ 			/* [VAR] */ 
long  td_time_labels = 600000; /* ccs */ 			/* [VAR] */ 
short td_left_scale_width = 32; /* bp */ 			/* [VAR] */ 
short td_right_scale_width = 0; /* bp */ 			/* [VAR] */ 

short td_top_height = 16; /* bp */ 				/* [VAR] */ 
short td_bottom_height = 16; /* bp */ 				/* [VAR] */ 

float td_graphic_step = 1.0; /* bp */ 				/* [VAR] */ 

short td_show_min_f[MAX_SIMULTANEOUS_TDS] = { 0, 0}; 		/* [VAR] */ 
short td_show_med_f[MAX_SIMULTANEOUS_TDS] = { 1, 1}; 		/* [VAR] */ 
short td_show_max_f[MAX_SIMULTANEOUS_TDS] = { 0, 0}; 		/* [VAR] */ 
short td_show_hist_f[MAX_SIMULTANEOUS_TDS] = { 0, 0}; 		/* [VAR] */
short td_show_pm_sigma_f[MAX_SIMULTANEOUS_TDS] = { 0, 0};
short td_show_sigma_f[MAX_SIMULTANEOUS_TDS] = { 0, 0};

/*--
  \end{code}

  \dataparagraph{Labels}

  \begin{code}
  --*/

short left_scale_title_f = 1; 					/* [VAR] */
short left_scale_title_x = 0; 					/* [VAR] */ 
char left_scale_title[64] = ""; 				/* [VAR] */ 

short left_scale_um_f = 1; 					/* [VAR] */ 
short left_scale_um_x = 0; 					/* [VAR] */ 
char left_scale_um[16] = ""; 					/* [VAR] */ 

short right_scale_title_f = 1; 					/* [VAR] */ 
short right_scale_title_x = 0; 					/* [VAR] */ 
char right_scale_title[64] = ""; 				/* [VAR] */ 

short right_scale_um_f = 1; 					/* [VAR] */ 
short right_scale_um_x = 0; 					/* [VAR] */ 
char right_scale_um[16] = ""; 					/* [VAR] */ 

char time_label[16] = ""; 					/* [VAR] */ 

short td_file_name_label_f = 1; 				/* [VAR] */ 
short td_file_name_label_x = 200; 				/* [VAR] */ 

short td_diag_no_f = 1; 					/* [VAR] */
short td_technology = 0;  /* 0=bw, 1=grays, 2=color */ 		/* [VAR] */ 

char td_title[256] = "";					/* [VAR] */ 
char td_comment[1024] = ""; 					/* [VAR] */ 

/*--
  \end{code}

  \dataparagraph{Typographic details}  % a name for the data structure

  \begin{code}
  --*/

float td_frame_width = 0.0;  /* bp */ 				/* [VAR] */ 
char *td_frame_color = "Cyan"; 					/* [VAR] */ 
float td_frame_gray = 0.0; 					/* [VAR] */ 

float td_grid_width = 0.0; /* bp */  				/* [VAR] */ 
float td_grid_gray = 0.0;  /* bp */ 				/* [VAR] */ 

char td_scales_font[64] = "Helvetica"; 				/* [VAR] */ 
char td_scales_font_size = 8; 					/* [VAR] */ 
char td_label_font[64] = "Helvetica"; 				/* [VAR] */ 
char td_label_font_size = 10; 					/* [VAR] */ 

float td_med_width=1.0; 					/* [VAR] */ 
float td_lim_width=0.4; 					/* [VAR] */ 

float td_signal_gray[MAX_SIMULTANEOUS_TDS] = { 0.0, 0.5 }; 	/* [VAR] */ 



/*--
  \end{code}

  \dataparagraph{Recommended time divisions}

  \begin{code}
  --*/

static long rec_time_divisions[30] = { 				/* [VAR] */ 
    200000,     /* 20 s  */
    600000,     /*  1 min */
   1800000,     /*  3 min */
   3000000,     /*  5 min */
   6000000,     /* 10 min */
   9000000,     /* 15 min */
  18000000,     /* 30 min */
  36000000,     /*  1 h   */
  72000000,     /*  2 h   */
 144000000,     /*  4 h   */
 432000000      /* 12 h --- never to be reached except in case of error */
  };

short rec_time_divisions_size = 10; 				/* [VAR] */ 

/*--
  \end{code}

  \funparagraph{Parameter setting functions}

  \begin{code}
  --*/

void set_td_resolution(long res)  /* in ccs */
 								/* [FUN] */ 
{
  td_computation_step = res;
}

void set_td_graphic_step(double t)  /* in bp */ 			/* [FUN] */ 

{
  td_graphic_step = t;
}

set_step_td_filtering() 					/* [FUN] */ 

{
  td_beat_filtering_f = 0;
}

set_td_width(short n) 						/* [FUN] */ 

{
  td_width = n;
}

set_td_filter_width(short n) 					/* [FUN] */ 

{
  td_filter_width = n;
}

set_td_title(char *t) 						/* [FUN] */ 

{
  strncpy(td_title, t, 256);
  td_title[255] = 0;
}

set_td_comment(char *c) 					/* [FUN] */ 

{
  strncpy(td_comment, c, 1024);
  td_comment[1023] = 0;
}

short get_td_type(short k) 					/* [FUN] */ 

{
  return td_types[k];
}

short set_td_type(short k, short t)

{
  td_types[k] = t;
}


set_td_lines_to_show(short k, short histf, 			/* [FUN] */ 
		     short minf, short medf, short maxf)        /* [FUN] */ 

{
  td_show_hist_f[k] = histf;
  td_show_min_f[k] = minf;
  td_show_med_f[k] = medf;
  td_show_max_f[k] = maxf;
}

set_td_use_range(short k, short minv, short maxv)

{
  td_max_to_display[k] = maxv;
  td_min_to_display[k] = minv;
}

short set_td_scale_factor(short k, float fact)

{
  fprintf(ecgnosislog, "Seting to %10.2f\n", fact);
  td_scale_factor[k] = fact;
  fprintf(ecgnosislog, "Set %d to %10.2f\n", k, td_scale_factor[k]);
}

void set_td_min_scale_value(short k, float val)

{
  td_min_scale_value[k] = val;
}

void set_td_scale_label_format(short k, char *ff)

{
  td_scale_label_format[k] = ff;
}

/*--
  \end{code}

  \subparagraph{Seting time divisions}

  The vertical time divisions are select so that they are as
  close as possible, but no closer than \variabletag{min_td_time_div}
  points.

  \begin{code}
  --*/

short min_td_time_div = 72;  /* 1 in */ 			/* [FUN] */ 

short set_time_divisions(long l) 				/* [FUN] */ 

{
  short sugested;
  
  if(l!=0) {
    td_time_divisions = l;
    return 1;
  }
  for(sugested = 0; sugested < rec_time_divisions_size; sugested++) {
    if(td_graphic_step * rec_time_divisions[sugested] / td_computation_step
       > min_td_time_div)
      break;
  }
  td_time_divisions = rec_time_divisions[sugested];
  return 1;
}

/*--
  \end{code}

  \funparagraph{eval\_td}  % The name of the function(s)

  \begin{code}
  --*/

#define EVAL_TD_BUFFER_SIZE 1024 				/* [DEF] */ 

static long etdb[EVAL_TD_BUFFER_SIZE]; 				/* [VAR] */ 
static long etdpos[EVAL_TD_BUFFER_SIZE]; 			/* [VAR] */ 

static short etdb_fill; 		 			/* [VAR] */ 


static long  themin, themax;  /* values returned by eval_td */  /* [VAR] */ 

static short hist_vals[MAX_HIST_VALS];   			/* [VAR] */ 
static short hist_delta;  					/* [VAR] */ 

short compute_hist_delta() 					/* [FUN] */ 

{
  short i, maxh;

  maxh = 0;
  for(i=0; i<nof_hist_vals; i++)
    if(hist_vals[i] > maxh)
      maxh = hist_vals[i];
  hist_delta = (maxh / 6) + 1;
}

long eval_td(PERFILE *p, long tdtype, long m0, 			/* [FUN] */ 
	     long hist_min, long hist_inc) 			/* [FUN] */ 

{
  short param;
  short nofp;
  long r, v;
  long i;
  short ign_count;

/*
  fprintf(ecgnosislog, "Called eval_td(, %ld, %ld)\n", tdtype, m0);
  fflush(ecgnosislog); 
  */

  for(i=0; i<nof_hist_vals; i++)
    hist_vals[i] = 0;

  param = CPQRSPOS;
  etdb_fill = 0;
  nofp = 2;
  
  switch(tdtype) {
  case F_DIAGRAM:
  case RR_DIAGRAM:
    param = CPQRSPOS;
    break;
  case ST1_DIAGRAM:
  case ST1MED_DIAGRAM:
    param = CPST1;
    break;
  case ST2_DIAGRAM:
  case ST2MED_DIAGRAM:
    param = CPST2;
    break;
  default:
    break;
  }

  if(td_beat_filtering_f)
    nofp = td_filter_width;
  else {
    nofp = complexes_between(p, m0,
			     m0 + td_filter_width * td_computation_step);
  }
/*
  fprintf(ecgnosislog, "nofp = %d ", nofp);
  fflush(ecgnosislog); 
  */
  if(nofp<1)
    nofp = 1;

/*  fprintf(ecgnosislog, " B1 ");
  fflush(ecgnosislog);*/
  
  cpars_at(p, etdpos, CPQRSPOS, m0, nofp+1);

/*  fprintf(ecgnosislog, " B2 ");
  fflush(ecgnosislog);*/
  
  cpars_at(p, etdb, param, m0, nofp+1);

/*  fprintf(ecgnosislog, " A2\n");
  fflush(ecgnosislog);*/
  
  r = 0;
  themax = -99999999;
  themin = 99999999;

  ign_count = 0;
  for(i=0; i<nofp; i++) {
    /* filter vector is ignored for the moment */
    /* individual values for the expression are computed */

/*    fprintf(ecgnosislog, "etdpos [%d] = %ld .. %ld\n", i, etdpos[i],
	    etdpos[i+1]);
    fflush(ecgnosislog);*/

    switch(tdtype) {
    case F_DIAGRAM:
      if(etdpos[i+1]<=etdpos[i]) {
	fprintf(ecgnosislog, "\nINTERNAL ERROR in eval_td():\n");
	fprintf(ecgnosislog, "    etdpos[(i=%d)+1]=%ld, etdpos[(i=%d)]=%ldn",
		i, etdpos[i+1], i, etdpos[i]);
	fprintf(ecgnosislog, "    recovering by ignoring the data.\n\n");
	ign_count ++;
	v = 0;
	break;
      }
      v = 600000 / (etdpos[i+1] - etdpos[i]);
      break;
    case RR_DIAGRAM:
      v = etdpos[i+1] - etdpos[i];
      break;
    case ST1_DIAGRAM:
    case ST2_DIAGRAM:
      if(nofp>2) {
	if(i==0 || i > nofp-2) {
	  ign_count ++;
	  v = 0;
	}
	else {
	  if(abs(etdb[i-1] - etdb[i])>2 || abs(etdb[i+1] - etdb[i])>2) {
	    ign_count ++;
	    v = 0;
	  }
	  else v = etdb[i] * 50;
	}
      }
      else
	v = etdb[i] * 50;  /* the value in microvolts */
      break;
    default:
      v = etdb[i];
      break;
    }
    if(v>themax)
      themax = v;
    if(v<themin)
      themin = v;
    r += v;
    v = (v - hist_min) / hist_inc;  /* the class of the value */
    if(v>=0 && v<MAX_HIST_VALS)
      hist_vals[v] ++;
    /* The temporal histogram is built here */
  }
  if(ign_count<nofp)
    r /= nofp - ign_count;
  else
    r /= nofp;
  return r;  
}


/*--
  \end{code}

  \funparagraph{compute\_time\_diagram}

  \begin{code}
  --*/

short compute_time_diagram(PERFILE *p, short n, long m0)

{
  float tdur;
  long td_in_ccs;
  long i, m, j;

  if(td_types[n]==0)
    return 0;   /* no result computed */
  /* asses duration of the time diagram */
  tdur = td_width / td_graphic_step  /* number of steps */
    * td_computation_step;
  td_in_ccs = tdur;

/*
  fprintf(ecgnosislog, "Duration = %ld\n", td_in_ccs);
  fflush(ecgnosislog); */
  
  for(m=m0, i=0; m < m0 + td_in_ccs; m+=td_computation_step, i++) {
    td_med[n][i] = eval_td(p, td_types[n], m,
			   td_min_to_display[n],
			   (td_max_to_display[n] - td_min_to_display[n]) /
			     nof_hist_vals);
    td_max[n][i] = themax;
    td_min[n][i] = themin;
    compute_hist_delta();
    if(td_show_hist_f[n] && i<MAX_HIST_SIZE)
      for(j=0; j<nof_hist_vals; j++)
        td_hist[j][i] = hist_vals[j] / hist_delta;
  }
  return i;
}


/*--
  \end{code}

  \funparagraph{put\_td\_to\_eps}

  \begin{code}
  --*/

short put_td_to_eps(FILE *f, PERFILE *p, long m0)

{
  float xl, yl;
  short i, j, k;
  float y, yfact;
  float t, t0, tt;
  long nov, m, absm;
  float y0;
  char sclabelbuf[16];

  /*--
    \end{code}

    \subparagraph{The bounding box}

    \begin{code}
    --*/
  
/*
  fprintf(ecgnosislog, "tdgraphicstep=%6.2f", td_graphic_step);
  fflush(ecgnosislog); */

  xl = td_width + td_left_scale_width + td_right_scale_width +
    4 * td_frame_width;
  yl = td_height + 4 * td_frame_width +
    td_top_height + td_bottom_height; /* place for two text lines */
  
  fprintf(f, "%%%%BoundingBox: 0 0 %6.2f %6.2f\n", xl, yl);

  fprintf(f, "/TdScaleFont /%s findfont %d scalefont def\n",
	  td_scales_font, td_scales_font_size);
  fprintf(f, "/TdLabelFont /%s findfont %d scalefont def\n",
	  td_label_font, td_label_font_size);

  /*--
    \end{code}

    \subparagraph{The grid of the time diagram}

    \begin{code}
    --*/

  fprintf(f, "%6.2f setgray %6.2f setlinewidth\n",
	  td_grid_gray, td_grid_width);

  /* the horizontal lines */

  fprintf(f, "[0.5 1.5] 0 setdash\n");
  for(i=1; i<td_val_divisions; i++)
    fprintf(f, "newpath %7.2f %7.2f moveto %7.2f 0 rlineto stroke\n",
	    2 * td_frame_width + td_left_scale_width,
	    2 * td_frame_width + td_bottom_height +
	      i * ((float)td_height) / td_val_divisions,
	    (float)td_width);
  

  /* the left scale labels */


  if(td_left_scale_width>0) {
    fprintf(f, "TdScaleFont setfont\n");
    for(i=1; i<td_val_divisions; i++) {
      fprintf(ecgnosislog, "scale_f=%10.2f max=%d min=%d\n",
	      td_scale_factor[0], td_max_to_display[0], td_min_to_display[0]);
      fflush(ecgnosislog);
      sprintf(sclabelbuf, td_scale_label_format[0], 
	      td_min_scale_value[0] +
	      td_scale_factor[0] * i *
	      (float)(td_max_to_display[0] - td_min_to_display[0]) /
	      td_val_divisions);
      fprintf(f, "%5.2f %6.2f moveto (%s) show\n", 3.0,
	      2 * td_frame_width + td_bottom_height +
	      i * ((float)td_height) / td_val_divisions,
	      sclabelbuf);
    }
  }

  /* The time divisions */


  t0 = td_time_divisions - (m0 % td_time_divisions);
  
  t0 = t0 / td_computation_step * td_graphic_step;
  tt = (float)td_time_divisions / (float)td_computation_step *
    (float)td_graphic_step;

/*
  fprintf(ecgnosislog, "Printing: %6.2f %6.2f ttd=%ld tcs=%ld tgs=%6.2f\n",
	  t0, tt, td_time_divisions, td_computation_step, td_graphic_step);
  fflush(ecgnosislog); */

  for(t = t0, m = (m0 / td_time_divisions + 1) * td_time_divisions;
      t<td_width; t+=tt, m+=td_time_divisions) {
    fprintf(f, "newpath %6.2f %6.2f moveto 0 %d rlineto stroke\n",
	    t + td_left_scale_width,
	    2 * td_frame_width + td_bottom_height, td_height);
    absm = rel2abs_time(m);
    sprintf(time_label, "%02ld:%02ld:%02ld", absm / 36000000,
	    (absm % 36000000) / 600000, (absm % 600000) / 200);
    fprintf(f, "%6.2f %6.2f moveto (%s) show\n",
	    t + td_left_scale_width - 20.0, 2.0, time_label);
  }

  /*--
    \end{code}

    \subparagraph{The signal to be displayed}

    \begin{code}
    --*/

  fprintf(ecgnosislog, "Before computing time diagram\n");
  fflush(ecgnosislog);

/*
  fprintf(f, "newpath %6.2f %6.2f moveto %d 0 rlineto 0 %d rlineto\n",
	  td_left_scale_width + 2 * td_frame_width,
	  2 * td_frame_width + td_bottom_height,
	  td_width, td_height);
  fprintf(f, "%d 0 rlineto 0 %d rlineto closepath clip\n\n",
	  -td_width, -td_height);
  */

  for(i=0; i<MAX_SIMULTANEOUS_TDS; i++) {
    nov = compute_time_diagram(p, i, m0);
    fprintf(ecgnosislog, "nov=%ld, m0=%ld\n", nov, m0);
    fflush(ecgnosislog);
    yfact = td_height * 1.0 / (td_max_to_display[i] - td_min_to_display[i]);
    if(td_show_hist_f[i]) {
      fprintf(f, "0.0 setgray [] 0 setdash\n");
      fprintf(f, "/blich %6.2f def\n",
	      (float)td_height / (float)nof_hist_vals);
      
      fprintf(f, "/blicw %6.2f def\n", td_graphic_step);
      fprintf(f, "/blic { newpath moveto blicw 0 rlineto 0 blich rlineto\n");
      fprintf(f, "        blicw neg 0 rlineto 0 blich neg rlineto\n");
      fprintf(f, "        closepath\n");
      fprintf(f, "        %d div 1.0 exch sub setgray gsave fill grestore\n", TD_HIST_LEVELS);
      fprintf(f, "        0 setlinewidth 0.0 setgray stroke\n");
      fprintf(f, "        } def\n\n");
      
      for(j=0; j<nov; j++) {
	for(k=0; k<nof_hist_vals; k++) {
	  if(td_hist[k][j]>0)
	    fprintf(f, "%d %6.2f %6.2f blic\n",
		    td_hist[k][j],
		    td_graphic_step * j + td_left_scale_width +
		      2 * td_frame_width,
		    td_frame_width * 2 + td_bottom_height +
		      k * (float)td_height / (float)nof_hist_vals);
	  
	}
      }
    }
    if(td_show_med_f[i]) {
      fprintf(f, "[] 0 setdash %5.2f setlinewidth %5.2f setgray\n",
	      td_med_width, td_signal_gray[i]);
      for(j=0; j<nov; j++) {
	if(td_med[i][j]<20) {
	  fprintf(ecgnosislog, "tdmed %d = %ld\n", j, td_med[i][j]);
	  fflush(ecgnosislog);
	}
	y = yfact * (td_med[i][j] - td_min_to_display[i]);
	if(j==0)
	  fprintf(f, "newpath %6.2f %6.2f moveto\n",
		  2 * td_frame_width + td_left_scale_width, y
		  + td_frame_width*2 + td_bottom_height);
	else
	  fprintf(f, "%6.2f %6.2f rlineto\n", td_graphic_step,
		  y - y0);
	y0 = y;
      }
      fprintf(f, "stroke\n");
    }
    if(td_show_min_f[i]) {
      fprintf(f, "[] 0 setdash %5.2f setlinewidth %5.2f setgray\n",
	      td_lim_width, td_signal_gray[i]);
      for(j=0; j<nov; j++) {
	if(td_med[i][j]<20) {
	  fprintf(ecgnosislog, "tdmin %d = %ld\n", j, td_med[i][j]);
	  fflush(ecgnosislog);
	}
	y = yfact * (td_min[i][j] - td_min_to_display[i]);
	if(j==0)
	  fprintf(f, "newpath %6.2f %6.2f moveto\n",
		  2 * td_frame_width + td_left_scale_width, y
		  + td_frame_width*2 + td_bottom_height);
	else
	  fprintf(f, "%6.2f %6.2f rlineto\n", td_graphic_step,
		  y - y0);
	y0 = y;
      }
      fprintf(f, "stroke\n");
    }
    if(td_show_max_f[i]) {
      fprintf(f, "[] 0 setdash %5.2f setlinewidth %5.2f setgray\n",
	      td_lim_width, td_signal_gray[i]);
      for(j=0; j<nov; j++) {
	if(td_max[i][j]<20) {
	  fprintf(ecgnosislog, "tdmax %d = %ld\n", j, td_med[i][j]);
	  fflush(ecgnosislog);
	}
	y = yfact * (td_max[i][j] - td_min_to_display[i]);
	if(j==0)
	  fprintf(f, "newpath %6.2f %6.2f moveto\n",
		  2 * td_frame_width + td_left_scale_width, y
		  + td_frame_width*2 + td_bottom_height);
	else
	  fprintf(f, "%6.2f %6.2f rlineto\n", td_graphic_step,
		  y - y0);
	y0 = y;
      }
      fprintf(f, "stroke\n");
    }
  }

}


/*--
  \end{code}

  \funparagraph{put\_td\_to\_latex\_list}

  \begin{code}
  --*/


void put_td_to_latex_list(PERFILE *p, long m0)

{
  char locn[64];

  open_eps_file();
  put_td_to_eps(the_eps_file(), p, m0);
    /* not yet closed in order to make label name available */
  
  fprintf(the_latex_list(), "\n\n\n");

  if(use_figure_environments){
    fprintf(the_latex_list(), "\n\n\n\\begin{figure}[hbt]\n");
  }
  fprintf(the_latex_list(), "\\epsffile{%s}\n", the_eps_file_name());

  if(use_figure_environments)
    fprintf(the_latex_list(), "\\caption{%s}\\label{%s}\n",
	    td_title, the_label_name());
  else
    fprintf(the_latex_list(), "\\centerline{%s}\n", td_title);
  if(use_figure_environments) {
    fprintf(the_latex_list(), "\\end{figure}\n\n");
  }
  sprintf(locn, "\\ref{%s}", the_label_name());
  fprintf(the_latex_list(), td_comment, locn);
  close_eps_file();  /* eps counter incremented */
}


/*--
  \end{code}


  \endoffile

  --*/



