
/*--

\FileTitle{ 
{\tt egevents.c} --- management of events
}
\CopyrightNotice{ 

    Copyright (c) 1991--1996 Alexandru Dan Corlan MD (http://dan.corlan.net)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
}

\section{Events manager}

\ImplementationNotes{ 
The goal of the implementation of this package is to allow a reasonably
fast answer to the problem: which are the events (intervals) starting in,
ending in, or taking place over a given interval of time (which is
to be displayed or otherwise processed). 

But, for the moment, we are only going to implement it as a
vector of events.
}

\begin{code}
--*/

#include "ilsix.h"
#include "ecgnosis.h"

#define MAX_N_EV_ARGS 3

typedef struct evtype {
  long  type;
  cell  name;
  cell  args[MAX_N_EV_ARGS];
  short argc;
} EVTYPE;

#define MAX_N_EV_TYPES 1000

short n_ev_types = 0;

EVTYPE evtypes[MAX_N_EV_TYPES];

static init_evtypes () { n_ev_types = 0; }



#define MAX_N_EVS 4000

typedef struct ev {
  long from;
  long to;
  long type;
  short na;
  cell args[MAX_N_EV_ARGS];
} EV;

short n_evs = 0;

EV evs[MAX_N_EVS];

void init_evs() { n_evs = 0; }




static void raise_internal_error() { exit(0); /* cry somehow */ }
static void raise_user_error()     { exit(0); /* cry somehow else */ }

short locate_ev_type(long t)  /* returns index in evtypes[] */

{
  short i;

  for(i=0; i<n_ev_types; i++)
     if (evtypes[i].type = t)
        return i;
  return -1;
}

void def_ev_type(cell n, long t)

{
  short e;

  if((e=locate_ev_type(t))!=-1)
    raise_internal_error();
  evtypes[e].type = t;
  evtypes[e].name = n;
  evtypes[e].argc = 0;
  n_ev_types ++;
  if(n_ev_types > MAX_N_EV_TYPES)
    raise_internal_error();
}

void def_ev_type_arg(long t, cell a)

{
  short e;

  if((e=locate_ev_type(t))==-1)
    raise_internal_error();
  if(evtypes[e].argc >= MAX_N_EV_ARGS)
    raise_internal_error();
  evtypes[e].args[evtypes[e].argc] = a;
  evtypes[e].argc ++;
}




short def_ev(long evtype, long from, long to)

{
  short e, i;

  if (n_evs >= MAX_N_EVS)
     raise_user_error();
  evs[n_evs].from = from;
  evs[n_evs].to = to;
  evs[n_evs].type = evtype;
  e = locate_ev_type(evtype);
  if(e==-1)
    raise_internal_error();
  evs[n_evs].na = ev_types[e].argc;
  for(i=0; i<ev_types[e].argc; i++)
      evs[n_evs].args[i] = NIL;
  n_evs ++;
  return n_evs-1;
}

void set_ev_arg(short e, short argth, cell v)

{
  ungstruct(evs[e].args[argth]);
  evs[e].args[argth] = v;
}

short ev_type_argth(long t, cell argname)

{
  short et, i;

  et = locate_ev_type(t);
  for(i=0; i<evtypes[et].argc; i++)
    if(evtypes[et].args[i]==argname)
      return i;
  raise_user_error();  /* No such event argument */
}


void egevents()

/* In order to simplify things, we are going to
   initialize here all the events */

{
  init_ev_types();
  init_evs();

}

/*--
\end{code}

\endoffile

--*/

