
/*--

  \FileTitle{ 
  EGINT --- Electrocardiognosis script interpreter
  }
  \CopyrightNotice{

    Copyright (c) 1991--1996 Alexandru Dan Corlan MD (http://dan.corlan.net)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  }

  \section{Interpreter implementation}

  \begin{code}
  --*/


#include <stdio.h>

#include "gliks.h"
#include "ecgnosis.h"


/*--
  \end{code}

  \funparagraph{batch\_qrs\_detect}

  \begin{code}
  --*/


short batch_qrs_detect(PERFILE *p, char *qrsfn) 		/* [FUN] */ 

{
  long m0, m1, delt;
  long lim, cbt;
  long cbl;
  long *cb;

  cbl = 0;
  cb = NULL;
  if(!open_qrs_file(p, qrsfn)) {
    fprintf(stderr, "Can't open QRS file for write.\n");
    return 0;
  }
  lim = rec_file_duration(p)-50000;
  delt = 1200000;
  m0 = 50000;
  while(m0<lim) {
    m1 = m0+delt;
    if(m1>lim)
      m1 = lim-1;
    if(m1-m0<50000)
      break;
    fprintf(stderr, "Computing QRSs in %02ld:%02ld:%02ld--%02ld:%02ld:%02ld ",
      m0 / 36000000, (m0 % 36000000) / 600000, (m0 % 600000) / 10000,
      m1 / 36000000, (m1 % 36000000) / 600000, (m1 % 600000) / 10000);
    cbt = complexes_between(p, m0, m1);
    fprintf(stderr, "%ld complexes ", cbt); 
    if(cbt==-1) /* for some reason */
      break;
    if(cbt>cbl) {
      if(cb!=NULL)
	free(cb);
      cb = (long *)malloc(cbt * sizeof(long));
      if(cb==NULL) {
	fprintf(stderr, "Allocation error\n");
	break;  /* Can't read further */
      }
    }
    cpars_at(p, cb, CPQRSPOS, m0, cbt);
/*    fprintf(stderr, "cpars_at done\n");*/
    if(!write_to_qrs_file(p, m0, m1)) {
      fprintf(stderr, "Writing to qrs file failed.\n");
      break;
    }
    fprintf(stderr, " WRITTEN\n");
    m0 = m1+1;
  }
  close_qrs_file(p);
  free(cb);
  return 1;
}


/*--
  \end{code}

  % MOVE THIS COMMENT TO THE END OF THE FILE
  \endoffile

  --*/



