
/*--

  \FileTitle{
  {\tt egmain.c} --- Main module of Electrocardiognosis
  }
  \CopyrightNotice{

    Copyright (c) 1991--1996 Alexandru Dan Corlan MD (http://dan.corlan.net)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

  }
  \vskip 1in

  \begin{code}
  --*/

#include <stdio.h> 
#include <stdlib.h>

#ifdef WIPORT2X
#include "wiport2x.h"
#else
#include <wiport2.h>
#endif


#ifdef _MSDOS_
#include <ilfive/shpell5.h>
#include <ilfive/il5vars.h>
#else
#include <shpell5.h>
#include <il5vars.h>
#endif

#include "gliks.h"
#include "ecgnosis.h"

/* External definitions that should be located in glikrec.h */

extern short nof_scat_divs;  /* from egscatv */

extern short thethr;         /* from glikalys */

extern FILE *the_epsf();     /* from gliktype */

#define TRACE                                                   /* [DEF] */ 
FILE *ecgnosislog;                                              /* [VAR] */ 

/* Time processing package */

long start_time = 0; /* in ccs from start of day */ 		/* [VAR] */ 

long rel2abs_time(long t)

{
  t += start_time;
  if(t > 864000000)
    t -= 864000000;
  return t;
}

long gtimeparse(char *tb)  /* in 5ms-samples */ 		/* [FUN] */ 

{
  long r;

  r = 0;
  r += atol(tb) * 3600 * 200;
  while(*tb && *tb<='9' && *tb>='0')
    tb++;
  if(*tb!=':') return r;
  tb++;
  r += atol(tb) * 60 * 200;
  while(*tb && *tb<='9' && *tb>='0')
    tb++;
  if(*tb!=':') return r;
  tb++;
  r += atol(tb) * 200;
  return r;
  }



/*--
\end{code}

   \section{Main program}

\begin{code}
--*/

char qrs_bat_buf[100];
short batmode = 0;
short assqrsmode = 0;

short crtqrslead = 0;

short from_diag = 1;

void errhandle() {}

extern PERFILE *open_template();

main(short argc, char **argv)                                   /* [FUN] */ 

{
GWIN *gg;
PERFILE *thep;
short i; 
char resbuf[70];

ecgnosislog = fopen("ecgnosis.log", "w");
if(ecgnosislog==NULL) {
  printf("Can't open the ECGNOSIS.LOG file.\n");
  exit(0);
}


printf("Before il5(1)\n");
il5(1);
termlevel = PLAIN_CHAR;


printf("Before init_tds()\n");
printf("Before init_tds()\n");
printf("Before init_tds()\n");
printf("Before init_tds()\n");
printf("Before init_tds()\n");
printf("Before init_tds()\n");
init_tds();

printf("Before wipres()\n");
wipres();

egviewdt_init();
egctrl();
read_site_info();

/******** DELETE ALL SCREEN DUMP FILES **********/

unlink("mainscr.eps");
unlink("tdreport.eps");
unlink("tdmenu.eps");
unlink("tdlook.eps");
unlink("streport.eps");
unlink("stdisply.eps");
unlink("stmarks.eps");
unlink("tdselect.eps");
unlink("hrvscr.eps");

/******** DEFINE DEFAULT VALUES FOR THE RESOURCES                    *******/

  /* The colors of the td screen */

/*  gliktype();*/

  setresprefix("TdScreen-");
  get_screen_colors(reds, greens, blues);
  color2defres(reds, greens, blues);
  setresprefix("ScatScreen-");
  color2defres(reds, greens, blues);
  setresprefix("");

/******** LAST STEP OF THE INITIALISATION: READ IN ANY RESOURCE FILE *******/

  resread("ecgnosis.res");


/* Open and display part of the ECG buffer */

#ifdef MAINTRACE
fprintf(ecgnosislog, "MAIN before the rec_file_open.%c\n", 7); fflush(ecgnosislog);
#endif


if(argc>1) {
  if(!strcmp(argv[1], "-?")) {
    wiportclose();
    fprintf(stderr,
     "Use: ecgnosis [-b|qrs n] [-from n] [-start t] [=qdl l] [-g name|file]\n");
    exit(0);
  }
  if(!strcmp(argv[1], "-b")) {
    strcpy (qrs_bat_buf, argv[2]);
    batmode = 1;
    argv += 2;
    argc -= 2;
  }
  if(!strcmp(argv[1], "-qrs")) {
    strcpy (qrs_bat_buf, argv[2]);
    assqrsmode = 1;
    argv += 2;
    argc -= 2;
  }
  if(!strcmp(argv[1], "-from")) {
    from_diag = atoi(argv[2]);
    argv += 2;
    argc -= 2;
  }

  if(!strcmp(argv[1], "-start")) {
    start_time = 50 * gtimeparse(argv[2]);
    argv += 2;
    argc -= 2;
  }
  crtqrslead = 0;
  if(!strcmp(argv[1], "=qdl")) {
    crtqrslead = atoi(argv[2]);
    argv += 2;
    argc -= 2;
  }
  
  thep = NULL;
  if(!strcmp(argv[1], "-g")) {
    generate_names_from(argv[2]);
    thep = open_template(AUTOMATA_COMPRESSION);
    open_latex_file();
    open_latex_list();
    if(from_diag>1)
      set_eps_counter(from_diag);
  }

  if(!strcmp(argv[1], "-a")) {
    generate_names_from(argv[2]);
    thep = open_template(ANTICOMPRESSION);
    open_latex_file();
    open_latex_list();
    if(from_diag>1)
      set_eps_counter(from_diag);
  }
/*    thep = rec_file_open(NULL, argv[2], ANTICOMPRESSION);
  else
    thep = rec_file_open(NULL, argv[1], AUTOMATA_COMPRESSION);*/
}
else {
  fprintf(ecgnosislog, "A name must be supplied.\n");
  fflush(ecgnosislog);
  goto proper_exit;
}

if(thep==NULL)
  {
  fprintf(ecgnosislog, "Can't open the ECG file at all.%c\n%s", 7, glikerrmsg);
  fflush(ecgnosislog);
  goto proper_exit;
  }
set_crt_qrs_lead(thep, crtqrslead);

if(batmode) {
  batch_qrs_detect(thep, qrs_bat_buf);
  fprintf(stderr, "Done.\n");
  exit(0);
}

wiportinit(NULL, 0, 0, argc, argv);
 printf("before newgwin\n"); 
gg = newgwin(640, 480, "Electrocardiognosis", "ECGnosis");
 printf("before use color s tech\n"); 
use_color_s_tech();
gwerrno = 0;
cleardwin(gg, 0, PLANE_DWIN);


if(assqrsmode) {
  assign_qrs_file(thep, qrs_bat_buf);
}

nof_scat_divs = rec_file_duration(thep) / 10000000 + 1;

/*
if(use_events_f) {
  thep->nof_events = get_button_information(thep->e,
					    thep->b_events, MAX_NO_EVENTS);
  for(i=0; i<thep->nof_events; i++) {
    thep->b_events[i] = thep->b_events[i] * 25;
  }
}
*/

td_interraction(gg, thep);
wi_set_eps_unit(700);
wi_print_gwin("mainscr.eps");
setresprefix("");

/**************** THE NAMES OF EPS FILES WITH FULL GWIN CONTENTS

  mainscr.eps  --- the main screen
  tdreport.eps --- inserting a time diagram in a report
  tdmenu.eps   --- time diagram menu
  streport.eps --- inserting a strip in a report
  stdisply.eps --- strip display menu
  stmarks.eps  --- selection of strip markings
  tdselect.eps --- selection of time diagram type

/* problem
XFreeGC(gg->gwid, gg->gwigc);
XDestroyWindow(gg->gwid, gg->gwiw);
XCloseDisplay(gg->gwid);
*/

proper_exit:
/* unlink("ecgnosis.res"); */
/* reswrite("ecgnosis.res");*/

wiportclose();
close_latex_list();
close_latex_file();
fprintf(ecgnosislog, "%d\n", gwerrno); fflush(ecgnosislog);
}

/*--
  \end{code}

  % MOVE THIS COMMENT TO THE END OF THE FILE
  \endoffile

  --*/


