/*--
  \FileTitle{
  {\tt egnames.c} --- external names used by the ECGNOSIS application
  }
  \CopyrightNotice{

    Copyright (c) 1991--1996 Alexandru Dan Corlan MD (http://dan.corlan.net)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  }
  
  
  \section{Names}
  
  \begin{code}
  --*/


#include <stdio.h>

#include "ilsix.h"

#include "gliks.h"
#include "ecgnosis.h"



/*--
  \end{code}

  \dataparagraph{templates and file names}

  \begin{code}
  --*/


#define PATH_LEN 140 						/* [DEF] */ 

char ecgnosis_home_dir[PATH_LEN] = "./";         		/* [VAR] */ 
char res_file_name[PATH_LEN] = "ecgnosis.res"; 			/* [VAR] */ 
char site_file_name[PATH_LEN] = "ecgnosis.sit"; 		/* [VAR] */ 
char user_file_name[PATH_LEN] = "ecgnosis.usr";    		/* [VAR] */ 

char ecg_file_template[PATH_LEN] = "%s.gwo"; 			/* [VAR] */ 
char analysis_file_template[PATH_LEN] = "%s.qst"; 		/* [VAR] */ 

char eps_file_template[PATH_LEN] = "%s.%d"; 			/* [VAR] */ 
char eps_file_def_template[PATH_LEN] = "%s.edf"; 		/* [VAR] */ 
char label_template[PATH_LEN] = "%s-%d";                        /* [VAR] */ 
char latex_list_template[PATH_LEN] = "%s.tex"; 			/* [VAR] */ 
char ps_list_template[PATH_LEN] = "%s.psl"; 			/* [VAR] */ 
char latex_report_template[PATH_LEN] = "%s.ltx"; 		/* [VAR] */ 
char ps_report_template[PATH_LEN] = "%s.ps"; 			/* [VAR] */ 
char control_file_template[PATH_LEN] = "%s.ctl"; 		/* [VAR] */ 

char report_file_template[PATH_LEN] = "%s.rep";

char ecg_file_name[PATH_LEN]; 					/* [VAR] */ 
char analysis_file_name[PATH_LEN]; 				/* [VAR] */ 

char eps_file_name[PATH_LEN]; 					/* [VAR] */ 
char label_name[PATH_LEN];        /* This is not a path */ 	/* [VAR] */ 
char latex_list_name[PATH_LEN]; 				/* [VAR] */ 
char ps_list_name[PATH_LEN]; 					/* [VAR] */ 
char latex_report_name[PATH_LEN]; 				/* [VAR] */ 
char ps_report_name[PATH_LEN]; 					/* [VAR] */ 
char control_file_name[PATH_LEN]; 				/* [VAR] */ 

char report_file_name[PATH_LEN];

char template[PATH_LEN] = "NoFile"; 				/* [VAR] */ 


/*--
  \end{code}

  \dataparagraph{Report files}

  \begin{code}
  --*/

short diag_count = 1; 						/* [VAR] */ 

FILE *latex_report_file = NULL; 				/* [VAR] */ 
FILE *latex_list_file = NULL; 					/* [VAR] */ 

FILE *ps_report_file = NULL; 					/* [VAR] */ 
FILE *ps_list_file = NULL; 					/* [VAR] */ 

FILE *report_file = NULL;

FILE *current_eps = NULL; 					/* [VAR] */ 

/*--
  \end{code}

  \funparagraph{generate\_names\_from}

  \begin{code}
  --*/

void generate_names_from(char *tem) 				/* [FUN] */ 

{
  diag_count = 1;
  strncpy(template, tem, PATH_LEN);
  sprintf(ecg_file_name, ecg_file_template, tem);
  sprintf(analysis_file_name, analysis_file_template, tem);
  sprintf(control_file_name, control_file_template, tem);

  sprintf(latex_list_name, latex_list_template, tem);
  sprintf(ps_list_name, ps_list_template, tem);
  sprintf(latex_report_name, latex_report_template, tem);
  sprintf(ps_report_name, ps_report_template, tem);

  sprintf(label_name, label_template, tem, diag_count);
  sprintf(eps_file_name, eps_file_template, tem, diag_count);

  sprintf(report_file_name, report_file_template, tem);

/*  sprintf(eps_file_def_name, eps_file_def_template, tem, diag_count); */
}



/*--
  \end{code}

  \funparagraph{set\_eps\_counter} 

  \begin{code}
  --*/

void set_eps_counter(short new_counter) 			/* [FUN] */ 

{
  diag_count = new_counter;

  sprintf(label_name, label_template, template, diag_count);
  sprintf(eps_file_name, eps_file_template, template, diag_count);
/*  sprintf(eps_file_def_name, eps_file_def_template, template, diag_count);*/
}


/*--
  \end{code}

  \funparagraph{inc\_eps\_counter}

  \begin{code}
  --*/


void inc_eps_counter() 			/* [FUN] */ 

{
  diag_count ++;

  sprintf(label_name, label_template, template, diag_count);
  sprintf(eps_file_name, eps_file_template, template, diag_count);
  add_assertion(cons(cl_make_symbol("new-diagram"),
		     cons(cl_make_symbol(":elt"),
			  cons(cl_make_integer(diag_count), NIL))));
/*  sprintf(eps_file_def_name, eps_file_def_template, template, diag_count);*/
}


/*--
  \end{code}

  \funparagraph{latex report files}

  \begin{code}
  --*/

short open_latex_file() 					/* [FUN] */ 

{
  latex_report_file = fopen(latex_report_name, "w");
  if(latex_report_file==NULL)
    return 0; /* Failure */

  fprintf(latex_report_file, "\\documentstyle[ecgnosis]{article}\n");
  fprintf(latex_report_file, "\n\n");
  fprintf(latex_report_file, "\\title{Raport pentru {\\tt %s}}\n",
	  ecg_file_name);
  fprintf(latex_report_file, "\\author{\\sf Electrocardiognosis}\n");
  fprintf(latex_report_file, "\\begin{document}\n");
  fprintf(latex_report_file, "\n");
  fprintf(latex_report_file, "\\maketitle\n");
  fprintf(latex_report_file, "\n");
  fprintf(latex_report_file, "\\input %s\n", latex_list_name);
  fprintf(latex_report_file, "\n");

  /* Printing into "latex-report-file" may add text after the list */
  
  return 1; /* Success */
}

short close_latex_file() 					/* [FUN] */ 

{
  if(latex_report_file==NULL)
    return 0;  /* Failure */
  fprintf(latex_report_file, "\\end{document}\n");
  fclose(latex_report_file);
  latex_report_file = NULL;
  return 1; /* Success */
}

char *the_latex_file_name() 					/* [FUN] */ 

{
  return latex_report_name;
}

FILE *the_latex_file() 						/* [FUN] */ 

{
  return latex_report_file;
}


/*--
  \end{code}

  \funparagraph{latex list files}

  \begin{code}
  --*/


short open_latex_list() 					/* [FUN] */ 

{

  /* OBS: opening in append mode must be considered here for the case
     in which some of the diagrams have already been produced in the file;
     the siuation in which appending is required also is an issue.
     */

  if(latex_list_file!=NULL)
    return 0;  /* Can't open again */
  
  latex_list_file = fopen(latex_list_name, "a");
  if(latex_list_file==NULL)
    return 0; /* Failure */
  return 1; /* Success */
}

short close_latex_list() 					/* [FUN] */ 

{
  if(latex_list_file==NULL)
    return 0;  /* Failure */
  fclose(latex_list_file);
  latex_list_file = NULL;
  return 1; /* Success */
}

char *the_latex_list_name() 					/* [FUN] */ 

{
  return latex_list_name;
}

FILE *the_latex_list() 						/* [FUN] */ 

{
  return latex_list_file;
}



/*--
  \end{code}

  \funparagraph{eps files}

  \begin{code}
  --*/

short open_eps_file() 						/* [FUN] */ 

{
  if(current_eps!=NULL)
    return 0;

  current_eps = fopen(eps_file_name, "w");
  if(current_eps==NULL)
    return 0;
  
  return 1;
}

short close_eps_file() 						/* [FUN] */ 

{
  if(current_eps==NULL)
    return 0;
  fclose(current_eps);
  current_eps = NULL;
  inc_eps_counter();
}

FILE *the_eps_file() 						/* [FUN] */ 

{
  return current_eps;
}

char *the_eps_file_name() 					/* [FUN] */ 

{
  return eps_file_name;
}

short open_report_file()

{
  if (report_file!=NULL)
    return 0;
  report_file = fopen(report_file_name, "a");
  if(report_file==NULL)
    return 0;
  return 1;
}

short close_report_file()

{
  if(report_file==NULL)
    return 0;
  fclose(report_file);
  report_file=NULL;
}

FILE *the_report_file()

{
  return report_file;
}

/*--
  \end{code}

  \funparagraph{the\_label\_name}

  \begin{code}
  --*/

char *the_label_name() 						/* [FUN] */ 

{
  return label_name;
}

short current_eps_count()

{
  return diag_count;
}


/*--
\end{code}

\funparagraph{the\_control\_file\_name}

\begin{code}
--*/

char *the_control_file_name()

{
  return control_file_name;
}

/*--
  \end{code}

  \subsection{Miscellaneous names}


  \begin{code}
  --*/

char *the_ecg_file_name()

{
  return ecg_file_name;
}

char *the_site_file_name()

{
  return site_file_name;
}


/*--
  \end{code}

  \endoffile

  --*/



