
/*--

  \FileTitle{
  EGVIEW --- Electrocardiognosis Viewer Implementation
  }
  \CopyrightNotice{ 

    Copyright (c) 1991--1996 Alexandru Dan Corlan MD (http://dan.corlan.net)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

  }
  \begin{code}
  --*/


/*--
  \end{code}

  \section{Viewer Implementation}

  \begin{code}
  --*/

#include <stdio.h>

#ifdef WIPORT2X
#include "wiport2x.h"
#else
#include <wiport2.h>
#endif

#include "gliks.h"
#include "ecgnosis.h"

/*--
  \end{code}

  \dataparagraph{Dwin variables}

  \begin{code}
  --*/

short upd;      /* Upper DWIN */
short lod;      /* Lower DWIN */
short umctrld;  /* Upper-Medium Control DWIN */
short lctrld;   /* Lower Control DWIN */
short rjumps;   /* Right side buttons */
short fulld;    /* Full display dwin */
short prod;     /* Place for progression indicator */
short scatd;    /* The scatter diagram */
short scatpd;   /* The scatter diagram position */
short topd;     /* The uppermost line of the screen */


/*--
  \end{code}

  \funparagraph{Screen definition}

  \begin{code}
  --*/

void part_td_screen(GWIN *gg)

{
  
  rjumps = newdwin(gg, 519, 1, 120, 389);
  fulld = newdwin(gg, 12, 25, 500, 430);
  prod = newdwin(gg, 519, 395, 110, 59);
  upd = newdwin(gg, 12, 25, 500, 240);
  umctrld = newdwin(gg, 9, 275, 540, 30);
  lod = newdwin(gg, 12, 309, 500, 130);
  lctrld = newdwin(gg, 9, 449, 500, 20);
  scatd = newdwin(gg, 20, 27, 400, 400);
  scatpd = newdwin(gg, 20, 432, 400, 18);
  topd = newdwin(gg, 12, 1, 500, 20);

  redd = newdwin(gg, COLPXORG, COLCYORG, COLU, 100);
  greend = newdwin(gg, COLPXORG + COLU + COLU / 2, COLCYORG, COLU, 100);
  blued = newdwin(gg, COLPXORG + 3 * COLU, COLCYORG, COLU, 100);
  colpadd = newdwin(gg, COLPXORG, COLPYORG, 4*COLU, 4*COLU);

}


/*--
  \end{code}

  \subsection{Colors management implementation}

  \begin{code}
  --*/


/*--
  \end{code}

  \dataparagraph{Local variables for color definitions}

  \begin{code}
  --*/


short reds[20];
short blues[20];
short greens[20];

short reds_bk[20];
short blues_bk[20];
short greens_bk[20];


/*--
  \end{code}

  \dataparagraph{Color dwins}

  \begin{code}
  --*/


short colpadd;
short redd;
short greend;
short blued;


/*--
  \end{code}

  \funparagraph{res2color}

  \begin{code}
  --*/


short res2color()  /* From the resources having
		      the current prefix---set the colors */

{
  short i, r, g, b;
  char resbuf[70];

  for(i=0; i<16; i++) {
    sprintf(resbuf, "red-of-%d", i);
    r = ires(resbuf);
    sprintf(resbuf, "blue-of-%d", i);
    b = ires(resbuf);
    sprintf(resbuf, "green-of-%d", i);
    g = ires(resbuf);
    def_screen_color(i, r, g, b);
  }
}


/*--
  \end{code}

  \funparagraph{color2res}

  \begin{code}
  --*/


color2res(short *r, short *g, short *b) {

  short i;
  char resbuf[70];
  
  for(i=0; i<16; i++) {
    sprintf(resbuf, "red-of-%d", i);
    iresset(resbuf, r[i]);
    sprintf(resbuf, "blue-of-%d", i);
    iresset(resbuf, b[i]);
    sprintf(resbuf, "green-of-%d", i);
    iresset(resbuf, g[i]);
  }
}


/*--
  \end{code}

  \funparagraph{color2defres}

  \begin{code}
  --*/


color2defres(short *r, short *g, short *b) {

  short i;
  char resbuf[70];
  
  for(i=0; i<16; i++) {
    sprintf(resbuf, "red-of-%d", i);
    iresdef(resbuf, r[i], NULL);
    sprintf(resbuf, "blue-of-%d", i);
    iresdef(resbuf, b[i], NULL);
    sprintf(resbuf, "green-of-%d", i);
    iresdef(resbuf, g[i], NULL);
  }
}


/*--
  \end{code}

  \funparagraph{show\_color\_adj}

  \begin{code}
  --*/

void show_color_adj(GWIN *gg)

{
  short i, x, y;

  cleardwin(gg, colpadd, LOWERED_DWIN);
  cleardwin(gg, redd, LOWERED_DWIN);
  cleardwin(gg, blued, LOWERED_DWIN);
  cleardwin(gg, greend, LOWERED_DWIN);

  for(x=0; x<4; x++)
    for(y=0; y<4; y++) {
      gsetcolor(gg, colpadd, x*4+y);
      for(i=0; i<COLU; i++) {
	gmoveto(gg, colpadd, x*COLU, y*COLU+i);
	glineto(gg, colpadd, x*COLU+COLU-1, y*COLU+i);
      }
    }

  defbutton(gg, rjumps, COLPXORG-520, COLCYORG-COLU-10, COLU, COLU, "+", 'R');
  defbutton(gg, rjumps, COLPXORG-520 + COLU + COLU/2, COLCYORG-COLU-10, COLU,
	    COLU, "+", 'G');
  defbutton(gg, rjumps, COLPXORG-520 + 3*COLU, COLCYORG-COLU-10, COLU,
	    COLU, "+", 'B');
  defbutton(gg, rjumps, COLPXORG-520, COLCYORG+COLU+90, COLU, COLU, "-", 'r');
  defbutton(gg, rjumps, COLPXORG-520 + COLU + COLU/2, COLCYORG+COLU+90, COLU,
	    COLU, "-", 'g');
  defbutton(gg, rjumps, COLPXORG-520 + 3*COLU, COLCYORG+COLU+90, COLU,
	    COLU, "-", 'b');

  defbutton(gg, rjumps, COLPXORG-520,
	    COLCYORG-COLU-30, COLU/2, 12, "<", 'p');
  defbutton(gg, rjumps, COLPXORG-520 + 3*COLU + COLU/2,
	    COLCYORG-COLU-30, COLU/2, 12, ">", 'n');
  

  defbutton(gg, rjumps, COLPXORG-520, COLCYORG+140, 4*COLU, COLU, "Undo", 'U');
  defbutton(gg, rjumps, COLPXORG-520, COLCYORG+170, 4*COLU, COLU, "Restore", 'u');
  defbutton(gg, rjumps, COLPXORG-520, COLCYORG+200, 4*COLU, COLU, "Done", 'd');

  setspotshot(gg, redd, 'r'-'a'+1);
  setspotshot(gg, greend, 'g'-'a'+1);
  setspotshot(gg, blued, 'b'-'a'+1);
  setspotshot(gg, colpadd, 'q'-'a'+1);

}



/*--
  \end{code}

  \funparagraph{show\_color\_levels}

  \begin{code}
  --*/

/* The following function displays the levels reached by the current color */
/* It assumes that the above function was called, so that it can use the */
/* already displayed redd, greend and blued */

static _scl(GWIN *gg, short d, short l) {
  
  gsetcolor(gg, d, 15);
  gmoveto(gg, d, 0, 100 - l);
  glineto(gg, d, COLU-1, 100 - l);

  gsetcolor(gg, d, 1);
  gmoveto(gg, d, 0, 101 - l);
  glineto(gg, d, COLU-1, 101 - l);
  gmoveto(gg, d, 0, 99 - l);
  glineto(gg, d, COLU-1, 99 - l);
}

void show_color_levels(GWIN *gg, short crtcol,
		       short r, short g, short b)  /* between 0..10000 */

{
  cleardwin(gg, redd, LOWERED_DWIN);
  cleardwin(gg, greend, LOWERED_DWIN);
  cleardwin(gg, blued, LOWERED_DWIN);

  
  _scl(gg, redd, r/100);
  _scl(gg, blued, b/100);
  _scl(gg, greend, g/100);
}


/*--
  \end{code}

  \funparagraph{adjust\_colors}

  \begin{code}
  --*/


/* The following function is given 3 vectors of numbers
 * between 0 and 10.000. It displays in the rjumps dwin
 * a set of cursors by means of which the red, green and
 * blue ammounts of color can be adjusted in the main window.
 * Adjustment is done until the desired color is reached. 
 * The values in the 3 vectors are adjusted accordingly.
 */


void adjust_colors(GWIN *gg, char *res_pref)

{
  short ch;
  short crtcolor, i;
  short locx, locy;
  
  setresprefix(res_pref);  
  show_color_adj(gg);
  get_screen_colors(reds, greens, blues);
  get_screen_colors(reds_bk, greens_bk, blues_bk);
  
  crtcolor = 0;
  do {
    show_color_levels(gg, crtcolor,
		      reds[crtcolor], greens[crtcolor], blues[crtcolor]);
    ch = gwinteract(gg);
    switch(ch) {
    case 'r':
      if(reds[crtcolor]>100)
	reds[crtcolor] -= 100;
      else
	reds[crtcolor] = 0;
      def_screen_color(crtcolor, reds[crtcolor],
		       greens[crtcolor], blues[crtcolor]);
      break;
    case 'R':
      if(reds[crtcolor]<9900)
	reds[crtcolor] += 100;
      else
	reds[crtcolor] = 9999;
      def_screen_color(crtcolor, reds[crtcolor],
		       greens[crtcolor], blues[crtcolor]);
      break;
    case 'r'-'a'+1:
      reds[crtcolor] = 100 * (100 - yspotshot);
      def_screen_color(crtcolor, reds[crtcolor],
		       greens[crtcolor], blues[crtcolor]);
      break;
    case 'g':
      if(greens[crtcolor]>100)
	greens[crtcolor] -= 100;
      else
	greens[crtcolor] = 0;
      def_screen_color(crtcolor, reds[crtcolor],
		       greens[crtcolor], blues[crtcolor]);
      break;
    case 'G':
      if(greens[crtcolor]<9900)
	greens[crtcolor] += 100;
      else
	greens[crtcolor] = 9999;
      def_screen_color(crtcolor, reds[crtcolor],
		       greens[crtcolor], blues[crtcolor]);
      break;
    case 'g'-'a'+1:
      greens[crtcolor] = 100 * (100 - yspotshot);
      def_screen_color(crtcolor, reds[crtcolor],
		       greens[crtcolor], blues[crtcolor]);
      break;
    case 'b':
      if(blues[crtcolor]>100)
	blues[crtcolor] -= 100;
      else
	blues[crtcolor] = 0;
      def_screen_color(crtcolor, reds[crtcolor],
		       greens[crtcolor], blues[crtcolor]);
      break;
    case 'B':
      if(blues[crtcolor]<9900)
	blues[crtcolor] += 100;
      else
	blues[crtcolor] = 9999;
      def_screen_color(crtcolor, reds[crtcolor],
		       greens[crtcolor], blues[crtcolor]);
      break;
    case 'b'-'a'+1:
      blues[crtcolor] = 100 * (100 - yspotshot);
      def_screen_color(crtcolor, reds[crtcolor],
		       greens[crtcolor], blues[crtcolor]);
      break;
    case 'p':
      if(crtcolor>0)
	crtcolor--;
      break;
    case 'n':
      if(crtcolor<15)
	crtcolor++;
      break;
    case 'q'-'a'+1:  /* SHOT in the color's region */
      locx = xspotshot / COLU;
      locy = yspotshot / COLU;
      crtcolor = locx*4 + locy;
/*      printf("COLOR SELECTED AT: %d, %d\n", xspotshot, yspotshot); */
      break;
    case 'u':
      reds[crtcolor] = reds_bk[crtcolor];
      greens[crtcolor] = greens_bk[crtcolor];
      blues[crtcolor] = blues_bk[crtcolor];
      def_screen_color(crtcolor, reds[crtcolor],
		       greens[crtcolor], blues[crtcolor]);
      break;
    case 'U':
      for(i=0; i<16; i++) {
	reds[i] = reds_bk[i];
	greens[i] = greens_bk[i];
	blues[i] = blues_bk[i];
	def_screen_color(i, reds[i],
			 greens[i], blues[i]);
      }
      break;
    default:
      color2res(reds, greens, blues);
      setresprefix("");
      goto out_of_it;
    }
  } while(1);

 out_of_it:
  cleardwin(gg, colpadd, PLANE_DWIN);
  cleardwin(gg, redd, PLANE_DWIN);
  cleardwin(gg, greend, PLANE_DWIN);
  cleardwin(gg, blued, PLANE_DWIN);
  cleardwin(gg, rjumps, PLANE_DWIN);
}


/*--
  \end{code}

  \endoffile

  --*/
