
/*--

  \FileTitle{
  {\tt egviewmf.c} --- Electrocardiognosis, view morphologies
  }
  \CopyrightNotice{


    Copyright (c) 1991--1996 Alexandru Dan Corlan MD (http://dan.corlan.net)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

  }

  \subsection{Morphologies}

  \begin{code}
  --*/


#include <stdio.h>   

#ifdef WIPORT2X
#include "wiport2x.h"
#else
#include <wiport2.h>
#endif

#include "gliks.h"
#include "ecgnosis.h"


/*--
  \end{code}

  \dataparagraph{Display and interraction parameters}

  \begin{code}
  --*/

void right_morph_buttons(GWIN *gg) { 				/* [FUN] */ 
  
  cleardwin(gg, rjumps, PLANE_DWIN);
  defbutton(gg, rjumps, 18,  23, 90, 20, "Help", 'h');
  defbutton(gg, rjumps, 18,  53, 90, 20, "Print", 'p');
  defbutton(gg, rjumps, 18, 113, 90, 20, "Back", ' ');
  defbutton(gg, rjumps, 18, 143, 90, 20, "Colors", 'c');
}


/*--
  \end{code}

  \funparagraph{make\_morph\_screen}

  \begin{code}
  --*/

make_morph_screen(GWIN *gg) 					/* [FUN] */ 

{
  cleardwin(gg, 0, PLANE_DWIN);
  cleardwin(gg, upd, PLANE_DWIN);
  cleardwin(gg, lod, PLANE_DWIN);
  cleardwin(gg, umctrld, PLANE_DWIN);
  cleardwin(gg, lctrld, PLANE_DWIN);
  cleardwin(gg, fulld, PLANE_DWIN);

  right_morph_buttons(gg);

  gsetcolor(gg, fulld, 14);

  defbutton(gg, fulld, 420, 385, 30, 16, "<<<", 'a');
  defbutton(gg, fulld, 460, 385, 30, 16, ">>>", 'f');
}


/*--
  \end{code}

  \funparagraph{morph\_display} 
  \ImplementationNotes{
  At most 8 morphologies are displayed, starting with the
  indicated one.
  }

  \begin{code}
  --*/

static short mbuf[100]; 					/* [VAR] */ 
static char mdtext[30]; 					/* [VAR] */ 
static short total_nof_morphs; 					/* [VAR] */ 
static long total_nof_gliks; 					/* [VAR] */ 

static char *sclasses[8] = { 					/* [VAR] */ 
  "     ",
  "     ",
  "     ",
  "     ",
  "     ",
  "     ",
  "     ",
  "     "}; /* just to allocate space */

static char *gtype_names[8] = { 				/* [VAR] */ 
  "UNKN", "ARTF", "SV", "BBB", "WPW", "VEN", "ATIP"
  };

void morph_display(GWIN *gg, PERFILE *p, short m) 		/* [FUN] */ 

{
  short nm, i;
  short dx, dy;
  short j, mn, mx, scalefactor;
  long ng;
  
  cleardwin(gg, scatd, PLANE_DWIN);
  nm = nof_morphologies(p);

  fprintf(ecgnosislog, "Number of morphologies = %d\n", nm);
  fflush(ecgnosislog);

  for(i=m; i<nm && i<m+8; i++) {

    /* compute morpho position on the screen */

    dx = 5 + ((i-m) % 4) * 100;
    dy = 195 + ((i-m) / 4) * 200;

    /* draw the frame */

    gsetcolor(gg, scatd, 2);
    
    gmoveto(gg, scatd, dx, dy);
    glineto(gg, scatd, dx+90, dy);
    glineto(gg, scatd, dx+90, dy-190);
    glineto(gg, scatd, dx, dy-190);
    glineto(gg, scatd, dx, dy);

    /* draw the number of gliks with given morpho */

    ng = nof_gliks_with_morph(p, i);
    if(ng<10000)
      sprintf(mdtext, "%4ld", ng);
    else
      sprintf(mdtext, "%3ldK", ng / 1000);
    gtext(gg, scatd, dx+4, dy-18, mdtext);
    sprintf(mdtext, "%4.1f%%", (float)ng * 100 / total_nof_gliks);
    gtext(gg, scatd, dx+40, dy-18, mdtext);

    /* display the class and the superclass */

    sprintf(sclasses[i-m], "%d", supclass_of(p, i));
    deftextpad(gg, scatd, dx+8, dy-40, 2, 1, 2, sclasses[i-m], 'A'+i-m);
    defbutton(gg, scatd, dx+35, dy-60, 10, 12, " ", 'M'+i-m);
    sprintf(mdtext, "#%d", i);
    gtext(gg, scatd, dx+8, dy-60, mdtext);
    gtext(gg, scatd, dx+50, dy-60, gtype_names[type_of_morph(p, i)]);

    /* draw a morphology example */

    sample_morphology(p, i, mbuf);
    mn = 30000;
    mx = -30000;
    for(j=0; j<60; j++) {   /* magic 60 */
      if(mbuf[j]<mn)
	mn = mbuf[j];
      if(mbuf[j]>mx)
	mx = mbuf[j];
    }
    scalefactor = 1;        /* number to divide with */
    if(mx - mn > 100)       /* rarely happens with 8 bit signals */
      scalefactor = (mx - mn) / 100;
    gmoveto(gg, scatd, dx+15, dy - 80 - (mbuf[0]-mn) / scalefactor);
    for(j=0; j<60; j++) {   /* as magic 60 as above */
      glineto(gg, scatd, dx+16+j, dy - 80 - (mbuf[j]-mn) / scalefactor);
    }
  }
}


/*--
  \end{code}

  \funparagraph{morph\_interraction}  % The name of the function(s)
  \ImplementationNotes{
  % How implementation of the function is done: algorithm---if special,
  % implementation decisions, discarded implementation alternatives
  }
  \Portability{ % What is assumed about the operating environment
  }
  \Changes{ % that are anticipated, if special provision was made for them
  }

  \begin{code}
  --*/

  
void morph_interraction(GWIN *gg, PERFILE *p) 			/* [FUN] */ 

{
  short k, sc;
  short c, crtmorph;

  make_morph_screen(gg);
  total_nof_morphs = nof_morphologies(p);
  total_nof_gliks = 0;
  for(k=0; k<total_nof_morphs; k++)
    total_nof_gliks += nof_gliks_with_morph(p, k);

  gtext(gg, fulld, 420, 305, "Complexes:");
  sprintf(mdtext, "%ld", total_nof_gliks);
  gtext(gg, fulld, 420, 325, mdtext);

  gtext(gg, fulld, 420, 345, "Morpho's:");
  sprintf(mdtext, "%ld", total_nof_morphs);
  gtext(gg, fulld, 420, 365, mdtext);

  crtmorph = 0;
  do {
    morph_display(gg, p, crtmorph);
    c = gwinteract(gg);
    switch(c) {
    case 'a':
      if(crtmorph>8)
	crtmorph -= 8;
      else
	crtmorph = 0;
      break;
    case 'f':
      if(crtmorph+8 <= total_nof_morphs)
	crtmorph += 8;
      else {
	crtmorph = total_nof_morphs - 8;
	if(crtmorph<0)
	  crtmorph = 0;
      }
      break;
    case 'A':
    case 'B':
    case 'C':
    case 'D':
    case 'E':
    case 'F':
    case 'G':
    case 'H':
      gwaccept(gg);
      sc = atoi(sclasses[c-'A']);
      if(sc>=0 && sc<total_nof_morphs) {
	set_supclass_of(p, crtmorph + (c - 'A'), sc);
      }
      break;
    case 'M':
    case 'N':
    case 'O':
    case 'P':
    case 'Q':
    case 'R':
    case 'S':
    case 'T':
      sc = type_of_morph(p, crtmorph + c - 'M');
      sc ++;
      if(sc>ATIP)
	sc = UNKN;
      set_type_of_morph(p, crtmorph + c - 'M', sc);
      break;
    default:
      return ;
    }
  } while(1);
  return ;
}



/*--
  \end{code}

  % MOVE THIS COMMENT TO THE END OF THE FILE
  \endoffile

  --*/

