/*--

  \FileTitle{
  {\tt egviewrc.c} --- The Electrocardiognosis recordings management
  }
  \CopyrightNotice{

    Copyright (c) 1991--1996 Alexandru Dan Corlan MD (http://dan.corlan.net)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

  }
  \vskip 1in

  \section{The control file}

  \begin{code}
  --*/

#include <stdio.h>
#include <malloc.h>

#include "ilsix.h"

#ifdef _MSDOS_
#include <unistd.h>
#else
#include <unistd.h>
#endif

#include "gliks.h"
#include "ecgnosis.h"


#ifdef WIPORT2X
#include "wiport2x.h"
#else
#include <sys/timeb.h>
#include <time.h>
#include <wiport2.h>
#include <wiptime.h>
#endif



/*--
\end{code}

\funparagraph{time\_stamp\_ctrl\_file}

\PurposeReturn{
Inserts in the ctrl file a notice stating that the file
was opened at the given moment. 
}

\begin{code}
--*/

/*--
\end{code}

\funparagraph{time\_stamp\_ctrl\_file}

\begin{code}
--*/


struct timeb tm;
char tb[64];
time_t tt;

extern cell the_current_user();

void time_stamp_ctrl_file()

{
  cell c;
  cell elog;


  ftime(&tm);
  tt = tm.time;
  c = cons(cl_make_symbol(":application-id"),
	   cons(cl_make_symbol("Electrocardiognosis-0-9-2"), NIL));
  c = cons(cl_make_symbol(":application-name"),
	   cons(cl_make_string("Electrocardiognosis"), c));
  c = cons(cl_make_symbol(":version"),
	   cons(cons(cl_make_integer(0),
		     cons(cl_make_integer(9),
			  cons(cl_make_integer(2), NIL))), c));
  c = cons(cl_make_symbol("file-open-stamp"),
           cons(cl_make_symbol(":open-moment"),
		cons(cl_make_string(ctime(&tt)),
		     cons(cl_make_symbol(":opened-by"),
			  cons(the_current_user(), c)))));
  add_assertion(c);

/*
  elog = strconnect(CHAR_FILE, ecgnosislog, OUTPUT);
  fprintf(ecgnosislog, "The c variable: ");
  pprint(c, elog);
  fflush(ecgnosislog);
  disconnect(elog); */
}

/*--
\end{code}

\funparagraph{open\_template}

\PurposeReturn{
Attempts to open an already existing GWO file and the
associated control file (which it creates if it is not present).
Returns a {\tt PERFILE *} structure
if success (and both the gwo and the control file
will be opened) and NULL if failure (neither ctrl nor
gwo file exists).
}
\Preconditions{
The \functiontag{generate\_names\_from} must be called before
this function in order to produce the actual control file and
recording file names from the desired template.
}

\begin{code}
--*/


/*--
\end{code}

\funparagraph{open\_template}  % The name of the function(s)
\ImplementationNotes{ 
}
\Portability{ % What is assumed about the operating environment
}
\Changes{ % that are anticipated, if special provision was made for them
}

\begin{code}
--*/

extern char *the_control_file_name();

PERFILE *open_template(short c_type)

{
  PERFILE *thep;

  if(!access(the_ecg_file_name(), F_OK)) {
    if(!access(the_ecg_file_name(), R_OK)) {
      /* all right, the gwo file may be accessed */
      if(!access(the_control_file_name(), F_OK) &&
	 !access(the_control_file_name(), W_OK)) {
	
	/* potential error? what to do if it exists but
	   write permissions are invalidated? */
	
	thep = rec_file_open(NULL, the_ecg_file_name(), c_type);
	open_ctrl_file(the_control_file_name());
	time_stamp_ctrl_file();
	
	/* Some ctrl file processing, such as parameter extraction
	   must probably be done here */
	
	return thep;
      }
      else {
	/* but the ctrl_file can't be, so create it*/
	thep = rec_file_open(NULL, the_ecg_file_name(), c_type);
	init_ctrl_file(the_control_file_name(), the_ecg_file_name(), 0);
	  /* it only works with ECGNOSIS-1 files for the moment */
	time_stamp_ctrl_file();

	return thep;
      }
    }
  }
  else {
    /* change here to accomodate the following situation:
       the ctrl file exists, but it has a different name
       from that of the gwo file, thus the gwo file specified
       inside the access file should be used
       */
    return NULL;
  }
}

extern char *bylanguage(char *, char *, char *, char *, char *, char *);

void right_rec_buttons(GWIN *gg)

{

  cleardwin(gg, rjumps, PLANE_DWIN);
  
  defbutton(gg, rjumps, 18,  23, 90, 20,
	    bylanguage("Open", "Deschide", "", "", "", ""), 1);
  defbutton(gg, rjumps, 18,  53, 90, 20,
	    bylanguage("Transfer", "Transfera", "", "", "", ""), 2);
  defbutton(gg, rjumps, 18,  83, 90, 20,
	    bylanguage("Process", "Prelucreaza", "", "", "", ""), 3);
  defbutton(gg, rjumps, 18, 113, 90, 20,
	    bylanguage("Info", "Informatii", "", "", "", ""), 4);
  defbutton(gg, rjumps, 18, 143, 90, 20,
	    bylanguage("Link to QST", "Leaga de QST", "", "", "", ""), 5);
  defbutton(gg, rjumps, 18, 173, 90, 20,
	    bylanguage("Link to RIR", "Leaga de RIR", "", "", "", ""), 6);
  defbutton(gg, rjumps, 18, 203, 90, 20,
	    bylanguage("Back", "Inapoi", "", "", "", ""), 7);
  defhbutton(gg, rjumps, 24);
}


void right_rec_info_buttons(GWIN *gg)

{

  cleardwin(gg, rjumps, PLANE_DWIN);
  
  defbutton(gg, rjumps, 18,  23, 90, 20,
	    bylanguage("Rec Info", "Inregistrare", "", "", "", ""), 1);
  defbutton(gg, rjumps, 18,  53, 90, 20,
	    bylanguage("Pat Info", "Pacient", "", "", "", ""), 2);
  defbutton(gg, rjumps, 18,  83, 90, 20,
	    bylanguage("Drugs", "Medicamente", "", "", "", ""), 3);
  defbutton(gg, rjumps, 18, 113, 90, 20,
	    bylanguage("Clinical", "Clinica", "", "", "", ""), 4);
  defbutton(gg, rjumps, 18, 143, 90, 20,
	    bylanguage("Button", "Buton", "", "", "", ""), 5);
  defbutton(gg, rjumps, 18, 173, 90, 20,
	    bylanguage("Back", "Inapoi", "", "", "", ""), 6);
  defhbutton(gg, rjumps, 24);
}

rec_open_interaction(GWIN *gg)

{

}

rec_transfer_interaction(GWIN *gg)

{

}


rec_process_interaction(GWIN *gg)

{

}

rec_link_qst_interaction(GWIN *gg)

{

}

rec_link_rir_interaction(GWIN *gg)

{

}

rec_info_interaction(GWIN *gg)

{
  short c;

  right_rec_info_buttons(gg);
  c = gwinteract(gg);
  switch(c) {
  case 1:
    rec_info_edit(gg);
    break;
  case 2:
    pat_info_edit(gg);
    break;
  case 3:
    /* drug_events_edit(GWIN *gg); */
    break;
  case 4:
    /* clinical_events_edit(GWIN *gg); */
    break;
  case 5:
    /* button_events_edit(GWIN *gg); */
    break;
  case 6:
  default:
    break;
  }
}

rec_interaction(GWIN *gg)

{
  short c;

  cleardwin(gg, 0, PLANE_DWIN);
  cleardwin(gg, upd, PLANE_DWIN);
  cleardwin(gg, lod, PLANE_DWIN);
  cleardwin(gg, umctrld, PLANE_DWIN);
  cleardwin(gg, lctrld, PLANE_DWIN);
  cleardwin(gg, rjumps, PLANE_DWIN);
  cleardwin(gg, fulld, PLANE_DWIN);
  
  right_rec_buttons(gg);
  c = gwinteract(gg);
  switch(c) {
  case 1:
    rec_open_interaction(gg);
    break;
  case 2:
    rec_transfer_interaction(gg);
    break;
  case 3:
    rec_process_interaction(gg);
    break;
  case 4:
    rec_info_interaction(gg);
    break;
  case 5:
    rec_link_qst_interaction(gg);
    break;
  case 6:
    rec_link_rir_interaction(gg);
    break;
  case 24:
    /* PRINT THIS SCREEN */
    break;
  default:
    break;
  }
}

