
/*--

  \FileTitle{
  EGVIEWRV --- Heart Rate Variability User Interface and Recording Editor
  }
  \CopyrightNotice{

    Copyright (c) 1991--1996 Alexandru Dan Corlan MD (http://dan.corlan.net)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

  }

  \begin{code}
  --*/


#include <stdio.h>   

#ifdef WIPORT2X
#include "wiport2x.h"
#else
#include <wiport2.h>
#endif


#include "gliks.h"
#include "ecgnosis.h"

short spec_color = 14;
short spec_grid_color = 6;
short spec_bound_color = 12;

extern double hrv_spectrum[];
extern double hrv_windowed[];
extern double hrv_sampled[];
extern double hrv_original[];

short meanlinef = 'm';  /* mean of the RR signal to be displayed */
short sampf = 's';      /* show the time domain signal */
short windf =  0 ;      /* show the windowed time domain signal */
short unfiltf = 'u';    /* show the unfiltered RR signal */

void time_domain_switches(GWIN *gg) {

  char c;

  enter_graphic_block();
  cleardwin(gg, rjumps, PLANE_DWIN);
  defswitch(gg, rjumps, 18, 23, 90, 20, "Sampled", &sampf, 's');
  defswitch(gg, rjumps, 18, 43, 90, 20, "Windowed", &windf, 'w');
  defswitch(gg, rjumps, 18, 63, 90, 20, "Mean Line", &meanlinef, 'm');
  defswitch(gg, rjumps, 18, 83, 90, 20, "Uncorrected", &unfiltf, 'u');
  defbutton(gg, rjumps, 18, 203, 90, 20, "Cancel", 0x01b);
  defbutton(gg, rjumps, 18, 233, 90, 20, "OK", 0x000d);
  exit_graphic_block();
  do {
     c = gwinteract(gg);
     if(c==0x000d) {
        gwaccept(gg);
        return;
     }
     if(c==0x001b)
        return;
   } while(c!=0x000d);
}
     

void right_hrv_buttons(GWIN *gg) { 				/* [FUN] */ 
  
  enter_graphic_block();
  cleardwin(gg, rjumps, PLANE_DWIN);
  defbutton(gg, rjumps, 18,  23, 90, 20, "Time Domain", 'T');
  defbutton(gg, rjumps, 18,  53, 90, 20, "Amplify", 'A');
  defbutton(gg, rjumps, 18,  83, 90, 20, "Reduce", 'R');
  defbutton(gg, rjumps, 18, 113, 90, 20, "Parameters", 'P');
  defbutton(gg, rjumps, 18, 143, 90, 20, "Store", 'S');
/*  defbutton(gg, rjumps, 18, 173, 90, 20, "Expr", 'E');*/
  defbutton(gg, rjumps, 18, 203, 90, 20, "Back", ' ');
  defbutton(gg, rjumps, 18, 233, 90, 20, "Colors", 'c');
  defhbutton(gg, rjumps, 24);
  exit_graphic_block();

}

void hrv_type_selection(GWIN *gg) {

  enter_graphic_block();
  cleardwin(gg, rjumps, PLANE_DWIN);
  defbutton(gg, rjumps, 18,  23, 90, 20, "HF/LF", 'H');
  defbutton(gg, rjumps, 18,  53, 90, 20, "VLF", 'V');
  defbutton(gg, rjumps, 18,  83, 90, 20, "ULF", 'U');
/*  defbutton(gg, rjumps, 18,  23, 90, 20, "HF/LF", 'H');*/
  defbutton(gg, rjumps, 18,  113, 90, 20, "Back", ' ');
  defhbutton(gg, rjumps, 24);
  exit_graphic_block();
  
}

void make_hrv_screen(GWIN *gg)

{

  enter_graphic_block();
  cleardwin(gg, upd, PLANE_DWIN);
  cleardwin(gg, lod, PLANE_DWIN);
  cleardwin(gg, umctrld, PLANE_DWIN);
  cleardwin(gg, lctrld, PLANE_DWIN);
  exit_graphic_block();
}

#define HRV_X_OFFS 60
#define HRV_Y_OFFS 10

double display_factor = 0.1;

extern double high_hf_boundary;
extern double low_hf_boundary;
extern double high_lf_boundary;
extern double low_lf_boundary;

extern double lf_area;
extern double hf_area;

extern double spec_resolution;

extern short ectopy_filter_error;
extern short ectopy_ratio;
extern short ectopy_filter_f;


extern double pNN50;
extern double r_MSSD;
extern double CLV;
extern double CLM;


void display_hf_lf_spectrum(GWIN *gg)

{
  short i, v;
  char buf[80];
  

  enter_graphic_block();
  gsetcolor(gg, upd, spec_grid_color);

  gmoveto(gg, upd, HRV_X_OFFS, HRV_Y_OFFS);
  glineto(gg, upd, HRV_X_OFFS+256, HRV_Y_OFFS);
  glineto(gg, upd, HRV_X_OFFS+256, HRV_Y_OFFS+200);
  glineto(gg, upd, HRV_X_OFFS, HRV_Y_OFFS+200);
  glineto(gg, upd, HRV_X_OFFS, HRV_Y_OFFS);

  for(i=0; i<256; i+=32) {
    gmoveto(gg, upd, HRV_X_OFFS + i, HRV_Y_OFFS);
    glineto(gg, upd, HRV_X_OFFS + i, HRV_Y_OFFS + 200);
  }

  gtext(gg, upd, HRV_X_OFFS +  64 - 16, HRV_Y_OFFS + 200 + 6, "0.25");
  gtext(gg, upd, HRV_X_OFFS + 128 - 16, HRV_Y_OFFS + 200 + 6, "0.50");
  gtext(gg, upd, HRV_X_OFFS + 192 - 16, HRV_Y_OFFS + 200 + 6, "0.75    Hz");

  for(i=0; i<200; i+=10) {
    gmoveto(gg, upd, HRV_X_OFFS, HRV_Y_OFFS + 200 - i);
    glineto(gg, upd, HRV_X_OFFS + 256, HRV_Y_OFFS + 200 - i);
    if (i%50 == 0) {
       gmoveto(gg, upd, HRV_X_OFFS, HRV_Y_OFFS + 200 - i + 1);
       glineto(gg, upd, HRV_X_OFFS + 256, HRV_Y_OFFS + 200 - i + 1);
       sprintf(buf, "%5.1f", i * display_factor);
       gtext(gg, upd, HRV_X_OFFS - 44, HRV_Y_OFFS + 200 - i + 4, buf);
    }
  }
  gtext(gg, upd, HRV_X_OFFS - 44, HRV_Y_OFFS + 16, "ms*ms");
  gtext(gg, upd, HRV_X_OFFS - 44, HRV_Y_OFFS + 32, " /Hz");
  gtext(gg, upd, HRV_X_OFFS - 44, HRV_Y_OFFS,     "*1000");

  gsetcolor(gg, upd, spec_color);

  for(i=0; i<128; i++) {  /* I believe that, in 1024 samples at 0.125 s */
                          /* this means from 0 to 1 Hz */
    v = (short)((hrv_spectrum[i] + hrv_spectrum[i+1] +
                    hrv_spectrum[i+2] + hrv_spectrum[i+3])/
                (400000.0 * display_factor));
    if (v>200 || v<0)
       v = 200;
    gmoveto(gg, upd, i*2 + HRV_X_OFFS, 200 + HRV_Y_OFFS);
    glineto(gg, upd, i*2 + HRV_X_OFFS, 200 + HRV_Y_OFFS - v);
    gmoveto(gg, upd, i*2 + 1 + HRV_X_OFFS, 200 + HRV_Y_OFFS);
    glineto(gg, upd, i*2 + 1 + HRV_X_OFFS, 200 + HRV_Y_OFFS - v);
    
  }

  gsetcolor(gg, upd, spec_bound_color);

  sprintf(buf, "   LF: %8.3f ms*ms", lf_area / 400.0);
  gtext(gg, upd, 270 + HRV_X_OFFS, HRV_Y_OFFS, buf);
/*  sprintf(buf, "ln LF: %8.3f      ", log(lf_area / 40.0));
  gtext(gg, upd, 270 + HRV_X_OFFS, HRV_Y_OFFS + 16, buf);*/

  sprintf(buf, "   HF: %8.3f ms*ms", hf_area / 400.0);
  gtext(gg, upd, 270 + HRV_X_OFFS, HRV_Y_OFFS + 32, buf);
/*  sprintf(buf, "ln HF: %8.3f      ", log(hf_area / 40.0));
  gtext(gg, upd, 270 + HRV_X_OFFS, HRV_Y_OFFS + 48, buf);*/

  sprintf(buf, "LF/HF: %8.3f" , lf_area / hf_area);
  gtext(gg, upd, 270 + HRV_X_OFFS, HRV_Y_OFFS + 64, buf);

  sprintf(buf, "  CLM: %8.3f ms", CLM);
  gtext(gg, upd, 270 + HRV_X_OFFS, HRV_Y_OFFS +  80, buf);
  sprintf(buf, "  CLV: %8.3f ms", CLV);
  gtext(gg, upd, 270 + HRV_X_OFFS, HRV_Y_OFFS +  96, buf);
  sprintf(buf, "pNN50: %8.3f%%", pNN50);
  gtext(gg, upd, 270 + HRV_X_OFFS, HRV_Y_OFFS + 112, buf);
  sprintf(buf, "rMSSD: %8.3f ms", r_MSSD);
  gtext(gg, upd, 270 + HRV_X_OFFS, HRV_Y_OFFS + 128, buf);
  

  if(ectopy_filter_f) {
    switch(ectopy_filter_error) {
    case 0:
      sprintf(buf, "Ectopy ratio: %d%%", ectopy_ratio);
      gtext(gg, upd, 270 + HRV_X_OFFS, HRV_Y_OFFS + 160, buf);
      break;
    case 1:
      gtext(gg, upd, 270 + HRV_X_OFFS, HRV_Y_OFFS + 160,
	    "ERR: No sust. rhythm.");
      break;
    case 2:
      gtext(gg, upd, 270 + HRV_X_OFFS, HRV_Y_OFFS + 160,
	    "ERR: Arrh. too long.");
      break;
    default: /* Can't happen */
      gtext(gg, upd, 270 + HRV_X_OFFS, HRV_Y_OFFS + 160,
	    "Obs: Ect. filt. err.");
      break;
    }
  }

  gmoveto(gg, upd, HRV_X_OFFS + 2*low_lf_boundary/spec_resolution, HRV_Y_OFFS);
  glineto(gg, upd, HRV_X_OFFS + 2*low_lf_boundary/spec_resolution, HRV_Y_OFFS + 200);
  gmoveto(gg, upd, HRV_X_OFFS + 2*high_lf_boundary/spec_resolution, HRV_Y_OFFS);
  glineto(gg, upd, HRV_X_OFFS + 2*high_lf_boundary/spec_resolution, HRV_Y_OFFS + 200);
  gmoveto(gg, upd, HRV_X_OFFS + 2*low_hf_boundary/spec_resolution, HRV_Y_OFFS);
  glineto(gg, upd, HRV_X_OFFS + 2*low_hf_boundary/spec_resolution, HRV_Y_OFFS + 200);
  gmoveto(gg, upd, HRV_X_OFFS + 2*high_hf_boundary/spec_resolution, HRV_Y_OFFS);
  glineto(gg, upd, HRV_X_OFFS + 2*high_hf_boundary/spec_resolution, HRV_Y_OFFS + 200);
  exit_graphic_block();
}

extern double vlf_area;
extern double low_vlf_boundary;
extern double high_vlf_boundary;

void display_vlf_spectrum(GWIN *gg)

{
  short i, v;
  char buf[80];

  enter_graphic_block();
  gsetcolor(gg, upd, spec_grid_color);

  gmoveto(gg, upd, HRV_X_OFFS, HRV_Y_OFFS);
  glineto(gg, upd, HRV_X_OFFS+256, HRV_Y_OFFS);
  glineto(gg, upd, HRV_X_OFFS+256, HRV_Y_OFFS+200);
  glineto(gg, upd, HRV_X_OFFS, HRV_Y_OFFS+200);
  glineto(gg, upd, HRV_X_OFFS, HRV_Y_OFFS);

  for(i=0; i<256; i+=32) {
    gmoveto(gg, upd, HRV_X_OFFS + i, HRV_Y_OFFS);
    glineto(gg, upd, HRV_X_OFFS + i, HRV_Y_OFFS + 200);
  }

  gtext(gg, upd, HRV_X_OFFS +  64 - 16, HRV_Y_OFFS + 200 + 6, "0.062");
  gtext(gg, upd, HRV_X_OFFS + 128 - 16, HRV_Y_OFFS + 200 + 6, "0.125");
  gtext(gg, upd, HRV_X_OFFS + 192 - 16, HRV_Y_OFFS + 200 + 6, "0.187   Hz");

  for(i=0; i<200; i+=10) {
    gmoveto(gg, upd, HRV_X_OFFS, HRV_Y_OFFS + 200 - i);
    glineto(gg, upd, HRV_X_OFFS + 256, HRV_Y_OFFS + 200 - i);
    if (i%50 == 0) {
       gmoveto(gg, upd, HRV_X_OFFS, HRV_Y_OFFS + 200 - i + 1);
       glineto(gg, upd, HRV_X_OFFS + 256, HRV_Y_OFFS + 200 - i + 1);
       sprintf(buf, "%5.1f", i * display_factor);
       gtext(gg, upd, HRV_X_OFFS - 44, HRV_Y_OFFS + 200 - i + 4, buf);
    }
  }
  gtext(gg, upd, HRV_X_OFFS - 44, HRV_Y_OFFS + 16, "ms*ms");
  gtext(gg, upd, HRV_X_OFFS - 44, HRV_Y_OFFS + 32, " /Hz");
  gtext(gg, upd, HRV_X_OFFS - 44, HRV_Y_OFFS,     "*1000");

  gsetcolor(gg, upd, spec_color);

  for(i=0; i<128; i++) {  /* I believe that, in 1024 samples at 0.5 s */
                          /* this means from 0 to 0.25 Hz */
    v = (short)((hrv_spectrum[i] + hrv_spectrum[i+1] +
                    hrv_spectrum[i+2] + hrv_spectrum[i+3])/
                (400000.0 * display_factor));
    if (v>200 || v<0)
       v = 200;
    gmoveto(gg, upd, i*2 + HRV_X_OFFS, 200 + HRV_Y_OFFS);
    glineto(gg, upd, i*2 + HRV_X_OFFS, 200 + HRV_Y_OFFS - v);
    gmoveto(gg, upd, i*2 + 1 + HRV_X_OFFS, 200 + HRV_Y_OFFS);
    glineto(gg, upd, i*2 + 1 + HRV_X_OFFS, 200 + HRV_Y_OFFS - v);
    
  }

  gsetcolor(gg, upd, spec_bound_color);

  sprintf(buf, "  VLF: %8.3f ms*ms", vlf_area / 400.0);
  gtext(gg, upd, 270 + HRV_X_OFFS, HRV_Y_OFFS, buf);
  sprintf(buf, "lnVLF: %8.3f      ", log(vlf_area / 40.0));
  gtext(gg, upd, 270 + HRV_X_OFFS, HRV_Y_OFFS+16, buf);


  sprintf(buf, "  CLM: %8.3f ms", CLM);
  gtext(gg, upd, 270 + HRV_X_OFFS, HRV_Y_OFFS +  80, buf);
  sprintf(buf, "  CLV: %8.3f ms", CLV);
  gtext(gg, upd, 270 + HRV_X_OFFS, HRV_Y_OFFS +  96, buf);
  sprintf(buf, "pNN50: %8.3f%%", pNN50);
  gtext(gg, upd, 270 + HRV_X_OFFS, HRV_Y_OFFS + 112, buf);
  sprintf(buf, "rMSSD: %8.3f ms", r_MSSD);
  gtext(gg, upd, 270 + HRV_X_OFFS, HRV_Y_OFFS + 128, buf);
  
  if(ectopy_filter_f) {
    switch(ectopy_filter_error) {
    case 0:
      sprintf(buf, "Ectopy ratio: %d%%", ectopy_ratio);
      gtext(gg, upd, 270 + HRV_X_OFFS, HRV_Y_OFFS + 160, buf);
      break;
    case 1:
      gtext(gg, upd, 270 + HRV_X_OFFS, HRV_Y_OFFS + 160,
	    "ERR: No sust. rhythm.");
      break;
    case 2:
      gtext(gg, upd, 270 + HRV_X_OFFS, HRV_Y_OFFS + 160,
	    "ERR: Arrh. too long.");
      break;
    default: /* Can't happen */
      gtext(gg, upd, 270 + HRV_X_OFFS, HRV_Y_OFFS + 160,
	    "Obs: Ect. filt. err.");
      break;
    }
  }

  gmoveto(gg, upd, HRV_X_OFFS + 2*low_vlf_boundary/spec_resolution, HRV_Y_OFFS);
  glineto(gg, upd, HRV_X_OFFS + 2*low_vlf_boundary/spec_resolution, HRV_Y_OFFS + 200);
  gmoveto(gg, upd, HRV_X_OFFS + 2*high_vlf_boundary/spec_resolution, HRV_Y_OFFS);
  glineto(gg, upd, HRV_X_OFFS + 2*high_vlf_boundary/spec_resolution, HRV_Y_OFFS + 200);
    exit_graphic_block();
}


void display_hrv_data(GWIN *gg, double *t, short n)

{
  short i;
 
  enter_graphic_block();
  gmoveto(gg, lod, 0, (short)(70 - (t[0] / 100.0)));
  for(i=0; i<n; i++)
    glineto(gg, lod, i/2, (short)(70 - (t[i] / 100.0)));
  if(meanlinef) {
    gmoveto(gg, lod, 0, 70);
    glineto(gg, lod, n/2, 70);
  }
  exit_graphic_block();

}

void hrv_interaction(GWIN *gg, PERFILE *p, long m0)

{
  short c;

  make_hrv_screen(gg);
  hrv_type_selection(gg);

  c = gwinteract(gg);
  switch(c) {
  case 'H':
    right_hrv_buttons(gg);
    compute_hf_lf_spectrum(p, m0);  /* 1024 samples of 125ms each */
    do {
      cleardwin(gg, upd, PLANE_DWIN);
      cleardwin(gg, lod, PLANE_DWIN);
      display_hf_lf_spectrum(gg);
      if(unfiltf) {
	gsetcolor(gg, lod, 2);
	display_hrv_data(gg, hrv_original, 1024);
      }
      if(sampf) {
	gsetcolor(gg, lod, 14);
	display_hrv_data(gg, hrv_sampled, 1024);
      }
      if(windf) {
	gsetcolor(gg, lod, 6);
	display_hrv_data(gg, hrv_windowed, 1024);
      }
      c = gwinteract(gg);
      switch(c) {
      case ' ':
	return;
      case 'A':
	display_factor /= 2;
	break;
      case 'R':
	display_factor *= 2;
	break;
      case 24:
	wi_set_eps_unit(700);
	wi_print_gwin("hrvscr.eps");
	break;
      case 'T':
	time_domain_switches(gg);
	right_hrv_buttons(gg);
	break;
      default:
	break;
      }
    } while (c!=' ');
    break;  /* case 'T' */
  case 'V':
    right_hrv_buttons(gg);
    compute_vlf_spectrum(p, m0);
    do {
      cleardwin(gg, upd, PLANE_DWIN);
      cleardwin(gg, lod, PLANE_DWIN);
      display_vlf_spectrum(gg);
      if(sampf) {
	gsetcolor(gg, lod, 14);
	display_hrv_data(gg, hrv_sampled, 1024);
      }
      if(windf) {
	gsetcolor(gg, lod, 6);
	display_hrv_data(gg, hrv_windowed, 1024);
      }
      c = gwinteract(gg);
      switch(c) {
      case ' ':
	return;
      case 'A':
	display_factor /= 2;
	break;
      case 'R':
	display_factor *= 2;
	break;
      case 24:
	wi_set_eps_unit(700);
	wi_print_gwin("hrvscr.eps");
	break;
      case 'T':
	time_domain_switches(gg);
	right_hrv_buttons(gg);
	break;
      default:
	break;
      }
    } while (c!=' ');
    break;  /* case 'V' */
    
  }
}

/*--
  \end{code}

  % MOVE THIS COMMENT TO THE END OF THE FILE
  \endoffile

  --*/

