
/*--

  \FileTitle{
  {\tt egviewsc.c} --- Electrocardiognosis, scatter diagrams interface
  }
  \CopyrightNotice{

    Copyright (c) 1991--1996 Alexandru Dan Corlan MD (http://dan.corlan.net)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

  }

  \subsection{Scatter Diagrams}

  \begin{code}
  --*/


#include <stdio.h>   

#ifdef WIPORT2X
#include "wiport2x.h"
#else
#include <wiport2.h>
#endif


#include "gliks.h"
#include "ecgnosis.h"


/*--
  \end{code}

  \dataparagraph{Display and interraction parameters}

  \begin{code}
  --*/


#define MAX_N_SCAT_COLORS 256 					/* [DEF] */ 

short n_scat_colors=4; 						/* [VAR] */ 
short n_scat_c[MAX_N_SCAT_COLORS] = { 3, 6, 9, 13 }; 		/* [VAR] */ 
short n_scat_l[MAX_N_SCAT_COLORS]; 				/* [VAR] */ 
/* limits for the use of each color */

short nof_scat_divs = 4; 					/* [VAR] */ 
short crt_scat_div  = -1;  /* This means ``data not shown'' */ 	/* [VAR] */ 

char scamtbuf[12] = "1";         /* mediation factor */ 	/* [VAR] */ 
char fromtbuf[12] = "00:00:05";  /* start diagram from */ 	/* [VAR] */ 
char totbuf[12] = "01:00:05"; 					/* [VAR] */ 
long scaton = 50000; 						/* [VAR] */ 
long scatoff = 36050000; 					/* [VAR] */ 

/* \variabletag{ccsperpixel} = temporar size of the ``scattergram
   position indicator'' at the lower side of the scattergram.
   */

long ccsperpixel = 1; 						/* [VAR] */ 
long rfd; 							/* [VAR] */ 

extern long gtimeparse(char *); 				/* [FUN] */ 

/*--
  \end{code}

  \funparagraph{right\_scat\_buttons}

  \begin{code}
  --*/

void right_scat_buttons(GWIN *gg) { 				/* [FUN] */ 
  
  cleardwin(gg, rjumps, PLANE_DWIN);
  defbutton(gg, rjumps, 18,  23, 90, 20, "Display", 'D');
  defbutton(gg, rjumps, 18,  53, 90, 20, "Report", 'R');
  /*  defbutton(gg, rjumps, 18,  83, 90, 20, "Mediate", 'M');
  defbutton(gg, rjumps, 18, 113, 90, 20, "Spread", 'S');
  defbutton(gg, rjumps, 18, 143, 90, 20, "3D-view", '3');
  defbutton(gg, rjumps, 18, 173, 90, 20, "Expr", 'E');*/
  defbutton(gg, rjumps, 18, 203, 90, 20, "Back", ' ');
  defbutton(gg, rjumps, 18, 233, 90, 20, "Colors", 'c');
}


/*--
  \end{code}

  \funparagraph{scat\_display\_buttons}

  \begin{code}
  --*/


void scat_display_buttons(GWIN *gg) { 				/* [FUN] */ 
  
  cleardwin(gg, rjumps, PLANE_DWIN);
  defbutton(gg, rjumps, 18,  23, 90, 20, "HorBdry", 'H');
  defbutton(gg, rjumps, 18,  53, 90, 20, "VerBdry", 'V');
  defbutton(gg, rjumps, 18,  83, 90, 20, "TwoBdry", 't');
  defbutton(gg, rjumps, 18, 113, 90, 20, "TMarks", 'M');
/*  defbutton(gg, rjumps, 18, 143, 90, 20, "GMarks", 'G'); */
  defbutton(gg, rjumps, 18, 173, 90, 20, "Marks Ovr", 'O');
  defbutton(gg, rjumps, 18, 203, 90, 20, "Marks Und", 'U');
  defbutton(gg, rjumps, 18, 233, 90, 20, "Back", ' ');
  defhbutton(gg, rjumps, 24);
}


/*--
  \end{code}

  \funparagraph{scat\_marks}

  \begin{code}
  --*/


short tmark_npoints  = 1;
short tmark_surface  = 0;
short tmark_xmed     = 0;
short tmark_ymed     = 0;
short tmark_s_i      = 0;
short tmark_s_ii     = 0;
short tmark_oval     = 0;
short tmark_corel    = 8;
short tmark_limsup   = 0;
short tmark_liminf   = 0;
short tmark_limleft  = 0;
short tmark_limright = 0;

void scat_tmarks(GWIN *gg) 						/* [FUN] */ 

{
  cleardwin(gg, rjumps, PLANE_DWIN);
  
  defswitch(gg, rjumps, 18,  23, 90, 20, "NPoints",   &tmark_npoints,   1);
  defswitch(gg, rjumps, 18,  43, 90, 20, "Surface",   &tmark_surface,   2);
  defswitch(gg, rjumps, 18,  63, 90, 20, "Mean X",    &tmark_xmed,      3);
  defswitch(gg, rjumps, 18,  83, 90, 20, "Mean Y",    &tmark_ymed,      4);
  defswitch(gg, rjumps, 18, 103, 90, 20, "Stdev-I",   &tmark_s_i,       5);
  defswitch(gg, rjumps, 18, 123, 90, 20, "Stdev-II",  &tmark_s_ii,      6);
  defswitch(gg, rjumps, 18, 143, 90, 20, "Ovality",   &tmark_oval,      7);
  defswitch(gg, rjumps, 18, 163, 90, 20, "Corelat",   &tmark_corel,     8);
  defswitch(gg, rjumps, 18, 183, 90, 20, "Lim Sup",   &tmark_limsup,    9);
  defswitch(gg, rjumps, 18, 203, 90, 20, "Lim Inf",   &tmark_liminf,   10);
  defswitch(gg, rjumps, 18, 223, 90, 20, "Lim Left",  &tmark_limleft,  11);
  defswitch(gg, rjumps, 18, 243, 90, 20, "Lim Right", &tmark_limright, 12);
  defbutton(gg, rjumps, 18, 273, 90, 20, "Cancel", ' ');
  defbutton(gg, rjumps, 18, 303, 90, 20, "OK", 'M');
}


/*--
  \end{code}

  \funparagraph{make\_scat\_screen}

  \begin{code}
  --*/

make_scat_screen(GWIN *gg) 					/* [FUN] */ 

{
  cleardwin(gg, 0, PLANE_DWIN);
  cleardwin(gg, upd, PLANE_DWIN);
  cleardwin(gg, lod, PLANE_DWIN);
  cleardwin(gg, umctrld, PLANE_DWIN);
  cleardwin(gg, lctrld, PLANE_DWIN);
  cleardwin(gg, fulld, PLANE_DWIN);

  right_scat_buttons(gg);

/*  setspotshot(gg, scatd, '!'); */

  gsetcolor(gg, fulld, 14);

  defbutton(gg, fulld, 445, 30, 20, 20, "^", '^');
  defbutton(gg, fulld, 430, 60, 20, 20, "<", '<');
  defbutton(gg, fulld, 460, 60, 20, 20, ">", '>');
  defbutton(gg, fulld, 445, 90, 20, 20, "v", 'v');

  defbutton(gg, fulld, 430, 140, 20, 20, "+", '+');
  defbutton(gg, fulld, 460, 140, 20, 20, "-", '-');

  defbutton(gg, fulld, 420, 180, 12, 16, "-", 'm');
  defbutton(gg, fulld, 475, 180, 12, 16, "+", 'p');
  deftextpad(gg, fulld, 440, 180, 2, 1, 2, scamtbuf, 'g');

  gtext(gg, fulld, 412, 290, "From:");
  deftextpad(gg, fulld, 412, 310, 9, 1, 9, fromtbuf, 'F');

  gtext(gg, fulld, 412, 340, "To:");
  deftextpad(gg, fulld, 412, 360, 9, 1, 9, totbuf, 'T');
  
  defbutton(gg, fulld, 420, 408, 30, 16, "div", '/');
  defbutton(gg, fulld, 460, 408, 30, 16, "cat", '*');
  defbutton(gg, fulld, 420, 385, 30, 16, "<<<", 'a');
  defbutton(gg, fulld, 460, 385, 30, 16, ">>>", 'f');
}


/*--
  \end{code}

  \funparagraph{show\_scat\_grid}

  \begin{code}
  --*/


void show_scat_grid(GWIN *gg, short xmin, short ymin, short step)  /* [FUN] */ 

{ 

  short i, j;
  static char bu[10];

  cleardwin(gg, scatd, PLANE_DWIN);
  enter_graphic_block();
  gsetcolor(gg, scatd, 5);
  gmoveto(gg, scatd, 0, 0);
  glineto(gg, scatd, 0, 399);
  glineto(gg, scatd, 399, 399);
  glineto(gg, scatd, 399, 0);
  glineto(gg, scatd, 0, 0);
  
  gmoveto(gg, scatd, 1, 1);
  glineto(gg, scatd, 1, 398);
  glineto(gg, scatd, 398, 398);
  glineto(gg, scatd, 398, 1);
  glineto(gg, scatd, 1, 1);
  
  for(i=50; i<400; i+=50) {
    gmoveto(gg, scatd, i, 0);
    glineto(gg, scatd, i, 399);
    j = xmin + step * (i / 50);
    if(j % 10)
      sprintf(bu, "%03d.%d", j / 10, j % 10);
    else
      sprintf(bu, "%03d", j / 10);
    gtext(gg, scatd, i+3, 3, bu);
    if(i==scat_dim) {
      gmoveto(gg, scatd, i+1, 0);
      glineto(gg, scatd, i+1, 399);
    }
    
    gmoveto(gg, scatd, 0, 400 - i);
    glineto(gg, scatd, 399, 400 - i);
    if(i==scat_dim) {
      gmoveto(gg, scatd, 0, 399 - i);
      glineto(gg, scatd, 399, 399 - i);
    }
    j = ymin + step * (i / 50);
    if(j % 10)
      sprintf(bu, "%03d.%d", j / 10, j % 10);
    else
      sprintf(bu, "%03d", j / 10);
    gtext(gg, scatd, 3, 406 - i, bu);
  }
  exit_graphic_block();

}



/*--
  \end{code}

  \funparagraph{show\_scat\_divs}

  \begin{code}
  --*/


void show_scat_divs(GWIN *gg) { 				/* [FUN] */ 

  short i, j;
  
  enter_graphic_block();
  cleardwin(gg, scatpd, PLANE_DWIN);
  gsetcolor(gg, scatpd, 3);
  gmoveto(gg, scatpd, 0, 0);
  glineto(gg, scatpd, 400, 0);
  glineto(gg, scatpd, 400, 16);
  glineto(gg, scatpd, 0, 16);
  glineto(gg, scatpd, 0, 0);

  for(i=scaton / ccsperpixel;
      i<scatoff / ccsperpixel;
      i++) {
    gmoveto(gg, scatpd, i, 0);
    glineto(gg, scatpd, i, 16);
  }
    exit_graphic_block();

}


void make_scat_limits() { 					/* [FUN] */ 

  short i, j;
  long m;

  m = 0;
  for(i=0; i<scat_dim; i++)
    for(j=0; j<scat_dim; j++)
      if(dscat[i][j]>m)
	m = dscat[i][j];
  n_scat_l[0] = 1;
  for(i=1; i<=n_scat_colors; i++) {
    n_scat_l[i] = (i * m) / (n_scat_colors);
    if(n_scat_l[i]==0)
      n_scat_l[i] = n_scat_l[i-1] + 1;
    if(n_scat_l[i]<n_scat_l[i-1])
      n_scat_l[i] = n_scat_l[i-1] + 1;
  }
  n_scat_l[n_scat_colors]++;
}

/* The following function displays the scattergram, starting---in the
 * dscat table--- with position [i0][j0] and using a square of
 * npix pixels for each of the values in the table
 *
 * the screen is assumed to be already cleared when display starts
 */

short color_given_val(short v) { 			/* [FUN] */ 

  short i;

  for(i=0; i<n_scat_colors; i++)
    if(v>=n_scat_l[i] && v<n_scat_l[i+1])
      return n_scat_c[i];
}


/*--
  \end{code}

  \funparagraph{display\_scat}

  \begin{code}
  --*/

char smbf[80];

short ovrmode = 0;  /* i.e. the text to be placed behind */

void display_scat(GWIN *gg, short i0, short j0, short npix) { 	/* [FUN] */ 

  short i, j, k, ii, jj;
  short horpos, verpos;

  if(ovrmode>0)
    goto SCATTERGRAM;
  
 MARKINGS:

  init_sm();
  _gen_scat_measures(scat, scat_dim, 0,
		     hor_bounds_fill * ver_bounds_fill - 1);

  enter_graphic_block();
  gsetcolor(gg, scatd, 13);

  for(j=0; j<npix; j++) {
    for(i=1; i<ver_bounds_fill; i++) {
      gmoveto(gg, scatd, npix * (ver_bounds[i] - i0) + j,   0);
      glineto(gg, scatd, npix * (ver_bounds[i] - i0) + j, 400);
    }

    for(i=1; i<hor_bounds_fill; i++) {
      gmoveto(gg, scatd,   0, 400 - npix * (hor_bounds[i] - i0) + j);
      glineto(gg, scatd, 400, 400 - npix * (hor_bounds[i] - i0) + j);
    }
  }
  
  for(i=0; i<ver_bounds_fill * hor_bounds_fill; i++) {
    horpos = npix * (ver_bounds[ver_bounds_ix(i)]-i0) + 10;
    verpos = 400 - npix * (hor_bounds[hor_bounds_ix(i)+1]-j0) + 10;

    if(horpos > 400 || verpos > 400 || horpos < 0 || verpos < 0)
      continue;

    if(tmark_npoints) {
      sprintf(smbf, "N=%d", sm[i].volume);
      gtext(gg, scatd, horpos, verpos, smbf);
      verpos += 18;
    }
    if(tmark_surface) {
      sprintf(smbf, "S=%d", sm[i].surface);
      gtext(gg, scatd, horpos, verpos, smbf);
      verpos += 18;
    }
    if(tmark_xmed) {
      sprintf(smbf, "X=%.1f %.1f", sm[i].xmed, sm[i].xdisp);
      gtext(gg, scatd, horpos, verpos, smbf);
      verpos += 18;
    }
    if(tmark_ymed) {
      sprintf(smbf, "Y=%.1f %.1f", sm[i].ymed, sm[i].ydisp);
      gtext(gg, scatd, horpos, verpos, smbf);
      verpos += 18;
    }
    if(tmark_s_i) {
      sprintf(smbf, "S1=%.1f", sm[i].dispi);
      gtext(gg, scatd, horpos, verpos, smbf);
      verpos += 18;
    }
    if(tmark_s_ii) {
      sprintf(smbf, "S2=%.1f", sm[i].dispii);
      gtext(gg, scatd, horpos, verpos, smbf);
      verpos += 18;
    }
    if(tmark_oval) {
      sprintf(smbf, "Ov=%.3f", sm[i].dispi / sm[i].dispii);
      gtext(gg, scatd, horpos, verpos, smbf);
      verpos += 18;
    }
    if(tmark_corel) {
      sprintf(smbf, "R=%.3f", sm[i].corr);
      gtext(gg, scatd, horpos, verpos, smbf);
      verpos += 18;
    }
  }  /* for(i...) */
  exit_graphic_block();

  if(ovrmode>0)
    return;

 SCATTERGRAM:

  enter_graphic_block();
  for(i=0; i<400; i+=npix)
    for(j=0; j<400; j+=npix) {
      ii = i/npix;
      jj = j/npix;
      if(ii+i0>=scat_dim || jj+j0>=scat_dim)
	continue;
      if(dscat[ii+i0][jj+j0]==0)
	continue;
      gsetcolor(gg, scatd, color_given_val(dscat[ii+i0][jj+j0]));
      for(k=0; k<npix; k++) {
	gmoveto(gg, scatd, i+k, 400 - j);
	glineto(gg, scatd, i+k, 400 - (j+npix-1));
      }
    }
  exit_graphic_block();
  if(ovrmode>0)
    goto MARKINGS;
}



/*--
  \end{code}

  \funparagraph{}

  \begin{code}
  --*/

void scat_display_interact(GWIN *gg)

{
  char c;

  scat_display_buttons(gg);
  setspotshot(gg, fulld, 20);
  c = gwinteract(gg);
  do {
    switch(c) {
    case 'O':
      ovrmode = 1;
      return;
    case 'U':
      ovrmode = 0;
      return;
    case 'H':
      c = gwinteract(gg);
      if(c==20)
	add_hor_boundary(200 - (yspotshot / 2));
      return;
    case 'V':
      c = gwinteract(gg);
      if(c==20)
	add_ver_boundary(xspotshot / 2);
      return;
    case 't':
      c = gwinteract(gg);
      if(c==20) {
	add_hor_boundary(200 - (yspotshot / 2));
	add_ver_boundary(xspotshot / 2);
      }
      return;
    case 24:
	wi_set_eps_unit(700);
	wi_print_gwin("poincscr.eps");
	return;
    case 'M':
      scat_tmarks(gg);
      do {
	c = gwinteract(gg);
      } while (c!=' ' && c!='M');
      if(c=='M')
	gwaccept(gg);
      return;
    default:
      return;
    }  /* switch(c) */
  } while(1);
}

/*--
  \end{code}

  \funparagraph{scat\_interraction}

  \begin{code}
  --*/

  
void scat_interraction(GWIN *gg, PERFILE *p) 			/* [FUN] */ 

{
  short i, j;
  short segwid;
  char cc;
  long delta_scat, tryval;
  short med = 1;
  short fact = 1;
  short x0=0;
  short y0=0;

  setresprefix("ScatScreen-");
  res2color();
 
  rfd = rec_file_duration(p);
  ccsperpixel = rfd/400;
  if(rfd<144000000)
    scatoff = rfd / 4;
  make_scat_screen(gg);
  show_scat_divs(gg);
  if(!alloc_scats(scat_dim))
    return;

  clear_scat_bounds();

  add_to_scat(p, scaton, scatoff);

  do {
    fprintf(ecgnosislog, "Before scat_mediate\n");
    fflush(ecgnosislog);
    scat_mediate(med);
    make_scat_limits();
    show_scat_grid(gg, 10*x0, 10*y0, 250 / fact);
    display_scat(gg, x0, y0, 2*fact);

    cc =  gwinteract(gg);
    delta_scat = scatoff - scaton;
    switch(cc) {
    case 'a':
      scatoff -= delta_scat;
      scaton -= delta_scat;
      if(scaton < 50000) {
	scaton = 50000;
	scatoff = scaton + delta_scat;
      }
      show_scat_divs(gg);
      show_scat_grid(gg, 10*x0, 10*y0, 250 / fact);
      clear_scat();
      add_to_scat(p, scaton, scatoff);
      break;
    case 'f':
      scaton += delta_scat;
      scatoff += delta_scat;
      if(scatoff >= rfd-50000) {
	scatoff = rfd - 50000;
	scaton = scatoff - delta_scat;
      }
      show_scat_divs(gg);
      show_scat_grid(gg, 10*x0, 10*y0, 250 / fact);
      clear_scat();
      add_to_scat(p, scaton, scatoff);
      break;
    case 'F':
      gwaccept(gg);
      tryval = 50 * gtimeparse(fromtbuf);
      if(tryval<50000)
	tryval = 50000;
      if(tryval>=scatoff)
	tryval = scaton;
      scaton = tryval;
      show_scat_divs(gg);
      show_scat_grid(gg, 10*x0, 10*y0, 250 / fact);
      clear_scat();
      add_to_scat(p, scaton, scatoff);
      break;
    case 'T':
      gwaccept(gg);
      tryval = 50 * gtimeparse(totbuf);
      if(tryval>rfd-50000)
	tryval = rfd - 50000;
      if(tryval<scaton)
	tryval = scatoff;
      scatoff = tryval;
      show_scat_divs(gg);
      show_scat_grid(gg, 10*x0, 10*y0, 250 / fact);
      clear_scat();
      add_to_scat(p, scaton, scatoff);
      break;
    case 'D':
      scat_display_interact(gg);
      make_scat_screen(gg);
      show_scat_divs(gg);
      break;
    case 'R':
      put_scat_to_latex_list();
      make_scat_screen(gg);
      show_scat_divs(gg);
      break;
    case '/':
      delta_scat = (3 * delta_scat) / 4;
      scatoff = scaton + delta_scat;
      show_scat_divs(gg);
      show_scat_grid(gg, 10*x0, 10*y0, 250 / fact);
      clear_scat();
      add_to_scat(p, scaton, scatoff);
      break;
    case '*':
      delta_scat = (delta_scat * 4) / 3;
      scatoff = delta_scat + scaton;
      if(scatoff > rfd - 50000) {
	delta_scat = rfd - 50000 - scaton;
	scatoff = scaton + delta_scat;
      }
      show_scat_divs(gg);
      show_scat_grid(gg, 10*x0, 10*y0, 250 / fact);
      clear_scat();
      add_to_scat(p, scaton, scatoff);
      break;
    case 'c':
      cleardwin(gg, rjumps, PLANE_DWIN);
      adjust_colors(gg, "ScatScreen-");
      right_scat_buttons(gg);
      break;
    case '+':
      fact++;
      if(fact>10)
	fact = 10;
      break;
    case '-':
      if(fact>1)
	fact--;
      break;
    case '^':
      if(y0+10+200/fact > 200)
	y0 = 200 - 200/fact;
      else
	y0 += 10;
      break;
    case 'v':
      if(y0-10<0)
	y0 = 0;
      else
	y0-=10;
      break;
    case '>':
      if(x0+10+200/fact > 200)
	x0 = 200 - 200/fact;
      else
	x0 += 10;
      break;
    case '<':
      if(x0-10<0)
	x0 = 0;
      else
	x0-=10;
      break;
    case 'm':
      med--;
      if(med<1)
	med = 1;
      goto deomed;
    case 'p':
      med++;
      if(med>20)
	med = 20;
      goto deomed;
    case 'g':
      gwaccept(gg);
      med = atoi(scamtbuf);
      if(med<1 || med>20)
	med = 1;
    deomed:
      sprintf(scamtbuf, "%d", med);
      gset_tp_contents(gg, fulld, 'g', scamtbuf);
      break;
    default:
      return;
    }
    sprintf(fromtbuf, "%02ld:%02ld:%02ld", 
	    scaton / 36000000,
	    (scaton % 36000000) / 600000,
	    (scaton % 600000) / 10000);
    gset_tp_contents(gg, fulld, 'F', fromtbuf);
    sprintf(totbuf, "%02ld:%02ld:%02ld", 
	    scatoff / 36000000,
	    (scatoff % 36000000) / 600000,
	    (scatoff % 600000) / 10000);
    gset_tp_contents(gg, fulld, 'T', totbuf);
  } while(1);
}



/*--
  \end{code}

  \endoffile

  --*/

