 
/*--

  \FileTitle{
  {\tt egviewst.c} --- Electrocardiognosis viewer of strips and time diagrams
  }
  \CopyrightNotice{

    Copyright (c) 1991--1996 Alexandru Dan Corlan MD (http://dan.corlan.net)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

  }

  \subsection{Strips and time diagrams}

  \begin{code}
  --*/

#include <stdio.h>

#ifdef WIPORT2X
#include "wiport2x.h"
#else
#include <wiport2.h>
#endif

#include "gliks.h"
#include "ecgnosis.h"

extern char *bylanguage(char *, char *, char *, char *, char *, char *);

void special_right_buttons(GWIN *gg)

{
  cleardwin(gg, rjumps, PLANE_DWIN);
  
  defbutton(gg, rjumps, 18,  23, 90, 20, "Exp RR,QRS", 1);
  defbutton(gg, rjumps, 18,  53, 90, 20, bylanguage("P wave", "Unda P", "",
						    "", "", ""), 2);
  defbutton(gg, rjumps, 18, 263, 90, 20, bylanguage("Back", "Inapoi", "",
						    "", "", ""), 9);
  defhbutton(gg, rjumps, 24);

}

/* ----------------------------------------------------------------
       Exporting RR and QRS morphology data
   ---------------------------------------------------------------- */

   /* External functions and variables needed */

extern long gtimeparse(char *tb);  /* from egmain.c (returns in 5ms units) */
extern long crtoffs;               /* current offset of the ECG
                                      (in 5ms units, I guess) */
extern long crt_td_org;            /* (guess) same for td start */

/*  gset_tp_contents(gg, umctrld, 'g', timebuf); garbage */

   /* Export parameters */

char rr_qrs_fn[80] = "";
char rr_qrs_ext[80] = "";    /* fn part that can be edited */

long rr_qrs_from;          /* place in the recording to start saving from */
char rr_qrs_from_buf[20];
long rr_qrs_to;            /* place in the recording to end saving to */
char rr_qrs_to_buf[20];

short n_before = 1000;     /* number of ccs to save before every QRS ref */
char n_before_buf[20] = "100";  /* this is in ms, you know */
short n_after  = 1000;     /*   ...  after every QRS ref */
char n_after_buf[20] = "100";   /* same applies to this */

extern char *bylanguage(char *, char *, char *,
			 char *, char *, char *);

void rr_qrs_export(GWIN *gg)

{
  short c;

  cleardwin(gg, rjumps, PLANE_DWIN);

  /* it will start with 5 minutes anyway */

  rr_qrs_from = crt_td_org * 50;
  rr_qrs_to   = rr_qrs_from + 3000000; /* 300 seconds */
  sprintf(rr_qrs_from_buf, "%02ld:%02ld:%02ld", rr_qrs_from / 36000000,
          (rr_qrs_from % 36000000) / 600000, (rr_qrs_from % 600000) / 10000);
  sprintf(rr_qrs_to_buf, "%02ld:%02ld:%02ld", rr_qrs_to / 36000000,
          (rr_qrs_to % 36000000) / 600000, (rr_qrs_to % 600000) / 10000);
  

  /* now, it can be displayed */
  
  gtext(gg, rjumps, 18, 23,
  	bylanguage("File ext", "Extensie", "", "", "", ""));
  deftextpad(gg, rjumps, 18, 43, 10, 1, 10, rr_qrs_ext, 1);
  gtext(gg, rjumps, 18, 73,
	bylanguage("From", "De la", "", "", "", ""));
  deftextpad(gg, rjumps, 18, 93, 10, 1, 10, rr_qrs_from_buf, 2);
  gtext(gg, rjumps, 18, 123,
	bylanguage("To", "La", "", "", "", ""));
  deftextpad(gg, rjumps, 18, 143, 10, 1, 10, rr_qrs_from_buf, 3);

  gtext(gg, rjumps, 18, 173,
	bylanguage("Before [ms]", "Inainte [ms]", "", "", "", ""));
  deftextpad(gg, rjumps, 18, 193, 10, 1, 10, n_before_buf, 4);
  gtext(gg, rjumps, 18, 223,
	bylanguage("After [ms]", "Dupa [ms]", "", "", "", ""));
  deftextpad(gg, rjumps, 18, 243, 10, 1, 10, rr_qrs_from_buf, 5);

  defbutton(gg, rjumps, 18, 273, 90, 20, "Go", 8);
  defbutton(gg, rjumps, 18, 303, 90, 20, "Back", 9);
  defhbutton(gg, rjumps, 24);

  while((c=gwinteract(gg))<1 && c>9 && c!=24);
  switch(c) {
  case 24:
    wi_set_eps_unit(700);
    wi_print_gwin("specmenu.eps");
    break;
  case  1:
    gwaccept(gg);  /* this must have transfered the extention into
                    * rr_qrs_ext */

    break;
  case 9:
    break;
  default:
    break;
  }
  
}


/* ----------------------------------------------------------------
       P wave analysis
   ---------------------------------------------------------------- */

void p_wave_right_buttons(GWIN *gg)

{
  cleardwin(gg, rjumps, PLANE_DWIN);
  
  defbutton(gg, rjumps, 18,  23, 90, 20, bylanguage("SKIP", "OMITE", "",
						    "", "", ""), 1);
  defbutton(gg, rjumps, 18,  53, 90, 20, bylanguage("ADD", "ADAUGA", "",
						    "", "", ""), 2);
  defbutton(gg, rjumps, 18,  83, 90, 20, bylanguage("Fiducial <<",
						    "Fiducial <<", "",
						    "", "", ""), 3);
  defbutton(gg, rjumps, 18, 113, 90, 20, bylanguage("Fiducial >>",
						    "Fiducial >>", "",
						    "", "", ""), 4);
  defbutton(gg, rjumps, 18, 263, 90, 20, bylanguage("Done", "Gata", "",
						    "", "", ""), 9);
  defhbutton(gg, rjumps, 24);

}


extern long crtoffs;  /* the current offset of the ECG */

long pvbuf[100];

short ecg_lead_1[200];
short ecg_lead_2[200];

extern short lead_1;
extern short lead_2;
extern short strip_1_color;
extern short strip_2_color;
extern short markings_color;
extern short strip_grid_col;

long lead_sum_1[200];
long lead_sum_2[200];
long n_in_sum;

short gridf = 1;

char lbuf[40];

void p_wave_analysis(GWIN *gg, PERFILE *p)

{
  short n_complexes;
  short i, j, yl1, yl2, x, y;
  short inc;
  long fiduci;
  long p_offs;
  long fidu_offs;

  p_wave_right_buttons(gg);

  p_offs = crtoffs * 50;
  fidu_offs = 0;
  n_in_sum = 0;
  do {
    cleardwin(gg, lod, PLANE_DWIN);

    enter_graphic_block();

    if(gridf) {
      gsetcolor(gg, lod, strip_grid_col);

      gmoveto(gg, lod, 70, 0);
      glineto(gg, lod, 250, 0);
      gmoveto(gg, lod, 70, 2);
      glineto(gg, lod, 250, 2);
      gmoveto(gg, lod, 70, 127);
      glineto(gg, lod, 250, 127);
      gmoveto(gg, lod, 70, 129);
      glineto(gg, lod, 250, 129);

      for(y=8; y<128; y+=8) {
	gmoveto(gg, lod, 72, y);
	glineto(gg, lod, 248, y);
	if((y % 40)==0) {
	  gmoveto(gg, lod, 72, y+1);
	  glineto(gg, lod, 248, y+1);
	}}

      gmoveto(gg, lod, 70, 0);
      glineto(gg, lod, 70, 129);
      gmoveto(gg, lod, 72, 0);
      glineto(gg, lod, 72, 129);
      gmoveto(gg, lod, 248, 0);
      glineto(gg, lod, 248, 129);
      gmoveto(gg, lod, 250, 0);
      glineto(gg, lod, 250, 129);

      for(x=78; x<250; x+=8) {
	gmoveto(gg, lod, x, 0);
	glineto(gg, lod, x, 129);
	if ((((x - 70) / 8) % 5)==0) {
	  gmoveto(gg, lod, x+1, 0);
	  glineto(gg, lod, x+1, 129);
	}
      }

      gmoveto(gg, lod, 320, 0);
      glineto(gg, lod, 499, 0);
      gmoveto(gg, lod, 320, 2);
      glineto(gg, lod, 499, 2);
      gmoveto(gg, lod, 320, 127);
      glineto(gg, lod, 499, 127);
      gmoveto(gg, lod, 320, 129);
      glineto(gg, lod, 499, 129);


      for(y=8; y<128; y+=8) {
	gmoveto(gg, lod, 322, y);
	glineto(gg, lod, 497, y);
	if((y % 40)==0) {
	  gmoveto(gg, lod, 322, y+1);
	  glineto(gg, lod, 497, y+1);
	}}


      gmoveto(gg, lod, 320, 0);
      glineto(gg, lod, 320, 129);
      gmoveto(gg, lod, 322, 0);
      glineto(gg, lod, 322, 129);
      gmoveto(gg, lod, 497, 0);
      glineto(gg, lod, 497, 129);
      gmoveto(gg, lod, 499, 0);
      glineto(gg, lod, 499, 129);

      for(x=328; x<500; x+=8) {
	gmoveto(gg, lod, x, 0);
	glineto(gg, lod, x, 129);
	if ((((x - 320) / 8) % 5)==0) {
	  gmoveto(gg, lod, x+1, 0);
	  glineto(gg, lod, x+1, 129);
	}
      }

      gmoveto(gg, lod,  15, 80);
      glineto(gg, lod,  15, 120);
      gmoveto(gg, lod,  16, 80);
      glineto(gg, lod,  16, 120);

      for(y=80; y<120; y+=8) {
	gmoveto(gg, lod, 15, y);
	glineto(gg, lod, 21, y);
      }

      gmoveto(gg, lod, 15, 119);
      glineto(gg, lod, 55, 119);
      gmoveto(gg, lod, 15, 120);
      glineto(gg, lod, 55, 120);

      for(x=15; x<=55; x+=8) {
	gmoveto(gg, lod, x, 120);
	glineto(gg, lod, x, 114);
      }

      gtext(gg, lod, 25, 70, "0.2mV");
      gtext(gg, lod, 35, 95, "0.2s");


      gmoveto(gg, lod,  265, 80);
      glineto(gg, lod,  265, 120);
      gmoveto(gg, lod,  266, 80);
      glineto(gg, lod,  266, 120);

      for(y=80; y<120; y+=8) {
	gmoveto(gg, lod, 265, y);
	glineto(gg, lod, 271, y);
      }

      gmoveto(gg, lod,  265, 119);
      glineto(gg, lod,  305, 119);
      gmoveto(gg, lod,  265, 120);
      glineto(gg, lod,  305, 120);

      for(x=265; x<=305; x+=8) {
	gmoveto(gg, lod, x, 120);
	glineto(gg, lod, x, 114);
      }

      gtext(gg, lod, 275, 70, "1.0mV");
      gtext(gg, lod, 285, 95, "0.2s");


    }
    
    reqitab(p, p_offs, p_offs+100000);
    n_complexes = complexes_between(p, p_offs, p_offs+100000);
    /*    sprintf(lbuf, "p_offs=%d", p_offs);
    gtext(gg, lod, 20, 20, lbuf);*/
    if (n_complexes > 99)
      n_complexes = 99;
    cpars_at(p, pvbuf, CPQRSPOS, p_offs, n_complexes);
    /*    sprintf(lbuf, "pvbuf[0]=%d", pvbuf[0]);
    gtext(gg, lod, 20, 50, lbuf); */
    /*    sprintf(lbuf, "pvbuf[1]=%d", pvbuf[1]);
    gtext(gg, lod, 20, 80, lbuf); */

    fiduci = pvbuf[1];

    get_lead_data(p, ecg_lead_1, lead_1, fiduci - 4000, fiduci + 5000, 50);
    get_lead_data(p, ecg_lead_2, lead_2, fiduci - 4000, fiduci + 5000, 50);

    yl1 = 100 - ecg_lead_1[0] + 128;
    /*    sprintf(lbuf, "ecg_lead_1[0]=%d", ecg_lead_1[0]);
    gtext(gg, lod, 20, 110, lbuf); */

    gsetcolor(gg, lod, markings_color);
    gmoveto(gg, lod, 320+80+fidu_offs, 1);
    glineto(gg, lod, 320+80+fidu_offs, 180);

    gsetcolor(gg, lod, strip_1_color);
    gmoveto(gg, lod, 320, yl1);

    for(i=1; i<180; i++) {
      yl1 = 100 - ecg_lead_1[i] + 128;
      glineto(gg, lod, 320 + i, yl1);
    }
    
    yl2 = 30  - ecg_lead_2[0] + 128;
    gsetcolor(gg, lod, strip_2_color);
    gmoveto(gg, lod, 320, yl2);
    
    for(i=1; i<180; i++) {
      yl2 = 30  - ecg_lead_2[i] + 128;
      glineto(gg, lod, 320 + i, yl2);
    }

    if (n_in_sum > 0) {
      gsetcolor(gg, lod, strip_grid_col);
      sprintf(lbuf, "N=%4d", n_in_sum);
      gtext(gg, lod, 15, 20, lbuf);

      gsetcolor(gg, lod, markings_color);
      gmoveto(gg, lod, 70+80, 1);
      glineto(gg, lod, 70+80, 180);
      
      gsetcolor(gg, lod, strip_1_color);
      yl1 = 100 - (short)(5.0 * lead_sum_1[0] / (double)n_in_sum);
      gmoveto(gg, lod, 70, yl1);
      
      for(i=1; i<180; i++) {
	yl1 = 100 - (short)(5.0 * lead_sum_1[i] / (double)n_in_sum);
	glineto(gg, lod, 70 + i, yl1);
      }
      
      gsetcolor(gg, lod, strip_2_color);
      yl2 = 30  - (short)(5.0 * lead_sum_2[0] / (double)n_in_sum);
      gmoveto(gg, lod, 70, yl2);
      
      for(i=1; i<180; i++) {
	yl2 = 30  - (short)(5.0 * lead_sum_2[i] / (double)n_in_sum);
	glineto(gg, lod, 70 + i, yl2);
      }

    }
    exit_graphic_block();

    inc = gwinteract(gg);
    switch(inc) {
    case 1: /* skip */
      p_offs = fiduci + 2000; /* fiduci + 0.2s */
      break;
    case 2: /* add */
      if (n_in_sum == 0) {
	for(i=0; i<180; i++) {
	  j = i + fidu_offs;
	  if (j>=0 && j<180) {
	    lead_sum_1[i] = ecg_lead_1[j] - 128;
	    lead_sum_2[i] = ecg_lead_2[j] - 128;
	  }
	}
	n_in_sum = 1;
      }
      else {
	for(i=0; i<180; i++) {
	  j = i + fidu_offs;
	  if (j>=0 && j<180) {
	    lead_sum_1[i] += ecg_lead_1[j] - 128;
	    lead_sum_2[i] += ecg_lead_2[j] - 128;
	  }
	}
	n_in_sum ++;
	}
      p_offs = fiduci + 2000; /* fiduci + 0.2s */
      break;
    case 3: /* fiduci << */
      fidu_offs --;
      break;
    case 4: /* fiduci >> */
      fidu_offs ++;
      break;
    }
  } while(inc!=9);
}


/* ----------------------------------------------------------------
       Main special interaction function
   ---------------------------------------------------------------- */


special_interaction(GWIN *gg, PERFILE *p)

{
  int c;
  
  special_right_buttons(gg);
  while((c=gwinteract(gg))<1 && c>9 && c!=24);
  /* may escape only by pressing one of the right buttons */
  switch(c) {
  case 24:
    wi_set_eps_unit(700);
    wi_print_gwin("specmenu.eps");
    break;
  case  1:
    rr_qrs_export(gg);
    break;
  case 2:
    p_wave_analysis(gg, p);
    break;
  case 9:
    break;
  default:
    break;
  }

}

