 
/*--

  \FileTitle{
  {\tt egviewst.c} --- Electrocardiognosis viewer of strips and time diagrams
  }
  \CopyrightNotice{

    Copyright (c) 1991--1996 Alexandru Dan Corlan MD (http://dan.corlan.net)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

  }

  \subsection{Strips and time diagrams}

  \begin{code}
  --*/

#include <stdio.h>

#ifdef WIPORT2X
#include "wiport2x.h"
#else
#include <wiport2.h>
#endif

#include "ilsix.h"

#include "gliks.h"
#include "ecgnosis.h"

extern char *bylanguage(char *, char *, char *, char *, char *, char *);

char zzbuf[24] = "Hello text";                                  /* [VAR] */ 
char timebuf[16] = "00:00:00";                                  /* [VAR] */ 
short use_events_f = 0; 					/* [VAR] */ 

/* The following variables describe the sizes of various
 * markings on the screen

drawing here!

 */

#define QRSPOS_MARKING_DIST 20  /* pixels */
#define BL_MARKING_SIZE 1000    /* ccs */
#define ST_MARKING_SIZE 1000    /* ccs */
#define QRS_MARKING_HEIGHT 30   /* pixels */
#define QRS_MARKING_DEPTH  30   /* pixels */


/* The following variables describe the markings to
 * be displayed on the normal ECG strip
 * They are changed in a dialog box by strdisplay_interraction
 * and are used by the recording  display function (ecg_display).
 */

short bl_marking = 0; 						/* [VAR] */ 
short j_marking = 0; 						/* [VAR] */ 
short rr_marking = 0; 						/* [VAR] */ 
short st_marking = 0; 						/* [VAR] */ 
short qrsdet_marking = 0; 					/* [VAR] */ 
short qrsref_marking = 0; 					/* [VAR] */ 
short morph_marking = 0; 					/* [VAR] */ 
short qrslen_marking = 0; 					/* [VAR] */ 

/* Color codes to be used in strip, time-diagrams and markings display */

short strip_1_color = 13; 					/* [VAR] */ 
short strip_2_color = 2; 					/* [VAR] */ 
short strip_grid_col = 3; 					/* [VAR] */ 
short td_color = 14; 						/* [VAR] */ 
short tdplus_color = 11; 					/* [VAR] */ 
short td_grid_color = 5; 					/* [VAR] */ 
short event_mark_color = 10; 					/* [VAR] */ 
short markings_color = 9;       				/* [VAR] */ 

short progress_color = 14; 					/* [VAR] */ 

       /* shiva-visnu: make and delete dwins and buttons */

/* </code>

   \section{Utility functions}

   <code> */

extern long gtimeparse(char *tb);

/*--

\end{code}

\section{Direct recording inspection}

This part of ecgnosis implements the package that performs
direct display of pieces of the recording. It is going
to be used in only one circumstance: displaying one or two stripes
of the recording in a single dwin. Two supplementary dwins
must be provided to the package --- one for the control
buttons (that move the recording around) and one for the
status line (in which some data input is also done).

The variables that control the display are:

\begin{itemize}
\item{\tt strip_1_scale},
\item{\tt strip_2_scale} that tell the scaling of each of
  the stripes (0 being the recording size in pixels);
\item{\tt mv_show_f} which tells the existence or the nonexistence
  of the left mV indication;
\item{\tt strip_1_shift},
\item{\tt strip_2_shift} telling the distance from the middle of
  the window for the two stripes;
\item{\tt crtxdiv} telling how many samples are to be displayed in one
  pixel;
\item{\tt rflag} telling wheather the back grid is to be displayed;
\end{itemize}


The following function displays an ECG grid at |np| pixels,
given a GWIN, a dwin, an xsize and an ysize, starting
from the top left corner and for the whole 
given dimension.

\begin{code}

--*/

double strip_1_scale = 1.0;                                     /* [VAR] */ 
double strip_2_scale = 1.0;                                     /* [VAR] */ 
short mv_show_f = 0;                                            /* [VAR] */ 
short strip_1_shift = 70;                                       /* [VAR] */ 
short strip_2_shift = -70;                                      /* [VAR] */ 

short crtxdiv=4;                                                /* [VAR] */ 
long crtoffs=0;                                                 /* [VAR] */ 
short rflag = 0;                                                /* [VAR] */ 
short crtyshift = 0;                                            /* [VAR] */ 
double crtyscale=1.0;                                           /* [VAR] */ 

/************ THE ECG RECORDING DRAWING PACKAGE ****************/

void draw_ecg_grid(GWIN *g, short d,                            /* [FUN] */ 
                   short xsize, short ysize, short np)          /* [FUN] */ 

{
short x, y;
char gridbuf[30];

/* The border */

gsetcolor(g, d, strip_grid_col);

enter_graphic_block();
gmoveto(g, d, 0, 0);
glineto(g, d, xsize-1, 0);
glineto(g, d, xsize-1, ysize-1);
glineto(g, d, 0, ysize-1);
glineto(g, d, 0, 0);

/* The smallest lines */

if(np<8)
  {
  for(x=0; x<xsize-1; x+=np)
    {
    for(y=0; y<ysize-1; y+=np)
      gpoint(g, d, x, y);
    }
  }
else
  {
  for(x=0; x<xsize-1; x+=np)
  {
  gmoveto(g, d, x, 0);
  glineto(g, d, x, ysize);
  }
  for(y=0; y<ysize-1; y+=np)
  {
  gmoveto(g, d, 0, y);
  glineto(g, d, xsize, y);
  }
}

/* The larger lines*/

if(np<8)
  {
for(x=np*5; x<xsize-1; x+=np*5)
  {
  gmoveto(g, d, x, 0);
  glineto(g, d, x, ysize-1);
  }
for(y=np*5; y<ysize-1; y+=np*5)
  {
  gmoveto(g, d, 0, y);
  glineto(g, d, xsize-1, y);
  }
}
else
  {
for(x=np*5; x<xsize-1; x+=np*5)
  {
  gmoveto(g, d, x+1, 0);
  glineto(g, d, x+1, ysize-1);
  }
for(y=np*5; y<ysize-1; y+=np*5)
  {
  gmoveto(g, d, 0, y+1);
  glineto(g, d, xsize-1, y+1);
}
}

sprintf(gridbuf, "%5.2f mV/cm, %d mm/s", 2.0/crtyscale , 100 / crtxdiv);
gtext(g, d, 1, 1, gridbuf);
exit_graphic_block();
}

#define EBUFSIZE 15100

short ebuf[EBUFSIZE];  /* of which only 15000 are actually used */ /* [VAR] */ 
short sebuf[EBUFSIZE]; /* to contain the second strip is needed */ /* [VAR] */

/* This very-complicated-to-call function displays an ECG strip in the 
 * given DWIN with the required number of samples, x-compression factor, 
 * y-compression factor and so on; if necessary, it reads the strip
 * to be displayed from the file
 */


static transformation = 0;                                      /* [VAR] */ 

static settransformation(int n)                                 /* [FUN] */ 

{ 
  transformation = n; 
}

/* The following variables will contain the results of the
 * analyses to be displayed as markings
 */

#define MAX_DISP_COMPLEXES 200

long m_qrspos[MAX_DISP_COMPLEXES];
long m_bl0[MAX_DISP_COMPLEXES];
long m_bl1[MAX_DISP_COMPLEXES];
long m_st0[MAX_DISP_COMPLEXES];
long m_st1[MAX_DISP_COMPLEXES];
long m_j[MAX_DISP_COMPLEXES];
long m_morph[MAX_DISP_COMPLEXES];
long m_qrslen[MAX_DISP_COMPLEXES];

short act_disp_complexes = 0;

/* The following variables state the upper and the lower lead
 * to be displayed on the screen
 */

short lead_1 = 1;
short lead_2 = 0;

extern cell _from;  /* in egctrl.c */
extern cell slot_of(cell, cell);
extern cell all_measurements_between(long, long);

void _ecg_display(GWIN *g, short d,                             /* [FUN] */ 
                  PERFILE *e, long offset,                      /* [FUN] */ 
                  short xdiv,  double yscale,                   /* [FUN] */ 
                  short yshift, short trace, short col)         /* [FUN] */ 

{
short i;
short xs, ys, y, ysc;
long l, nos;
short r0, r1, r2, r3, r4;

cell evs;
cell ev;
long evp;
cell pvs;

char markbuf[30];

xs = dwin_x_size(g, d);
ys = dwin_y_size(g, d);

/* ecg_seek(e, offset);
ecg_read(e, ebuf, 0, xs*xdiv/2); */

l = get_lead_data(e, ebuf, trace, offset * 50, (offset + xs*xdiv/2) * 50, 50);
if(l<=0)
  exit(0);


if(transformation==1) {  /* display the first derivative */
  for(i=0; i<xs*xdiv/2-4; i++) {
    ebuf[i] = qrs_discrim(ebuf+i);
  }
}

#ifdef TRACE
  fprintf(ecgnosislog, "    get_samples returned %ld OK.\n", l);
  fflush(ecgnosislog);
#endif


gsetcolor(g, d, col);
gmoveto(g, d, 1, ys/2);

/*
for(i=0; i<20; i++)
  printf("%d ", ebuf[i]);
printf("\n");
*/

ysc = (short)(yscale * 10);
nos = xs*xdiv/2;

fprintf(ecgnosislog, "Sample=%d, ysc=%d\n", ebuf[0], ysc);
fflush(ecgnosislog);

for(i=0; i<nos; i++)
  {
  y = (short)(ysc*(ebuf[i]-128)/10);
  glineto(g, d, (2*i)/xdiv, ys/2 - y + yshift);
  }
if(use_events_f) {
  for(i=0; i<e->nof_events; i++) {
    if(e->b_events[i]>(offset*50+1000) &&
       e->b_events[i]<(offset+xs*xdiv/2)*50-1000) {
      gsetcolor(g, d, event_mark_color);
      y = ((e->b_events[i]/50 - offset) * 2)/xdiv;  /* rather called x */
      gmoveto(g, d, y, 0);
      glineto(g, d, y, 50);
      gmoveto(g, d, y+1, 0);
      glineto(g, d, y+1, 40);
      gmoveto(g, d, y-1, 0);
      glineto(g, d, y-1, 40);
    }
  }
}

/* Display the measurements (specific code for each one supported) */

evs = all_measurements_between(offset*50, (offset+nos)*50);

for(ev = evs; ev!=NIL; ev = cdr(ev)) {
  /* first, locate event position */
  if(is_binary(slot_of(car(ev),_from)))
    evp = binary(slot_of(car(ev),_from));
  else
    continue;
  if(slot_of(car(ev), cl_make_symbol(":param")) ==
     cl_make_symbol("art-pressure")) {
    pvs = slot_of(car(ev), cl_make_symbol(":val"));
    if(!is_binary(car(pvs)) || !is_binary(cadr(pvs)))
      continue;
    sprintf(markbuf, "TA=%d/%d", binary(car(pvs)), binary(cadr(pvs)));
    gsetcolor(g, d, event_mark_color);
    y = ((evp/50 - offset) * 2)/xdiv;  /* rather called x */
    if (y<400)
      gtext(g, d, y+5, 20, markbuf);
    else
      gtext(g, d, y-85, 20, markbuf);
    gmoveto(g, d, y, 0);
    glineto(g, d, y, 50);
    gmoveto(g, d, y+1, 0);
    glineto(g, d, y+1, 40);
    gmoveto(g, d, y-1, 0);
    glineto(g, d, y-1, 40);
  }
}

/* display the detections */
/* if(transformation==0) {
  thethr = asses_qrs_threshold(ebuf, nos);
#ifdef MAINTRACE
    fprintf(ecgnosislog, "     THRESHOLD = %d\n", thethr);
    fflush(ecgnosislog);
#endif
  gsetcolor(g, d, markings_color);
  i = 0;
  do {
    y = _next_qrs(ebuf+i, nos-i-40, &thethr);
    i += y;
#ifdef MAINTRACE
    fprintf(ecgnosislog, "     QRS detected at %d, thr = %d\n", i, thethr);
    fflush(ecgnosislog);
#endif
    if (y>=0) {
      gmoveto(g, d, (2*i)/xdiv, ys/2 + yshift+50);
      glineto(g, d, (2*i)/xdiv, ys/2 + yshift-50);
    }
    i += 40;
  } while (y>=0 && i<(nos-40));
}
*/
}

void ecg_display(GWIN *g, short d,                              /* [FUN] */ 
                 PERFILE *p, long offset,                       /* [FUN] */ 
                 short xdiv, double yscale, short yshift)       /* [FUN] */ 

{
  long xsize_in_ccs;
  long ysize_in_pixels;
  short xp;

  short sp, cbt, i;
  long offs;
  char markbuf[30];

  enter_graphic_block();
  offs = offset * 50;

/* Displaying the line markings and other markings */

  if(offs>50000 &&
     (bl_marking || j_marking || rr_marking || st_marking ||
      qrsdet_marking || qrsref_marking || qrslen_marking
      || morph_marking)) {

/* The sizes of the marking information */
    
    xsize_in_ccs = dwin_x_size(g, d) * xdiv * 25;
    ysize_in_pixels = dwin_y_size(g, d);
    
    sp = reqitab(p, offs, offs+xsize_in_ccs+10000); /* exception! */
    cbt = complexes_between(p, offs, offs+xsize_in_ccs+10000);
      /* exception! */
    if(cbt>MAX_DISP_COMPLEXES-2) /* exception! */
      cbt = MAX_DISP_COMPLEXES-2;

/* obtaining the marking information */
    
    cpars_at(p, m_qrspos, CPQRSPOS, offs, cbt);
    if(j_marking)
      cpars_at(p, m_j, CPJPOS, offs, cbt);
    if(qrslen_marking)
      cpars_at(p, m_qrslen, CPQRSLEN, offs, cbt);
    if(morph_marking)
      cpars_at(p, m_morph, CPQRSMORPH, offs, cbt);
    if(bl_marking) {
      cpars_at(p, m_bl0, CPBL1, offs, cbt);
/*       cpars_at(p, m_bl1, CPBL2, offs, cbt); /* Can't be yet */
    }
    if(st_marking) {
      cpars_at(p, m_st0, CPST1, offs, cbt);
    }

/* Displaying the marking information */

    gsetcolor(g, d, markings_color);
    if(qrsdet_marking) { /* a vertical line will be displayed at QRSPOS */
      for(i=0; i<cbt; i++) {
	gmoveto(g, d, (m_qrspos[i] - offs)/(xdiv*25),
		QRSPOS_MARKING_DIST);
	glineto(g, d, (m_qrspos[i] - offs)/(xdiv*25),
		ysize_in_pixels-QRSPOS_MARKING_DIST);
      }
    }
    if(j_marking) { /* a vertical line will be displayed at QRSPOS */
      for(i=0; i<cbt; i++) {
	gmoveto(g, d, (m_qrspos[i] - offs)/(xdiv*25),
		3 * QRSPOS_MARKING_DIST); 
	glineto(g, d, (m_qrspos[i] + m_j[i] - offs)/(xdiv*25),
		3 * QRSPOS_MARKING_DIST);
	glineto(g, d, (m_qrspos[i] + m_j[i] - offs)/(xdiv*25),
		ysize_in_pixels - 3 * QRSPOS_MARKING_DIST);
      }
    }
    if(rr_marking) {
      for(i=0; i<cbt-1; i++) {
	sprintf(markbuf, "%4ldms", (m_qrspos[i+1] - m_qrspos[i])/10);
	xp = (m_qrspos[i] - offs)/(xdiv*25)+6;
	if(xp + strlen(markbuf)*8 < dwin_x_size(g, d))
	  gtext(g, d, xp, 
		1 * QRSPOS_MARKING_DIST, markbuf);
	sprintf(markbuf, "%4ld/'", 600000 / (m_qrspos[i+1] - m_qrspos[i]));
	if(xp + strlen(markbuf)*8 < dwin_x_size(g, d))
	  gtext(g, d, (m_qrspos[i] - offs)/(xdiv*25)+6,
		2 * QRSPOS_MARKING_DIST - 3, markbuf);
      }
    }
    if(qrslen_marking) {
      for(i=0; i<cbt-1; i++) {
	sprintf(markbuf, "%dms", m_qrslen[i] / 10);
	xp = (m_qrspos[i] - offs)/(xdiv*25)+6;
	if(xp + strlen(markbuf)*8 < dwin_x_size(g, d))
	  gtext(g, d, xp, 
		3 * QRSPOS_MARKING_DIST, markbuf);
      }
    }
    if(morph_marking) {
      for(i=0; i<cbt-1; i++) {
	sprintf(markbuf, "#%d", m_morph[i]);
	xp = (m_qrspos[i] - offs)/(xdiv*25)+6;
	if(xp + strlen(markbuf)*8 < dwin_x_size(g, d))
	  gtext(g, d, xp, 
		4 * QRSPOS_MARKING_DIST, markbuf);
      }
    }
    if(bl_marking) {
      for(i=0; i<cbt; i++) {
	gmoveto(g, d, (m_qrspos[i] - offs)/(xdiv*25),
		ysize_in_pixels/2 - (short)(yscale * (m_bl0[i]-128)) +
		yshift + strip_1_shift);
	glineto(g, d, (m_qrspos[i] - offs)/(xdiv*25) - BL_MARKING_SIZE/50,
		ysize_in_pixels/2 - (short)(yscale * (m_bl0[i]-128)) +
		yshift + strip_1_shift);
      }
    }
    if(st_marking) {
      for(i=0; i<cbt; i++) {
	gmoveto(g, d, (m_qrspos[i] - offs)/(xdiv*25),
		ysize_in_pixels/2 - (short)(yscale * (m_bl0[i]+m_st0[i]-128)) +
		yshift + strip_1_shift);
	glineto(g, d, (m_qrspos[i] - offs)/(xdiv*25) + ST_MARKING_SIZE/50,
		ysize_in_pixels/2 - (short)(yscale * (m_bl0[i]+m_st0[i]-128)) +
		yshift + strip_1_shift);
      }
    }
  }
  settransformation(0);  /* the ecg itself */
  if(ecg_noft(p->e)>1)
    _ecg_display(g, d, p, offset, xdiv, yscale, yshift+strip_2_shift,
		 lead_2, strip_2_color);
  settransformation(0);  
  _ecg_display(g, d, p, offset, xdiv, yscale, yshift+strip_1_shift,
	       lead_1, strip_1_color);

  exit_graphic_block();
}


/* </code>


\section{The time diagrams interraction}

The following td-resolutions exist:

1: four pixels per second
2: one pixel per second (the standard resolution)
3: one pixel per two seconds
4: one pixel per 10 seconds
5: one pixel per minute
6: one pixel per 4 minutes (15 per hour, 360 per 24 hours)

A description of a time diagram consists of the starting moment
(ccs from the start of the recording), the basis and the increment
for the scale --- in displayed and in used units ---
the resolution (one of the above), the width and a set
of values to be displayed. A ``band'' cursor is also, usually, displayed
in the time diagram. Its start and its size in ccs are to be specified
as well.

*/

#define PIX4SEC1   0
#define PIX2SEC1   1
#define PIX1SEC1   2
#define PIX1SEC2   3
#define PIX1SEC10  4
#define PIX1MIN1   5
#define PIX1MIN4   6

#define MAX_TD_XSIZE 440                                        /* [DEF] */ 

short td_res = PIX2SEC1;       /* one of the above symbols */   /* [VAR] */ 

long crt_td_org = 0;                                            /* [VAR] */ 

long td_start;                                                  /* [VAR] */ 
long td_band_on;   /* location of band in ccs from start */     /* [VAR] */ 
long td_band_off;                                               /* [VAR] */
long td_xsize = MAX_TD_XSIZE;                                   /* [VAR] */ 

short td_type = F_DIAGRAM; 					/* [VAR] */ 
short tdplus_type = 0; 				                /* [VAR] */ 

short td_enable = 1; 						/* [VAR] */ 
short tdplus_enable = 1; 					/* [VAR] */ 

short td_u_base = 0;    /* use base */                          /* [VAR] */ 
short td_u_incr = 20;    /* use increment */                    /* [VAR] */ 
short td_d_base = 0;    /* display base */                      /* [VAR] */ 
short td_d_incr = 10;    /* display increment */                /* [VAR] */ 


short tdtop[MAX_TD_XSIZE]; /* upper limit of data to display */ /* [VAR] */ 
short tdbot[MAX_TD_XSIZE]; /* lower limit of data to display */ /* [VAR] */ 
short tdy[MAX_TD_XSIZE];   /* value to display */               /* [VAR] */ 

short tdplus_u_base = -50; 					/* [VAR] */ 
short tdplus_u_incr = 10; 					/* [VAR] */ 
short tdplus_d_base = -50; 					/* [VAR] */ 
short tdplus_d_incr = 10; 					/* [VAR] */ 

short tdplustop[MAX_TD_XSIZE];   				/* [VAR] */ 
short tdplusbot[MAX_TD_XSIZE]; 					/* [VAR] */ 
short tdplusy[MAX_TD_XSIZE]; 					/* [VAR] */ 

/* There are many means of displaying this information:
 * 
 * 1. to draw a line between the |tdy| values 
 * 2. to draw (a line between the |tdy| values and) a vertical line
 *    between each of the tdtop and tdbot values
 * 3. to draw 3 lines: between the bottoms, between the tops
 *    and between the middles.
 */

#define LINE_TD_CHART   1                                       /* [DEF] */ 
#define RANGE_TD_CHART  2                                       /* [DEF] */ 
#define LIMITS_TD_CHART 3                                       /* [DEF] */ 
#define PLOT_TD_CHART   4                                       /* [DEF] */

char tickbuf[10];                                               /* [VAR] */ 

#define NOF_TD_HORLINES 10                                      /* [DEF] */ 

void draw_td_grid(GWIN *g, short d, short xsize)                /* [FUN] */ 

{
long td_x_base;
long td_x_incr;
short x, i;
short nom, noh, nos;

enter_graphic_block();

  for(i=0; i<NOF_TD_HORLINES; i++) {
    gsetcolor(g, d, td_grid_color);
    gmoveto(g, d, 30, (NOF_TD_HORLINES - i)*td_d_incr);
    glineto(g, d, xsize-30, (NOF_TD_HORLINES - i)*td_d_incr);
    if(i%2 == 1) {
      gsetcolor(g, d, td_color);
      sprintf(tickbuf, "%3d", td_u_base + i * td_u_incr);
      gtext(g, d, 0, (NOF_TD_HORLINES - i)*td_d_incr - 8, tickbuf);

      gsetcolor(g, d, tdplus_color);
      sprintf(tickbuf, "%3d", tdplus_u_base + i * tdplus_u_incr);
      gtext(g, d, 474, (NOF_TD_HORLINES - i)*tdplus_d_incr - 8, tickbuf);
    }
  }

  gsetcolor(g, d, td_grid_color);

  switch(td_res) {
  case PIX4SEC1:
    td_x_incr = 80;  /* at 20 s */
    td_x_base = 200000 - (td_start % 200000);  /* distance in ccs */
    td_x_base /= 2500; /* number of pixels to the first one */
    noh = td_start / 36000000;
    nom = (td_start % 36000000) / 600000;
    nos = (td_start % 600000) / 200000;
    nos *= 20;
    for(x=td_x_base; x<xsize-60; x+=td_x_incr) {
      gmoveto(g, d, x+30, 0);
      glineto(g, d, x+30, NOF_TD_HORLINES * td_d_incr);
      nos += 20;
      if(nos>=60) {
        nom ++;
        nos -= 60;
      }
      if(nom>=60) {
        noh++;
        nom -= 60;
      }
      if(x>50 && x<xsize-50) {
        sprintf(tickbuf, "%02d:%02d:%02d", noh, nom, nos);
        gtext(g, d, x-2, NOF_TD_HORLINES * td_d_incr + 6, tickbuf);
      }
    }
    break;
  case PIX2SEC1:
    td_x_incr = 120;  /* at 60 s */
    td_x_base = 600000 - (td_start % 600000);  /* distance in ccs */
    td_x_base /= 5000; /* number of pixels to the first one */
    noh = td_start / 36000000;
    nom = (td_start % 36000000) / 600000;
    for(x=td_x_base; x<xsize-60; x+=td_x_incr) {
      gmoveto(g, d, x+30, 0);
      glineto(g, d, x+30, NOF_TD_HORLINES * td_d_incr);
      nom++;
      if(nom>=60) {
        noh++;
        nom -= 60;
      }
      if(x>50 && x<xsize-50) {
        sprintf(tickbuf, "%02d:%02d", noh, nom);
        gtext(g, d, x+10, NOF_TD_HORLINES * td_d_incr + 6, tickbuf);
      }
    }
    break;
  case PIX1SEC1:
    td_x_incr = 60;  /* at 60 s */
    td_x_base = 600000 - (td_start % 600000);  /* distance in ccs */
    td_x_base /= 10000; /* number of pixels to the first one */
    noh = td_start / 36000000;
    nom = (td_start % 36000000) / 600000;
    for(x=td_x_base; x<xsize-60; x+=td_x_incr) {
      gmoveto(g, d, x+30, 0);
      glineto(g, d, x+30, NOF_TD_HORLINES * td_d_incr);
      nom++;
      if(nom>=60) {
        noh++;
        nom -= 60;
      }
      if(x>50 && x<xsize-50) {
        sprintf(tickbuf, "%02d:%02d", noh, nom);
        gtext(g, d, x+10, NOF_TD_HORLINES * td_d_incr + 6, tickbuf);
      }
    }
  default:
    break;
  }
exit_graphic_block();
}

/* The following function must be called with |tdbot|, |tdy| and |tdtop|
 * set to the ``argument'' values; the |td_xsize| (global variable) is used for
 * the (accessible) sizes of the three vectors.
 */

draw_td(GWIN *g, short d, short dmode, short color)             /* [FUN] */ 

{
  short i;
  short verd;
  long timeint;

  cell evs;
  cell ev;
  long evp;
  cell pvs;

  char markbuf[30];
  short y;

  verd = NOF_TD_HORLINES * td_d_incr;

  /* Display the measurements (specific code for each one supported) */
  
  switch(td_res) {
  case PIX4SEC1:
    timeint = (td_xsize/4 + 1) * 10000;
    break;
  case PIX2SEC1:
    timeint = (td_xsize/2 + 1) * 10000;
    break;
  default:
    return;  /* no such resolution yet */
  }
  
  evs = all_measurements_between(td_start, td_start+timeint);
  
  for(ev = evs; ev!=NIL; ev = cdr(ev)) {
    /* first, locate event position */
    if(is_binary(slot_of(car(ev),_from)))
      evp = binary(slot_of(car(ev),_from));
    else
      continue;
    if(slot_of(car(ev), cl_make_symbol(":param")) ==
       cl_make_symbol("art-pressure")) {
      pvs = slot_of(car(ev), cl_make_symbol(":val"));
      if(!is_binary(car(pvs)) || !is_binary(cadr(pvs)))
	continue;
      sprintf(markbuf, "TA=%d/%d", binary(car(pvs)), binary(cadr(pvs)));
      gsetcolor(g, d, event_mark_color);

      switch(td_res) {
      case PIX4SEC1:
	y = (evp - td_start) / 2500 + 30;
	break;
      case PIX2SEC1:
	y = (evp - td_start) / 5000 + 30;
	break;
      default:
	return;  /* no such resolution yet */
      }

      if (y<400)
	gtext(g, d, y+5, 20, markbuf);
      else
	gtext(g, d, y-85, 20, markbuf);

      gmoveto(g, d, y, 0);
      glineto(g, d, y, 50);
      gmoveto(g, d, y+1, 0);
      glineto(g, d, y+1, 40);
      gmoveto(g, d, y-1, 0);
      glineto(g, d, y-1, 40);
    }
  }
  
  gsetcolor(g, d, color);

  switch(dmode) {
  case LINE_TD_CHART:
    enter_graphic_block();
    gmoveto(g, d, 30,
            verd - ((tdy[0] - td_u_base) * td_d_incr / td_u_incr + td_d_base));
    for(i=0; i<td_xsize; i++) {
      /*      gmoveto(g, d, 30+i,
              verd - ((tdy[i] - td_u_base) * td_d_incr / td_u_incr + td_d_base)); */
      glineto(g, d, 31+i,
              verd - ((tdy[i] - td_u_base) * td_d_incr / td_u_incr + td_d_base));
    }
    exit_graphic_block();
    break;
  }
}

void compute_td(long *vb, long *vbdata, long vbsz, long m0)        /* [FUN] */ 


{
  short delt;
  short i, j, k, t;
  long m;

  td_start = m0;
  if(td_res==PIX4SEC1 || td_res==PIX2SEC1) {
    if(td_res==PIX4SEC1)
      delt = 2500;
    if(td_res==PIX2SEC1)
      delt = 5000;
    i = 0;  /* index in the input table, |vb| */
    j = 0;  /* index in the output table, |tdy| */
    m = m0;
    while (i<vbsz-1 && j<td_xsize) {
      while(i<vbsz-1 && m>vb[i+1])
        i++;
      if(i<vbsz-1) {
	switch(td_type) {
	case F_DIAGRAM:
	  tdy[j] = (600000 / (vb[i+1] - vb[i] + 1));
	  if(tdy[j]>200)
	    tdy[j] = 201;
	  j++;
	  break;
	case RR_DIAGRAM:
	  tdy[j] = (vb[i+1] - vb[i])/100;  /* it is in cs */
	  j++;
	  break;
	case ST1_DIAGRAM:
	case ST2_DIAGRAM:
	  tdy[j] = 50 + 5 * vbdata[i];
	  j++;
	  break;
	case ST1MED_DIAGRAM:
	case ST2MED_DIAGRAM:
	  #define STMEDN 5
	  if(i<vbsz-STMEDN-1)
	    for(t=0, k=0; k<STMEDN; k++)
	      t+=vbdata[i+k];
	  else
	    for(t=0, k=-STMEDN; k<0; k++)
	      t+=vbdata[i+k];
	  tdy[j] = 50 + (5 * t) / STMEDN;
	  j++;
	  break;
	default:  /* the complex general computation should be added here */
	  break;
	}
      }
      m += delt;
    }
  }
  return ;        
}

/* the following trick is done in order to avoid replicating
 * the complicated and changing code from above; but, it means
 * that compute_tdplus MUST always be called before compute_td
 * and not after.
 */

void draw_tdplus(GWIN *g, short d, short mode, short color)

{
  short ubt, uit, dbt, dit;

  ubt = td_u_base;
  uit = td_u_incr;
  dbt = td_d_base;
  dit = td_d_incr;
  
  td_u_base = tdplus_u_base;
  td_u_incr = tdplus_u_incr;
  td_d_base = tdplus_d_base;
  td_d_incr = tdplus_d_incr;
  
  draw_td(g, d, mode, color);

  td_u_base = ubt;
  td_u_incr = uit;
  td_d_base = dbt;
  td_d_incr = dit;
}


void compute_tdplus(long *vb, long *vbdata, long vbsz, long m0)

{
  short i;
  short temp;

  temp = td_type;
  td_type = tdplus_type;
  compute_td(vb, vbdata, vbsz, m0);
  for(i=0; i<MAX_TD_XSIZE; i++) {
    tdplustop[i] = tdtop[i];
    tdplusbot[i] = tdbot[i];
    tdplusy[i] = tdy[i];
  }
  td_type = temp;
}



long *vbuf = NULL;                                              /* [VAR] */ 
long *vbufdata = NULL;

void display_td(GWIN *g, short d, PERFILE *p, long m0)          /* [FUN] */ 

  /* uses the current resolution */

{
long cbt;
short sp, c;
long timeint;

switch(td_res) {
  case PIX4SEC1:
    timeint = (td_xsize/4 + 1) * 10000;
    break;
  case PIX2SEC1:
    timeint = (td_xsize/2 + 1) * 10000;
    break;
  default:
    return;  /* no such resolution yet */
  }
  
#ifdef MAINTRACE
  fprintf(ecgnosislog, "BEFORE CALLING REQITAB.\n");
  fflush(ecgnosislog);
#endif
sp = reqitab(p, m0, m0+timeint);  /* Require the itab for the first 5 min */
#ifdef MAINTRACE
  fprintf(ecgnosislog, "     REQITAB returned %d\n", sp);
  fprintf(ecgnosislog, "BEFORE CALLING complexes_between\n");
  fflush(ecgnosislog);
#endif

cbt = complexes_between(p, m0, m0+timeint);
#ifdef TRACE
  fprintf(ecgnosislog, "     complexes_between returned %ld\n", cbt);
  fflush(ecgnosislog);
#endif TRACE

#ifdef MAINTRACE
  fprintf(ecgnosislog, "Before free\n");
  fflush(ecgnosislog);
#endif

/*
if(vbuf!=NULL)
  free(vbuf);
*/

#ifdef MAINTRACE
  fprintf(ecgnosislog, "Before malloc\n");
  fflush(ecgnosislog);
#endif

vbuf = (long *)malloc(cbt * sizeof(long));
vbufdata = (long *)malloc(cbt * sizeof(long));
if(vbuf!=NULL && vbufdata!=NULL) {
#ifdef MAINTRACE
  fprintf(ecgnosislog, "vbuf allocated OK.\n");
  fprintf(ecgnosislog, "calling cpars_at with CPQRSPOS.\n");
  fflush(ecgnosislog);
#endif TRACE
  cpars_at(p, vbuf, CPQRSPOS, m0, cbt);
  switch(td_type) {
  case ST1_DIAGRAM:
  case ST1MED_DIAGRAM:
    cpars_at(p, vbufdata, CPST1, m0, cbt);
    break;
  case ST2_DIAGRAM:
  case ST2MED_DIAGRAM:
    cpars_at(p, vbufdata, CPST2, m0, cbt);
    break;
  default:
    break;  /* This table not needed */
  }
  
#ifdef MAINTRACE
  fprintf(ecgnosislog, "cpars_at returned.\n");
  for(c=0; c<100; c++)
    fprintf(ecgnosislog, "CPARS RESULT #%d = %ld\n", c, vbuf[c]);
  fflush(ecgnosislog);
#endif TRACE
}
else {
  fprintf(ecgnosislog, "ERROR: can't alloc vbuf\n");
  exit(0);
}

draw_td_grid(g, d, 500);

  /* otherwise it uses the td global variable for the computation */

if(td_type>0) {
  compute_td(vbuf, vbufdata, cbt, m0);
  draw_td(g, d, LINE_TD_CHART, td_color);
}

switch(tdplus_type) {
 case ST1_DIAGRAM:
 case ST1MED_DIAGRAM:
  cpars_at(p, vbufdata, CPST1, m0, cbt);
  break;
 case ST2_DIAGRAM:
 case ST2MED_DIAGRAM:
  cpars_at(p, vbufdata, CPST2, m0, cbt);
  break;
 default:
  break;  /* This table not needed */
}

if(tdplus_type>0) {
  compute_tdplus(vbuf, vbufdata, cbt, m0);
  draw_tdplus(g, d, LINE_TD_CHART, tdplus_color);
}

free(vbuf);
free(vbufdata);
vbuf = vbufdata = NULL;
setspotshot(g, lod, '&');
}

/* </code>

   \section{Drawing a progression indicator}
   <code> */


short prog_x = 0;
char prog_t[30] = "NONAME";

#define PROG_X_O 5
#define PROG_Y_O 25

void clear_progression(GWIN *gg)

{
  cleardwin(gg, prod, PLANE_DWIN);
  prog_x = 0;
}

void show_progression(GWIN *gg, char *prog_text)

{
  cleardwin(gg, prod, PLANE_DWIN);
  gsetcolor(gg, prod, progress_color);
  if(strlen(prog_text)<30) {
    gtext(gg, prod, 5, 5, prog_text);
    strcpy(prog_t, prog_text);
  } 
  gmoveto(gg, prod, PROG_X_O, PROG_Y_O);
  glineto(gg, prod, PROG_X_O+104, PROG_Y_O);
  glineto(gg, prod, PROG_X_O+104, PROG_Y_O+24);
  glineto(gg, prod, PROG_X_O, PROG_Y_O+24);
  glineto(gg, prod, PROG_X_O, PROG_Y_O);
  prog_x = 0;
}

void set_progression(GWIN *gg, short p)   /* p must be between 0 and 100 */

{
  short j;

  if(p<0 || p>100)
    return ;
  if(p<prog_x)
    show_progression(gg, prog_t);
  gsetcolor(gg, prod, progress_color);
  enter_graphic_block();
  for(j=prog_x; j<p; j++) {
    gmoveto(gg, prod, PROG_X_O + 2 + j, PROG_Y_O + 2);
    glineto(gg, prod, PROG_X_O + 2 + j, PROG_Y_O + 22);
  }
  exit_graphic_block();
  prog_x = p;
}


/*--
  \end{code}

  \funparagraph{select\_a\_time\_diagram}

  \begin{code}
  --*/


short sel_td; 							/* [VAR] */ 

short select_a_time_diagram(GWIN *gg, short prev_td)  		/* [FUN] */ 



{
  short cc;
  
  sel_td = prev_td;
  cleardwin(gg, rjumps, PLANE_DWIN);
  
  enter_graphic_block();
  defaltswitch(gg, rjumps, 18,  23, 90, 20, tds[F_DIAGRAM].tdname,
	       &sel_td, F_DIAGRAM);
  defaltswitch(gg, rjumps, 18,  53, 90, 20, tds[RR_DIAGRAM].tdname,
	       &sel_td, RR_DIAGRAM);
  defaltswitch(gg, rjumps, 18,  83, 90, 20, tds[ST1_DIAGRAM].tdname,
	       &sel_td, ST1_DIAGRAM);
  defaltswitch(gg, rjumps, 18, 113, 90, 20, tds[ST2_DIAGRAM].tdname,
	       &sel_td, ST2_DIAGRAM);
  defaltswitch(gg, rjumps, 18, 143, 90, 20, tds[ST1MED_DIAGRAM].tdname,
	       &sel_td, ST1MED_DIAGRAM);
  defaltswitch(gg, rjumps, 18, 173, 90, 20, tds[ST2MED_DIAGRAM].tdname,
	       &sel_td, ST2MED_DIAGRAM);
  defaltswitch(gg, rjumps, 18, 203, 90, 20, "None", &sel_td, 0);
  defbutton(gg, rjumps, 18, 233, 90, 20, "OK", 7);
  defbutton(gg, rjumps, 18, 263, 90, 20, "Cancel", 8);
  defhbutton(gg, rjumps, 24);
  exit_graphic_block();

  while((cc=gwinteract(gg))!=7 && cc!=8 && cc!=24) ;
  if(cc==24) {
    wi_set_eps_unit(700);
    wi_print_gwin("tdselect.eps");
  }
  if(cc==7)
    gwaccept(gg);
  return sel_td;
}

/*--
  \end{code}

  \subsection{Reporting time diagrams}

  \begin{code}
  --*/

short print_td1 = '1';
short print_td2 = 0;

short print_td1_hist = 0;
short print_td1_min = 0;
short print_td1_med = '=';
short print_td1_max = 0;

short print_td2_hist = 0;
short print_td2_min = 0;
short print_td2_med = ':';
short print_td2_max = 0;

short td_print_res = 'a';
short td_print_width = 'n';
short td_print_mediation = '0';
short td_print_g_res = 'N';

char td_print_title[256];
char td_print_comment[1024];

td_print_screen_right_buttons(GWIN *gg)

{
  cleardwin(gg, rjumps, PLANE_DWIN);

  enter_graphic_block();
  defbutton(gg, rjumps, 18,  23, 90, 20, "L type", 'l');
  defbutton(gg, rjumps, 18,  53, 90, 20, "R type", 'r');
  defbutton(gg, rjumps, 18,  83, 90, 20, "Go", 'g');
  defbutton(gg, rjumps, 18, 113, 90, 20, "Export", 'e');
  defbutton(gg, rjumps, 18, 143, 90, 20, "Cancel", 0x001b);
  defhbutton(gg, rjumps, 24);
  exit_graphic_block();

}

/* The following must take its defaults from the values stored 
 * in the egdiagtd global variables, and only modify them
 */

make_td_print_screen(GWIN *gg)

{
  cleardwin(gg, 0, PLANE_DWIN);
  cleardwin(gg, upd, PLANE_DWIN);
  cleardwin(gg, lod, PLANE_DWIN);
  cleardwin(gg, umctrld, PLANE_DWIN);
  cleardwin(gg, lctrld, PLANE_DWIN);
  cleardwin(gg, fulld, PLANE_DWIN);

  td_print_screen_right_buttons(gg);

  gsetcolor(gg, fulld, 14);

  enter_graphic_block();
  gtext(gg, fulld, 10, 20, "Leads to print");
  defswitch(gg, fulld, 10,  40, 100, 16, td_name(get_td_type(0)),
	    &print_td1, '1');
  defswitch(gg, fulld, 120, 40, 100, 16, td_name(get_td_type(1)),
	    &print_td2, '2');

  gtext(gg, fulld, 10, 180, "Information to print");
  defswitch(gg, fulld, 10, 200, 70, 16, "Hst",
	    &print_td1_hist, '_');
  defswitch(gg, fulld, 90, 200, 70, 16, "Min",
	    &print_td1_min, '-');
  defswitch(gg, fulld, 170, 200, 70, 16, "Med",
	    &print_td1_med, '=');
  defswitch(gg, fulld, 250, 200, 70, 16, "Max",
	    &print_td1_max, '+');

  defswitch(gg, fulld, 10, 220, 70, 16, "Hst",
	    &print_td2_hist, '|');
  defswitch(gg, fulld, 90, 220, 70, 16, "Min",
	    &print_td2_min, '\\');
  defswitch(gg, fulld, 170, 220, 70, 16, "Med",
	    &print_td2_med, ':');
  defswitch(gg, fulld, 250, 220, 70, 16, "Max",
	    &print_td2_max, '/');


  gtext(gg, fulld, 10, 60, "Resolution");
  defaltswitch(gg, fulld,  10, 80, 100, 16,   "a: 0.25 s",
	       &td_print_res, 'a');
  defaltswitch(gg, fulld, 120, 80, 100, 16,   "c: 10 s",
	       &td_print_res, 'c');
  defaltswitch(gg, fulld,  10, 100, 100, 16,  "x: 30 s",
	       &td_print_res, 'x');
  defaltswitch(gg, fulld, 120, 100, 100, 16,  "z: 1 min",
	       &td_print_res, 'z');

  gtext(gg, fulld, 10, 120, "Width of the diagram");
  defaltswitch(gg, fulld,  10, 140, 100, 16,  "v: 12in",
	       &td_print_width, 'v');
  defaltswitch(gg, fulld, 120, 140, 100, 16,  "b: 3in",
	       &td_print_width, 'b');
  defaltswitch(gg, fulld,  10, 160, 100, 16,  "n: 6in",
	       &td_print_width, 'n');
  defaltswitch(gg, fulld, 120, 160, 100, 16,  "m: 5in",
	       &td_print_width, 'm');

/*
  gtext(gg, fulld, 10, 180, "Height of the strip") ;
  defaltswitch(gg, fulld,  10, 200, 100, 16,  "1: 0.5in",
	       &print_sheight, HEIGHT05);
  defaltswitch(gg, fulld, 120, 200, 100, 16,  "2: 1.0in",
	       &print_sheight, HEIGHT10);
  defaltswitch(gg, fulld,  10, 220, 100, 16,  "3: 1.5in",
	       &print_sheight, HEIGHT15);
  defaltswitch(gg, fulld, 120, 220, 100, 16,  "4: 2.0in",
	       &print_sheight, HEIGHT20);
*/

  gtext(gg, fulld, 300, 20, "Mediation");
  defaltswitch(gg, fulld, 300,  40, 190, 16,  "0: No mediation",
	       &td_print_mediation, '0');
  defaltswitch(gg, fulld, 300,  60, 190, 16,  "A: resolution",
	       &td_print_mediation, 'A');
  defaltswitch(gg, fulld, 300,  80, 190, 16,  "B: 2 * resolution",
	       &td_print_mediation, 'B');
  defaltswitch(gg, fulld, 300,  100, 190, 16, "C: 3 * resolution",
	       &td_print_mediation, 'C');

  gtext(gg, fulld, 300, 170, "Graphic resolution");
  defaltswitch(gg, fulld, 300,  190, 100, 16,  "N: 1pt",
	       &td_print_g_res, 'N');
  defaltswitch(gg, fulld, 400,  190, 100, 16,  "L: 3pt",
	       &td_print_g_res, 'L');
  defaltswitch(gg, fulld, 300, 210, 100, 16, "K: 0.5pt",
	       &td_print_g_res, 'K');
  defaltswitch(gg, fulld, 400, 210, 100, 16, "J: 0.25pt",
	       &td_print_g_res, 'J');

  gtext(gg, fulld, 10, 270, "Title");
  deftextpad(gg, fulld, 10, 290, 56, 1, 56, td_print_title, '&');

  gtext(gg, fulld, 10, 310, "Comment");
  deftextpad(gg, fulld, 10, 330, 56, 1, 56, td_print_comment, '*');
  exit_graphic_block();

}

void print_td_interraction(GWIN *gg, PERFILE *e, long offset)

{
  short gwr;

  do {
    gwr = gwinteract(gg);
    switch(gwr) {
    case 24:
      wi_set_eps_unit(700);
      wi_print_gwin("tdreport.eps");
      break;
    case 'l':
      set_td_type(0, select_a_time_diagram(gg, get_td_type(0)));
      make_td_print_screen(gg);
      break;
    case 'r':
      set_td_type(1, select_a_time_diagram(gg, get_td_type(1)));
      make_td_print_screen(gg);
      break;
    case 0x001b:
      return;
    case 'e':
      return;  /* but will do the export, which is not easy at all */
    case 'g':
      switch(td_print_res) {
      case 'a': /* 0.25 s */
	set_td_resolution(2500);
	break;
      case 'c':
	set_td_resolution(100000);
	break;
      case 'x':
	set_td_resolution(300000);
	break;
      case 'z':
	set_td_resolution(600000);
	break;
      default:
	td_print_res = 'a';
	set_td_resolution(2500);
	break;
      }
      switch(td_print_g_res) {
      case 'N':
	set_td_graphic_step(1.0);
	break;
      case 'L':
	set_td_graphic_step(3.0);
	break;
      case 'K':
	set_td_graphic_step(0.5);
	break;
      case 'J':
	set_td_graphic_step(0.25);
	break;
      default:
	set_td_graphic_step(1.0);
	td_print_g_res = 'N';
	break;
      }
      switch(td_print_mediation) {
      case '0':
      case 'A':
	set_td_filter_width(1);
	break;
      case 'B':
	set_td_filter_width(2);
	break;
      case 'C':
	set_td_filter_width(3);
	break;
      default:
	td_print_mediation = 'A';
	set_td_filter_width(1);
	break;
      }
      switch(td_print_width) {
      case 'v':
	set_td_width(144);
	break;
      case 'b':
	set_td_width(216);
	break;
      case 'n':
	set_td_width(432);
	break;
      case 'm':
	set_td_width(504);
	break;
      default:
	set_td_width(432);
	td_print_width = 'n';
	break;
      }
      set_td_title(td_print_title);
      set_td_comment(td_print_comment);
      set_time_divisions(0);  /* automatic */
      set_td_lines_to_show(0, print_td1_hist, print_td1_min,
			   print_td1_med, print_td1_max);
      set_td_lines_to_show(1, print_td2_hist,
			   print_td2_min, print_td2_med, print_td2_max);
      switch(get_td_type(0)) {
      case F_DIAGRAM:
	set_td_use_range(0, 0, 200);
	set_td_min_scale_value(0, 0.0);
	set_td_scale_label_format(0, "%4.0f");
	set_td_scale_factor(0, 1.0);
	fprintf(ecgnosislog, "Printing an F diagram.\n");
	fflush(ecgnosislog);
	break;
      case ST1_DIAGRAM:
      case ST2_DIAGRAM:
	set_td_use_range(0, -500, 500);
	set_td_min_scale_value(0, -0.5);
	set_td_scale_label_format(0, "%4.2f");
	set_td_scale_factor(0, 0.001);
	fprintf(ecgnosislog, "Printing an STx diagram.\n");
	fflush(ecgnosislog);
	break;
      default:
	break;
      }
      put_td_to_latex_list(e, offset);
      return;
    default:
      gwaccept(gg);
      continue;
    }
  } while(gwr!=0x001b);
  return ;
}

/*--
  \end{code}

  \subsection{Reporting strips}

  \ImplementationNotes{
  % How implementation of the whole package is done: algorithms---if special,
  % implementation decisions, discarded implementation alternatives
  }
  \Portability{ % What is assumed about the operating environment
  }
  \Changes{ % that are anticipated, if special provision was made for them
  }

  \begin{code}
  --*/


/*** MAKING THE STRIP PRINTING SCREEN                             ***/
/*** THIS SCREEN IS ONLY INVOKED FROM THE TIME DIAGRAMS' SCREEN   ***/
/*** THUS, IT TRIES TO USE THE SAME DWINS AS THE INVOKER          ***/

/*** The variables that control printing are defined here         ***/

#define DOSTRIP1       1       /* CTRL-A */
#define DOSTRIP2       2       /* CTRL-B */

#define EPS_FILE      'e'
#define PIC_FILE      'p'
#define TeX_FILE      't'
#define LaTeX_FILE    'l'
#define REPORT_FILE   'r'
#define EXPORT_ACTION ';'
#define BACK_ACTION   ':'

#define BW_PAPER      '1'
#define GSH_PAPER     '2'
#define COL_PAPER     '3'
#define BW_SLIDE      '4'     /* this means transparency */
#define GSH_SLIDE     '5'     /* this means film */
#define COL_SLIDE     '6'     /* film also */

#define SPEED10       'c'
#define SPEED25       'x'
#define SPEED50       'z'
#define SPEED2p5      'a'

#define HEIGHT05      '1'
#define HEIGHT10      '2'
#define HEIGHT15      '3'
#define HEIGHT20      '4'

#define AMPL1000      'A'
#define AMPL2000      'B'
#define AMPL5000      'C'
#define AMPL0500      'Z'
#define AMPL0200      'X'
#define AMPLAUTO      '0'

#define NULLGRID      'N'
#define LINEDGRID     'L'

#define LENGTH12IN      'v'
#define LENGTH4IN      'b'
#define LENGTH6IN      'n'
#define LENGTH5IN      'm'
#define MANY_STRIPS    'o'

short print_st1 = DOSTRIP1;
short print_st2 = DOSTRIP2;

char print_st_fn[64] = "strip%d.eps"; /* name of the output file */
char print_descr[100] = "";           /* description of the strip */
char print_title[100] = "";           /* title of the strip */
char print_ns_buf[20] = "1";          /* number of strips to print */
char print_ns = 1;                    /* same, parsed */
short print_sspeed = SPEED25;         /* speed of the strip */
short print_swid = LENGTH5IN;         /* width of the strip */
short print_sheight = HEIGHT15;       /* height of the strip */
short print_sampl = AMPLAUTO;         /* amplification of the strip */
short print_gridtype = LINEDGRID;     /* type of grid used in output */
short out_hardcopy = BW_PAPER;        /* type of output media */

make_print_screen(GWIN *gg)

{
  cleardwin(gg, 0, PLANE_DWIN);
  cleardwin(gg, upd, PLANE_DWIN);
  cleardwin(gg, lod, PLANE_DWIN);
  cleardwin(gg, umctrld, PLANE_DWIN);
  cleardwin(gg, lctrld, PLANE_DWIN);
  cleardwin(gg, rjumps, PLANE_DWIN);
  cleardwin(gg, fulld, PLANE_DWIN);

  enter_graphic_block();
  defbutton(gg, rjumps, 18,  23, 90, 20, "Help", 'h');
  defbutton(gg, rjumps, 18,  53, 90, 20, "-> Report", REPORT_FILE);
  defbutton(gg, rjumps, 18,  83, 90, 20, "-> TeX", TeX_FILE);
  defbutton(gg, rjumps, 18, 113, 90, 20, "-> LaTeX", LaTeX_FILE);
  defbutton(gg, rjumps, 18, 143, 90, 20, "-> PIC", PIC_FILE);
  defbutton(gg, rjumps, 18, 173, 90, 20, "-> EPS", EPS_FILE);
  defbutton(gg, rjumps, 18, 203, 90, 20, "Export", EXPORT_ACTION);
  defbutton(gg, rjumps, 18, 233, 90, 20, "Back", BACK_ACTION);
  defhbutton(gg, rjumps, 24);

  gsetcolor(gg, fulld, 14);

  gtext(gg, fulld, 10, 20, "Leads to print");
  defswitch(gg, fulld, 10,  40, 100, 16, "Lead A", &print_st1, DOSTRIP1);
  defswitch(gg, fulld, 120, 40, 100, 16, "Lead B", &print_st2, DOSTRIP2);

  gtext(gg, fulld, 10, 60, "Paper speed");
  defaltswitch(gg, fulld,  10, 80, 100, 16,   "a: 2.5mm/s",
	       &print_sspeed, SPEED2p5);
  defaltswitch(gg, fulld, 120, 80, 100, 16,   "c:  10mm/s",
	       &print_sspeed, SPEED10);
  defaltswitch(gg, fulld,  10, 100, 100, 16,  "x:  25mm/s",
	       &print_sspeed, SPEED25);
  defaltswitch(gg, fulld, 120, 100, 100, 16,  "z:  50mm/s",
	       &print_sspeed, SPEED50);

  gtext(gg, fulld, 10, 120, "Width of the strip");
  defaltswitch(gg, fulld,  10, 140, 100, 16,  "v: 12in",
	       &print_swid, LENGTH12IN);
  defaltswitch(gg, fulld, 120, 140, 100, 16,  "b: 4in",
	       &print_swid, LENGTH4IN);
  defaltswitch(gg, fulld,  10, 160, 100, 16,  "n: 6in",
	       &print_swid, LENGTH6IN);
  defaltswitch(gg, fulld, 120, 160, 100, 16,  "m: 5in",
	       &print_swid, LENGTH5IN);


  gtext(gg, fulld, 10, 180, "Height of the strip") ;
  defaltswitch(gg, fulld,  10, 200, 100, 16,  "1: 0.5in",
	       &print_sheight, HEIGHT05);
  defaltswitch(gg, fulld, 120, 200, 100, 16,  "2: 1.0in",
	       &print_sheight, HEIGHT10);
  defaltswitch(gg, fulld,  10, 220, 100, 16,  "3: 1.5in",
	       &print_sheight, HEIGHT15);
  defaltswitch(gg, fulld, 120, 220, 100, 16,  "4: 2.0in",
	       &print_sheight, HEIGHT20);

  gtext(gg, fulld, 300, 20, "Signal amplification");
  defaltswitch(gg, fulld, 300,  40, 190, 16,  "0: automatic",
	       &print_sampl, AMPLAUTO);
  defaltswitch(gg, fulld, 300,  60, 190, 16,  "A:  10mm/mV",
	       &print_sampl, AMPL1000);
  defaltswitch(gg, fulld, 300,  80, 190, 16, "B:   5mm/mV",
	       &print_sampl, AMPL2000);
  defaltswitch(gg, fulld, 300,  100, 190, 16, "C:   2mm/mV",
	       &print_sampl, AMPL5000);
  defaltswitch(gg, fulld, 300, 120, 190, 16,  "Z:  20mm/mV",
	       &print_sampl, AMPL0500);
  defaltswitch(gg, fulld, 300, 140, 190, 16,  "X:  50mm/mV",
	       &print_sampl, AMPL0200);



  gtext(gg, fulld, 300, 170, "Grid type");
  defaltswitch(gg, fulld, 300,  190, 100, 16,  "N: none",
	       &print_gridtype, NULLGRID);
  defaltswitch(gg, fulld, 400,  190, 100, 16,  "L: lined",
	       &print_gridtype, LINEDGRID);


/*
  gtext(gg, fulld, 300, 50, "Final hardcopy media");
  defaltswitch(gg, fulld, 300,  70, 190, 16, "1: B&W on paper",
	       &out_hardcopy, BW_PAPER);
  defaltswitch(gg, fulld, 300,  90, 190, 16, "2: Grays on paper",
	       &out_hardcopy, GSH_PAPER);
  defaltswitch(gg, fulld, 300,  110, 190, 16, "3: Color on paper",
	       &out_hardcopy, COL_PAPER);
  defaltswitch(gg, fulld, 300,  130, 190, 16, "4: Transparency",
	       &out_hardcopy, BW_SLIDE);
  defaltswitch(gg, fulld, 300, 150, 190, 16, "5: B&W slide",
	       &out_hardcopy, GSH_SLIDE);
  defaltswitch(gg, fulld, 300, 170, 190, 16, "6: Color slide",
	       &out_hardcopy, COL_SLIDE);
*/


  gtext(gg, fulld, 300, 220, "N strips");
  deftextpad(gg, fulld, 400, 220, 5, 1, 5, print_ns_buf, '$');

  gtext(gg, fulld, 10, 270, "Title");
  deftextpad(gg, fulld, 10, 290, 56, 1, 56, print_title, '&');

  gtext(gg, fulld, 10, 310, "Comment");
  deftextpad(gg, fulld, 10, 330, 56, 1, 56, print_descr, '*');
  exit_graphic_block();
}

/* max size of the sampled buffer, 7in at 10mm/s = 17.5s = 175000 ccs */

#define MAX_ST_PRINT_SIZE 180000

void print_screen_interraction(GWIN *gg, PERFILE *e, long offset)

{
  short gwr;
  short speed;
  short width;
  short height;
  short clb;
  short l;
  long nofsamps, biglt;
  long i;
  FILE *ef;

  do {
  gwr = gwinteract(gg);
  switch(gwr) {

#ifdef NEVER_DEFINED
  case BW_PAPER:
    gwaccept(gg);
/*    set_gliktype_technology(1);*/
    continue;
  case GSH_PAPER:
    gwaccept(gg);
/*    set_gliktype_technology(2);*/
    continue;
  case COL_PAPER:
    gwaccept(gg);
/*    set_gliktype_technology(3);*/
    continue;
  case BW_SLIDE:
    gwaccept(gg);
/*    set_gliktype_technology(4);*/
    continue;
  case GSH_SLIDE:
    gwaccept(gg);
/*    set_gliktype_technology(5);*/
    continue;
  case COL_SLIDE:
    gwaccept(gg);
/*    set_gliktype_technology(6);*/
    continue;
#endif

  case 24:
    wi_set_eps_unit(700);
    wi_print_gwin("streport.eps");
  case LENGTH12IN:
  case LENGTH4IN:
  case LENGTH6IN:
  case LENGTH5IN:

  case HEIGHT05:
  case HEIGHT10:
  case HEIGHT15:
  case HEIGHT20:

  case DOSTRIP1:
  case DOSTRIP2:

  case SPEED2p5:
  case SPEED10:
  case SPEED25:
  case SPEED50:

  case AMPLAUTO:
  case AMPL0200:
  case AMPL0500:
  case AMPL1000:
  case AMPL2000:
  case AMPL5000:

  case LINEDGRID:
  case NULLGRID:

  case '&':
  case '*':
    gwaccept(gg);
    continue;
  case '$':
    gwaccept(gg);
    print_ns = atoi(print_ns_buf);
    if(print_ns<1 || print_ns>60) {
      sprintf(print_ns_buf, "1");
      print_ns = 1;
    }
    continue;

  case EPS_FILE:
    clear_strip_description();
    
    switch(print_sspeed) {
    case SPEED10:
      speed = S10mmps;
      break;
    case SPEED50:
      speed = S50mmps;
      break;
/*    case SPEED100:
      speed = S100mmps;
      break;*/
    case SPEED2p5:
      speed = S2p5mmps;
      break;
    case SPEED25:
    default:
      speed = S25mmps;
      break;
    }
    set_strip_paper_speed(speed);

    switch(print_swid) {
    case LENGTH12IN:
      width = 864;
      break;
    case LENGTH4IN:
      width = 288;
      break;
    case LENGTH5IN:
      width = 360;
      break;
    case LENGTH6IN:
    default:
      width = 432;
      break;
    }
    set_strip_width(width);

    switch(print_sheight) {
    case HEIGHT05:
      height = 36;
      break;
    case HEIGHT10:
      height = 72;
      break;
    case HEIGHT15:
      height = 108;
      break;
    case HEIGHT20:
      height = 144;
      break;
    }
    set_strip_height(height);

    set_strip_amplification(0);
    switch(print_sampl) {
    case AMPLAUTO:
      set_strip_amplification(0);
      break;
    case AMPL0200:
      set_strip_amplification(200);
      break;
    case AMPL0500:
      set_strip_amplification(500);
      break;
    case AMPL1000:
      set_strip_amplification(1000);
      break;
    case AMPL2000:
      set_strip_amplification(2000);
      break;
    case AMPL5000:
      set_strip_amplification(5000);
      break;
    }

    switch(print_gridtype) {
    case LINEDGRID:
      set_strip_grid_type(2);
      break;
    case NULLGRID:
      set_strip_grid_type(0);
      break;
    }

    set_strip_leads_to_print(print_st1, print_st2);
    set_strip_title(print_title);
    set_strip_comment(print_descr);
    set_number_of_strips(print_ns);

    show_progression(gg, "PRINTING");
    put_strip_to_latex_list(e, offset*50);
    set_progression(gg, 100);

#ifdef NEVER_DEFINED

    clb = 0;
    if(print_st1==DOSTRIP1) {
/*      set_epsfiles(print_st_fn);*/
/*      open_epsf();*/
/*      l = get_samples(e, ebuf, 0, offset * 50, offset * 50 + MAX_ST_PRINT_SIZE,*/
/*             50);*/
/**/
/*      fprintf(ecgnosislog, "get_samples returned %d in PRINTING\n", l);*/
/*      fflush(ecgnosislog);*/
/**/
/*      set_progression(gg, 25);*/
/*      if(l)*/
/*	psout_strip(ebuf, EBUFSIZE-10, 5, speed, width, 72, 30*clb);*/
/*      close_epsf();*/
    }
/*    set_progression(gg, 50);*/
    if(print_st2==DOSTRIP2) {
/*      set_epsfiles(print_st_fn);*/
/*      open_epsf();*/
/*      l = get_samples(e, ebuf, 1, offset * 50, offset * 50 + MAX_ST_PRINT_SIZE,*/
/*          50);*/
/*      set_progression(gg, 75);*/
/**/
/*      fprintf(ecgnosislog, "get_samples returned %d in PRINTING\n", l);*/
/*      fflush(ecgnosislog);*/
/**/
/*      if(l)*/
/*	psout_strip(ebuf, EBUFSIZE-10, 5, speed, width, 72, 30*clb);*/
/*      close_epsf();*/
    }
/*    set_progression(gg, 100);*/
#endif

    return;


  case REPORT_FILE:
    clear_strip_description();
    
    switch(print_sspeed) {
    case SPEED10:
      speed = S10mmps;
      break;
    case SPEED50:
      speed = S50mmps;
      break;
/*    case SPEED100:
      speed = S100mmps;
      break;*/
    case SPEED2p5:
      speed = S2p5mmps;
      break;
    case SPEED25:
    default:
      speed = S25mmps;
      break;
    }
    set_strip_paper_speed(speed);

    switch(print_swid) {
    case LENGTH12IN:
      width = 864;
      break;
    case LENGTH4IN:
      width = 288;
      break;
    case LENGTH5IN:
      width = 360;
      break;
    case LENGTH6IN:
    default:
      width = 432;
      break;
    }
    set_strip_width(width);

    switch(print_sheight) {
    case HEIGHT05:
      height = 36;
      break;
    case HEIGHT10:
      height = 72;
      break;
    case HEIGHT15:
      height = 108;
      break;
    case HEIGHT20:
      height = 144;
      break;
    }
    set_strip_height(height);

    set_strip_amplification(0);
    switch(print_sampl) {
    case AMPLAUTO:
      set_strip_amplification(0);
      break;
    case AMPL0200:
      set_strip_amplification(200);
      break;
    case AMPL0500:
      set_strip_amplification(500);
      break;
    case AMPL1000:
      set_strip_amplification(1000);
      break;
    case AMPL2000:
      set_strip_amplification(2000);
      break;
    case AMPL5000:
      set_strip_amplification(5000);
      break;
    }

    switch(print_gridtype) {
    case LINEDGRID:
      set_strip_grid_type(2);
      break;
    case NULLGRID:
      set_strip_grid_type(0);
      break;
    }

    set_strip_leads_to_print(print_st1, print_st2);
    set_strip_title(print_title);
    set_strip_comment(print_descr);
    set_number_of_strips(print_ns);

    show_progression(gg, "REPORTING");
    put_strip_to_report(e, offset*50);
    set_progression(gg, 100);

#ifdef NEVER_DEFINED

    clb = 0;
    if(print_st1==DOSTRIP1) {
/*      set_epsfiles(print_st_fn);*/
/*      open_epsf();*/
/*      l = get_samples(e, ebuf, 0, offset * 50, offset * 50 + MAX_ST_PRINT_SIZE,*/
/*             50);*/
/**/
/*      fprintf(ecgnosislog, "get_samples returned %d in PRINTING\n", l);*/
/*      fflush(ecgnosislog);*/
/**/
/*      set_progression(gg, 25);*/
/*      if(l)*/
/*	psout_strip(ebuf, EBUFSIZE-10, 5, speed, width, 72, 30*clb);*/
/*      close_epsf();*/
    }
/*    set_progression(gg, 50);*/
    if(print_st2==DOSTRIP2) {
/*      set_epsfiles(print_st_fn);*/
/*      open_epsf();*/
/*      l = get_samples(e, ebuf, 1, offset * 50, offset * 50 + MAX_ST_PRINT_SIZE,*/
/*          50);*/
/*      set_progression(gg, 75);*/
/**/
/*      fprintf(ecgnosislog, "get_samples returned %d in PRINTING\n", l);*/
/*      fflush(ecgnosislog);*/
/**/
/*      if(l)*/
/*	psout_strip(ebuf, EBUFSIZE-10, 5, speed, width, 72, 30*clb);*/
/*      close_epsf();*/
    }
/*    set_progression(gg, 100);*/
#endif

    return;
  case EXPORT_ACTION:
    switch(print_sspeed) {
    case SPEED10:
      speed = S10mmps;
      break;
    case SPEED50:
      speed = S50mmps;
      break;
/*    case SPEED100:
      speed = S100mmps;
      break;*/
    case SPEED25:
    default:
      speed = S25mmps;
      break;
    }
    switch(print_swid) {
    case LENGTH12IN:
      width = 864;
      break;
    case LENGTH4IN:
      width = 288;
      break;
    case LENGTH5IN:
      width = 360;
      break;
    case LENGTH6IN:
    default:
      width = 432;
      break;
    }

    switch(speed)
      {
      case S25mmps:   biglt = 200;  break;
      case S50mmps:   biglt = 100;  break;
      case S100mmps:  biglt = 50;   break;
      case S200mmps:  biglt = 25;   break;
      case S10mmps:   biglt = 500;  break;
      case S2p5mmps:  biglt = 2000; break;
      default:        biglt = 200;  break;
      }
    nofsamps = width * biglt;
    nofsamps /= 72;

    ef = fopen("LASTEXPO.ECG", "w");
    if(ef!=NULL) {
      show_progression(gg, "EXPORTING");
      set_progression(gg, 0);
      if(print_st1==DOSTRIP1)
	l = get_samples(e, ebuf, 0, offset * 50,
			offset * 50 + MAX_ST_PRINT_SIZE, 50);
      set_progression(gg, 20);
      if(print_st2==DOSTRIP2)
	l = get_samples(e, sebuf, 1, offset * 50,
			offset * 50 + MAX_ST_PRINT_SIZE, 50);
      set_progression(gg, 40);
      l = nofsamps/60;
      for(i=0; i<nofsamps; i++) {
	if(print_st1==DOSTRIP1)
	  fprintf(ef, "%d ", ebuf[i]);
	if(print_st2==DOSTRIP2)
	  fprintf(ef, "%d", sebuf[i]);
	fprintf(ef, "\n");
	set_progression(gg, 40 + (short)(i / l));
      }
    }
    fclose(ef);
    return;
  default:
    return;
  }
} while(1);
}


void strip_td_buttons(GWIN *gg)  /* for the tracing and the time-diagram */

{
  
cleardwin(gg, topd, PLANE_DWIN);
cleardwin(gg, upd, LOWERED_DWIN);
cleardwin(gg, lod, LOWERED_DWIN);

enter_graphic_block();
defbutton(gg, umctrld,   1,  2, 16, 15, "<<", 'S');
defbutton(gg, umctrld,  27,  2, 11, 15, "<",  's');
defbutton(gg, umctrld,  48,  2, 11, 15, ">",  'd');
defbutton(gg, umctrld,  69,  2, 16, 15, ">>", 'D');

defbutton(gg, umctrld,  100,  2, 16, 15, "++", '*');
defbutton(gg, umctrld,  126,  2, 11, 15, "+",  '+');
defbutton(gg, umctrld, 147,  2, 11, 15, "-",  '-');
defbutton(gg, umctrld, 168,  2, 16, 15, "--", '/');
defbutton(gg, umctrld, 199,  2, 11, 15, "^",  'H');
defbutton(gg, umctrld, 220,  2, 11, 15, "^",  'h');
defbutton(gg, umctrld, 241,  2, 11, 15, "#",  'r');
defbutton(gg, umctrld, 262,  2, 11, 15, "v",  'L');
defbutton(gg, umctrld, 283,  2, 11, 15, "v",  'l');

defbutton(gg, umctrld, 310,  2, 16, 15, "<>", 'w');
defbutton(gg, umctrld, 337,  2, 16, 15, "><", 'p');

defbutton(gg, umctrld, 369,  2, 11, 15, "V", 'v');

deftextpad(gg, umctrld, 411, 2, 10, 1, 10, timebuf, 'g');

/* Buttons that control the time diagram */

defbutton(gg, lctrld,   1,  2, 16, 15, "<<", 'K');
defbutton(gg, lctrld,  27,  2, 11, 15, "<",  'k');
defbutton(gg, lctrld,  48,  2, 11, 15, ">",  'j');
defbutton(gg, lctrld,  69,  2, 16, 15, ">>", 'J');

defbutton(gg, lctrld,  100,  2, 16, 15, "++", 'I');
defbutton(gg, lctrld,  126,  2, 11, 15, "+",  'i');
defbutton(gg, lctrld, 147,  2, 11, 15, "-",  'm');
defbutton(gg, lctrld, 168,  2, 16, 15, "--", 'M');

defbutton(gg, lctrld, 199,  2, 11, 15, "^",  'y');
defbutton(gg, lctrld, 220,  2, 11, 15, "R",  'R');
defbutton(gg, lctrld, 241,  2, 11, 15, "v",  'b');

defbutton(gg, lctrld, 267,  2, 16, 15, "<>", 'W');
defbutton(gg, lctrld, 294,  2, 16, 15, "><", 'P');

setspotshot(gg, lod, '&');
exit_graphic_block();

}

void right_td1_buttons(GWIN *gg)

{

/* Display the right side buttons */

cleardwin(gg, rjumps, PLANE_DWIN);

enter_graphic_block();
defbutton(gg, rjumps, 18,  23, 90, 20,
	  bylanguage("System", "Sistem", "", "", "", ""), 1);
defbutton(gg, rjumps, 18,  53, 90, 20,
	  bylanguage("Recording", "Inreg.", "", "", "", ""), 9);
defbutton(gg, rjumps, 18,  83, 90, 20,
	  bylanguage("Report", "Raport", "", "", "", ""), 3);
defbutton(gg, rjumps, 18, 113, 90, 20,
	  bylanguage("Events", "Evenimente", "", "", "", ""), 10);
defbutton(gg, rjumps, 18, 143, 90, 20,
	  bylanguage("Parameters", "Parametri", "", "", "", ""), 5);
defbutton(gg, rjumps, 18, 173, 90, 20,
	  bylanguage("ECG", "Traseu ECG", "", "", "", ""), 2);
defbutton(gg, rjumps, 18, 203, 90, 20,
	  bylanguage("TimeDiag", "Diag. Timp", "", "", "", ""), 4);
defbutton(gg, rjumps, 18, 233, 90, 20,
	  bylanguage("Scatter", "Corelograme", "", "", "", ""), 7);
defbutton(gg, rjumps, 18, 263, 90, 20,
	  bylanguage("HRV", "Variab. RS", "", "", "", ""), 6);
defbutton(gg, rjumps, 18, 293, 90, 20,
	  bylanguage("Special", "Special", "", "", "", ""), 8);
defbutton(gg, rjumps, 18, 323, 90, 20,
	  bylanguage("Help", "Ajutor", "", "", "", ""), 12);
defbutton(gg, rjumps, 18, 353, 90, 20,
	  bylanguage("Exit", "Termina", "", "", "", ""), 11);
defhbutton(gg, rjumps, 24);
exit_graphic_block();
}

/* </code>

   \section{The ``Strip Display'' dialog boxes}

   <code>
*/

void right_strdisplay_buttons(GWIN *gg)

{

  cleardwin(gg, rjumps, PLANE_DWIN);

  enter_graphic_block();  
  defbutton(gg, rjumps, 18,  23, 90, 20, "Markings", 1);
  defbutton(gg, rjumps, 18,  53, 90, 20, "Colors", 2);
  defbutton(gg, rjumps, 18,  83, 90, 20, "Upper Lead", 3);
  defbutton(gg, rjumps, 18, 113, 90, 20, "Lower Lead", 4);
  defbutton(gg, rjumps, 18, 143, 90, 20, "Back", 5);
  defhbutton(gg, rjumps, 24);
  exit_graphic_block();
}

void strlead_interraction(GWIN *gg, short *lead_var)

{
    short c;

/* IMPORTANT NOTE: the codes used in the following definitions
                   must match exactly the names and codes from
                   egdiagsr.c
 */
    cleardwin(gg, rjumps, PLANE_DWIN);

    enter_graphic_block();
    defaltswitch(gg, rjumps, 18,  23, 90, 20, "L0 (A)",
	      lead_var, 0);
    defaltswitch(gg, rjumps, 18,  53, 90, 20, "L1 (B)",
	      lead_var, 1);
    defaltswitch(gg, rjumps, 18,  83, 90, 20, "L1-L0 (C)",
	      lead_var, 4);
    defaltswitch(gg, rjumps, 18, 113, 90, 20, "I",
	      lead_var, 5);
    defaltswitch(gg, rjumps, 18, 143, 90, 20, "II",
	      lead_var, 6);
    defaltswitch(gg, rjumps, 18, 173, 90, 20, "III",
	      lead_var, 7);
    defaltswitch(gg, rjumps, 18, 203, 90, 20, "aVF",
	      lead_var, 8);
    defaltswitch(gg, rjumps, 18, 233, 90, 20, "aVL",
	      lead_var, 9);
    defaltswitch(gg, rjumps, 18, 263, 90, 20, "aVR",
              lead_var, 10);
    defbutton(gg, rjumps, 18, 293, 90, 20, "OK", 11);
    defbutton(gg, rjumps, 18, 323, 90, 20, "Cancel", 12);
    defhbutton(gg, rjumps, 24);
    exit_graphic_block();

    while((c=gwinteract(gg))!=11 && c!=12 && c!=24) ;
/*    if(c==24) {
      wi_set_eps_unit(700);
      wi_print_gwin("...");
    } */
    if(c==11)
      gwaccept(gg);

}

void strdisplay_interraction(GWIN *gg, PERFILE *p)

{
  char c;

  right_strdisplay_buttons(gg);
  /* Other buttons should be disactivated at this moment */
  while(((c=gwinteract(gg))<1 || c>5) && c!=24);
  /* may escape only by pressing one of the right buttons */
  switch(c) {
  case 24:
    wi_set_eps_unit(700);
    wi_print_gwin("stdisply.eps");
    break;
  case 1: /* Markings */
    cleardwin(gg, rjumps, PLANE_DWIN);

    enter_graphic_block();
    defswitch(gg, rjumps, 18,  23, 90, 20, "RR & HRate",
	      &rr_marking, 1);
    defswitch(gg, rjumps, 18,  53, 90, 20, "QRS detect",
	      &qrsdet_marking, 2);
    defswitch(gg, rjumps, 18,  83, 90, 20, "QRS ref",
	      &qrsref_marking, 3);
    defswitch(gg, rjumps, 18, 113, 90, 20, "ST level",
	      &st_marking, 4);
    defswitch(gg, rjumps, 18, 143, 90, 20, "Baseline",
	      &bl_marking, 5);
    defswitch(gg, rjumps, 18, 173, 90, 20, "J point",
	      &j_marking, 6);
    defswitch(gg, rjumps, 18, 203, 90, 20, "QRS length",
	      &qrslen_marking, 9);
    defswitch(gg, rjumps, 18, 233, 90, 20, "Morphology",
	      &morph_marking, 10);
    defbutton(gg, rjumps, 18, 263, 90, 20, "OK", 7);
    defbutton(gg, rjumps, 18, 293, 90, 20, "Cancel", 8);
    defhbutton(gg, rjumps, 24);
    exit_graphic_block();

    while((c=gwinteract(gg))!=7 && c!=8 && c!=24) ;
    if(c==24) {
      wi_set_eps_unit(700);
      wi_print_gwin("stmarks.eps");
    }
    if(c==7)
      gwaccept(gg);
    break;
  case 2: /* colors */
    cleardwin(gg, rjumps, PLANE_DWIN);
    adjust_colors(gg, "TdScreen-");
    /* This is the prefix for the resources to be used
       for this screen's colors */
    right_td1_buttons(gg);
    break;
  case 3:
    strlead_interraction(gg, &lead_2);
    break;
  case 4:
    strlead_interraction(gg, &lead_1);
    break;
  case 5: /* just go back */
    break;  
  default: /* can't possibly happen */
    break;
  }
  /* it remains the responsibility of the caller to clear the
     rjumps dwin and redraw it with its buttons */
}


/* </code>

   \section{The ``Strip Display'' dialog boxes}

   <code>
*/

short mean_f     = 1;
short max_f      = 0;
short min_f      = 0;
short pm_sigma_f = 0;
short sigma_f    = 0;
short histo_f    = 0;

void select_a_look(GWIN *gg)

{
  short c;

  cleardwin(gg, rjumps, PLANE_DWIN);
  
  enter_graphic_block();
  defswitch(gg, rjumps, 18,  23, 90, 20, "Mean",
	    &mean_f, 1);
  defswitch(gg, rjumps, 18,  53, 90, 20, "Max",
	    &max_f, 2);
  defswitch(gg, rjumps, 18,  83, 90, 20, "Min",
	    &min_f, 3);
  defswitch(gg, rjumps, 18, 113, 90, 20, "+/- Sigma",
	    &pm_sigma_f, 4);
  defswitch(gg, rjumps, 18, 143, 90, 20, "Sigma",
	    &sigma_f, 5);
  defswitch(gg, rjumps, 18, 173, 90, 20, "Histogram",
	    &histo_f, 6);
  defbutton(gg, rjumps, 18, 263, 90, 20, "OK", 7);
  defbutton(gg, rjumps, 18, 293, 90, 20, "Cancel", 8);
  defhbutton(gg, rjumps, 24);
  exit_graphic_block();

  while((c=gwinteract(gg))!=7 && c!=8 && c!=24) ;
  if(c==7)
    gwaccept(gg);
  if(c==24) {
    wi_set_eps_unit(700);
    wi_print_gwin("tdlook.eps");
  }
}

void right_timediag_buttons(GWIN *gg)

{

  cleardwin(gg, rjumps, PLANE_DWIN);

  enter_graphic_block();  
  defbutton(gg, rjumps, 18,  23, 90, 20, "L-type", 1);
  defbutton(gg, rjumps, 18,  53, 90, 20, "R-type", 2);
  defbutton(gg, rjumps, 18,  83, 90, 20, "L-look", 3);
  defbutton(gg, rjumps, 18, 113, 90, 20, "R-look", 4);
  defbutton(gg, rjumps, 18, 143, 90, 20, "L-filter", 5);
  defbutton(gg, rjumps, 18, 173, 90, 20, "R-filter", 6);
  defbutton(gg, rjumps, 18, 203, 90, 20, "Define", 7);
  defbutton(gg, rjumps, 18, 233, 90, 20, "Report", 8);
  defbutton(gg, rjumps, 18, 263, 90, 20, "Back", 9);
  defhbutton(gg, rjumps, 24);
  exit_graphic_block();

}

extern short td_show_min_f[];
extern short td_show_med_f[];
extern short td_show_max_f[];
extern short td_show_hist_f[];
extern short td_show_pm_sigma_f[];
extern short td_show_sigma_f[];

void timediag_interraction(GWIN *gg, PERFILE *p, long offs)

{
  char c;
  char cc;

  right_timediag_buttons(gg);
  /* Other buttons should be disactivated at this moment */
  while((c=gwinteract(gg))<1 && c>9 && c!=24);
  /* may escape only by pressing one of the right buttons */
  switch(c) {
  case 24:
    wi_set_eps_unit(700);
    wi_print_gwin("tdmenu.eps");
    break;
  case 3:
  case 4:
    mean_f     = td_show_med_f[c - 3]    ? 1 : 0;
    max_f      = td_show_max_f[c - 3]    ? 2 : 0;
    min_f      = td_show_min_f[c - 3]    ? 3 : 0;
    pm_sigma_f = td_show_pm_sigma_f[c - 3] ? 4 : 0;
    sigma_f    = td_show_sigma_f[c - 3]  ? 5 : 0;
    histo_f    = td_show_hist_f[c - 3]   ? 6 : 0;

    select_a_look(gg);
    
    td_show_med_f[c - 3] = (mean_f > 0);
    td_show_max_f[c - 3] = (max_f > 0);
    td_show_min_f[c - 3] = (min_f > 0);
    td_show_pm_sigma_f[c - 3] = (pm_sigma_f > 0);
    td_show_sigma_f[c - 3] = (sigma_f > 0);
    td_show_hist_f[c - 3] = (histo_f > 0);
    break;
  case 1:
  case 2:
  
  if(c==1)
    select_a_time_diagram(gg, td_type);
  else
    select_a_time_diagram(gg, tdplus_type);
    
  if(c==1) {
    td_type = sel_td;
    if(td_type!=0) {
      td_u_base = tds[sel_td].u_base;
      td_u_incr = tds[sel_td].u_incr;
      td_d_base = tds[sel_td].d_base;
      td_d_incr = tds[sel_td].d_incr;
    }
  }
  else {
    tdplus_type = sel_td;
    if(tdplus_type!=0) {
      tdplus_u_base = tds[sel_td].u_base;
      tdplus_u_incr = tds[sel_td].u_incr;
      tdplus_d_base = tds[sel_td].d_base;
      tdplus_d_incr = tds[sel_td].d_incr;
    }
  }
  break;
 case 8:
  make_td_print_screen(gg);
      print_td_interraction(gg, p, offs);
      cleardwin(gg, fulld, PLANE_DWIN);
      cleardwin(gg, rjumps, PLANE_DWIN);
      cleardwin(gg, 0, PLANE_DWIN);
      strip_td_buttons(gg);
      if(rflag)
	draw_ecg_grid(gg, upd, 500, 280, 4);
      ecg_display(gg, upd, p, crtoffs, crtxdiv, crtyscale, crtyshift);
      break;

    /* the above gets completed with the behaviour
       found after the return */
    /*

    switch(td_res) {
    case PIX4SEC1:
      set_td_resolution(2500);
      set_td_graphic_step(1.0);
      set_step_td_filtering();
      set_td_filter_width(1);
      break;
    case PIX2SEC1:
      set_td_resolution(2500);
      set_td_graphic_step(0.5);
      set_step_td_filtering();
      set_td_filter_width(1);
      break;
    }
    put_td_to_latex_list(p, crt_td_org);
    */
    break;
  default: 
    break;
  }
  /* it remains the responsibility of the caller to clear the
     rjumps dwin and redraw it with its buttons */
}

/*
   </code>

   \section{The actual main interraction function}

   <code> */


td_interraction(GWIN *gg, PERFILE *thep)

{

char b[4];
short c;
short sp;
long cbt;
long cblen;

ECG *ec;

short pp = 0;
short swip = 0;
short swit = 1;
char topline[80];

setresprefix("TdScreen-");
res2color();
part_td_screen(gg);

/* Display now the control buttons for the upper and medium dwins */

strip_td_buttons(gg);

right_td1_buttons(gg);

ecg_display(gg, upd, thep, crtoffs, crtxdiv, crtyscale, crtyshift);
if (gwerrno) return INTERRACTION_EXIT;  
  /* return mode the interraction system */

/*
display_td(gg, lod, thep, crt_td_org = 50000);
*/

crt_td_org = 50000;

/* The dimension of the recording on the top line */

printf("Total file number of samples: %ld\n", rec_file_size(thep));
cbt = rec_file_duration(thep);
printf("Total file duration: %ld\n", cbt);
sprintf(topline, "%s total: %02ld:%02ld:%02ld", rec_file_name(thep),
	cbt / 36000000, (cbt % 36000000) / 600000, (cbt % 600000) / 10000);

gsetcolor(gg, topd, 1);
gtext(gg, topd, 1, 1, topline);

/* The gwinteract loop */

sprintf(timebuf, "%02ld:%02ld:%02ld", crtoffs / 720000,
        (crtoffs % 720000) / 12000, (crtoffs % 12000) / 200);
gset_tp_contents(gg, umctrld, 'g', timebuf);

       switch(td_res) {
       case PIX4SEC1:
	 cbt = 600000;
	 cblen = 1250000;
	 break;
       case PIX2SEC1:
	 cbt = 1200000;
	 cblen = 2500000;
	 break;
       case PIX1SEC1:
	 cbt = 2400000;
	 cblen = 5000000;
	 break;
       case PIX1SEC10:
	 cbt = 24000000;
         cblen = 50000000;
	 break;
       case PIX1MIN1:
       case PIX1MIN4:
	 cbt = 144000000;
	 break;
       default:
	 cbt = 0;
	 break;
       }

while((c=gwinteract(gg))!=11)
  {
  if (gwerrno) return INTERRACTION_EXIT;
  switch(c)
    {
    case 24:
      wi_set_eps_unit(700);
      wi_print_gwin("mainscr.eps");
      break;
    case 'd':
      cleardwin(gg, upd, LOWERED_DWIN);
      if(rflag) draw_ecg_grid(gg, upd, 500, 280, 4);
      if(crtoffs<rec_file_size(thep)-200) {
	crtoffs += 200;
	ecg_display(gg, upd, thep, crtoffs, crtxdiv, crtyscale, crtyshift);
      }
      break;
    case 'D':
      cleardwin(gg, upd, LOWERED_DWIN);
      if(rflag) draw_ecg_grid(gg, upd, 500, 280, 4);
      if(crtoffs<rec_file_size(thep)-500) {
	crtoffs += 500;
	ecg_display(gg, upd, thep, crtoffs, crtxdiv, crtyscale, crtyshift);
      }
      break;
    case 'g':
      gwaccept(gg);
      crtoffs = gtimeparse(timebuf);
      cleardwin(gg, upd, LOWERED_DWIN);
      #ifdef MAINTRACE
      fprintf(ecgnosislog, "     The g option was called for %s.\n", timebuf);
      fflush(ecgnosislog);
      #endif
      if(rflag) draw_ecg_grid(gg, upd, 500, 280, 4);
      ecg_display(gg, upd, thep, crtoffs, crtxdiv, crtyscale, crtyshift);
      break;
    case 's':
      cleardwin(gg, upd, LOWERED_DWIN);
      if(rflag) draw_ecg_grid(gg, upd, 500, 280, 4);
      if(crtoffs>200)
        crtoffs -= 200;
      else
        crtoffs = 0;
      ecg_display(gg, upd, thep, crtoffs, crtxdiv, crtyscale, crtyshift);
      break;
    case 'S':
      cleardwin(gg, upd, LOWERED_DWIN);
      if(rflag) draw_ecg_grid(gg, upd, 500, 280, 4);
      if(crtoffs>500)
        crtoffs -= 500;
      else
        crtoffs = 0;
      ecg_display(gg, upd, thep, crtoffs, crtxdiv, crtyscale, crtyshift);
      break;
    case '+':
      cleardwin(gg, upd, LOWERED_DWIN);
      crtyscale *= 1.2;
      if(rflag) draw_ecg_grid(gg, upd, 500, 280, 4);
      ecg_display(gg, upd, thep, crtoffs, crtxdiv, crtyscale, crtyshift);
      break;
    case '-':
      cleardwin(gg, upd, LOWERED_DWIN);
      crtyscale /= 1.2;
      if(rflag) draw_ecg_grid(gg, upd, 500, 280, 4);
      ecg_display(gg, upd, thep, crtoffs, crtxdiv, crtyscale, crtyshift);
      break;
    case '*':
      cleardwin(gg, upd, LOWERED_DWIN);
      crtyscale *= 1.5;
      if(rflag) draw_ecg_grid(gg, upd, 500, 280, 4);
      ecg_display(gg, upd, thep, crtoffs, crtxdiv, crtyscale, crtyshift);
      break;
    case '/':
      cleardwin(gg, upd, LOWERED_DWIN);
      crtyscale /= 1.5;
      if(rflag) draw_ecg_grid(gg, upd, 500, 280, 4);
      ecg_display(gg, upd, thep, crtoffs, crtxdiv, crtyscale, crtyshift);
      break;
    case 'h':
      cleardwin(gg, upd, LOWERED_DWIN);
      if(rflag) draw_ecg_grid(gg, upd, 500, 280, 4);
      strip_1_shift -= 20;
      ecg_display(gg, upd, thep, crtoffs, crtxdiv, crtyscale, crtyshift);
      break;
    case 'H':
      cleardwin(gg, upd, LOWERED_DWIN);
      if(rflag) draw_ecg_grid(gg, upd, 500, 280, 4);
      strip_2_shift -= 20;
      ecg_display(gg, upd, thep, crtoffs, crtxdiv, crtyscale, crtyshift);
      break;
    case 'l':
      cleardwin(gg, upd, LOWERED_DWIN);
      if(rflag) draw_ecg_grid(gg, upd, 500, 280, 4);
      strip_1_shift += 20;
      ecg_display(gg, upd, thep, crtoffs, crtxdiv, crtyscale, crtyshift);
      break;
    case 'L':
      cleardwin(gg, upd, LOWERED_DWIN);
      if(rflag) draw_ecg_grid(gg, upd, 500, 280, 4);
      strip_2_shift += 20;
      ecg_display(gg, upd, thep, crtoffs, crtxdiv, crtyscale, crtyshift);
      break;
    case 'r':
      cleardwin(gg, upd, LOWERED_DWIN);
      rflag = !rflag;
      if(rflag) draw_ecg_grid(gg, upd, 500, 280, 4);
      ecg_display(gg, upd, thep, crtoffs, crtxdiv, crtyscale, crtyshift);
      break;
    case 'w':
      cleardwin(gg, upd, LOWERED_DWIN);
      switch(crtxdiv)
        {
        case 60: crtxdiv = 10; break;
        case 20: crtxdiv = 10; break;
        case 10: crtxdiv =  4; break;
        case  2:
        case  4: 
        default: crtxdiv =  2; break;
        }
      if(rflag) draw_ecg_grid(gg, upd, 500, 280, 4);
      ecg_display(gg, upd, thep, crtoffs, crtxdiv, crtyscale, crtyshift);
      break;
    case 1:
      file_interaction(gg);
      cleardwin(gg, fulld, PLANE_DWIN);
      cleardwin(gg, rjumps, PLANE_DWIN);
      cleardwin(gg, 0, PLANE_DWIN);
      strip_td_buttons(gg);
      right_td1_buttons(gg);
      if(rflag)
	draw_ecg_grid(gg, upd, 500, 280, 4);
      ecg_display(gg, upd, thep, crtoffs, crtxdiv, crtyscale, crtyshift);
      display_td(gg, lod, thep, crt_td_org);
      break;
    case 2:
      strdisplay_interraction(gg, thep);
      right_td1_buttons(gg);
      cleardwin(gg, upd, LOWERED_DWIN);
      if(rflag)
	draw_ecg_grid(gg, upd, 500, 280, 4);
      ecg_display(gg, upd, thep, crtoffs, crtxdiv, crtyscale, crtyshift);
      break;
    case 3:
      make_print_screen(gg);
      print_screen_interraction(gg, thep, crtoffs);
      cleardwin(gg, fulld, PLANE_DWIN);
      cleardwin(gg, rjumps, PLANE_DWIN);
      cleardwin(gg, 0, PLANE_DWIN);
      strip_td_buttons(gg);
      right_td1_buttons(gg);
      if(rflag)
	draw_ecg_grid(gg, upd, 500, 280, 4);
      ecg_display(gg, upd, thep, crtoffs, crtxdiv, crtyscale, crtyshift);
      display_td(gg, lod, thep, crt_td_org);
      break;
    case 4:
      timediag_interraction(gg, thep, crt_td_org);
      right_td1_buttons(gg);
      cleardwin(gg, lod, LOWERED_DWIN);
      display_td(gg, lod, thep, crt_td_org);
      break;
    case 5:
      morph_interraction(gg, thep);
      cleardwin(gg, scatd, PLANE_DWIN);
      cleardwin(gg, fulld, PLANE_DWIN);
      cleardwin(gg, rjumps, PLANE_DWIN);
      cleardwin(gg, 0, PLANE_DWIN);
      strip_td_buttons(gg);
      right_td1_buttons(gg);
      if(rflag)
	draw_ecg_grid(gg, upd, 500, 280, 4);
      ecg_display(gg, upd, thep, crtoffs, crtxdiv, crtyscale, crtyshift);
      display_td(gg, lod, thep, crt_td_org);
      break;
    case 6:
      hrv_interaction(gg, thep, crt_td_org);
      cleardwin(gg, 0, PLANE_DWIN);
      cleardwin(gg, rjumps, PLANE_DWIN);
      strip_td_buttons(gg);
      right_td1_buttons(gg);
      if(rflag)
	draw_ecg_grid(gg, upd, 500, 280, 4);
      ecg_display(gg, upd, thep, crtoffs, crtxdiv, crtyscale, crtyshift);
      display_td(gg, lod, thep, crt_td_org);
      break;
    case 7:
      scat_interraction(gg, thep);
      setresprefix("TdScreen-");
      res2color();
      cleardwin(gg, fulld, PLANE_DWIN);
      cleardwin(gg, rjumps, PLANE_DWIN);
      cleardwin(gg, 0, PLANE_DWIN);
      strip_td_buttons(gg);
      right_td1_buttons(gg);
      if(rflag)
	draw_ecg_grid(gg, upd, 500, 280, 4);
      ecg_display(gg, upd, thep, crtoffs, crtxdiv, crtyscale, crtyshift);
      display_td(gg, lod, thep, crt_td_org);
      break;

    case 8:
      special_interaction(gg, thep);
      cleardwin(gg, rjumps, PLANE_DWIN);
      cleardwin(gg, 0, PLANE_DWIN);
      strip_td_buttons(gg);
      right_td1_buttons(gg);
      if(rflag)
	draw_ecg_grid(gg, upd, 500, 280, 4);
      ecg_display(gg, upd, thep, crtoffs, crtxdiv, crtyscale, crtyshift);
      display_td(gg, lod, thep, crt_td_org);
      break;

    case 9:
      rec_interaction(gg);
      cleardwin(gg, rjumps, PLANE_DWIN);
      cleardwin(gg, 0, PLANE_DWIN);
      strip_td_buttons(gg);
      right_td1_buttons(gg);
      if(rflag)
	draw_ecg_grid(gg, upd, 500, 280, 4);
      ecg_display(gg, upd, thep, crtoffs, crtxdiv, crtyscale, crtyshift);
      display_td(gg, lod, thep, crt_td_org);
      break;
    case 12:
      break;
    case 'p':
      cleardwin(gg, upd, LOWERED_DWIN);
      switch(crtxdiv)
        {
        case 60: crtxdiv = 10; break;
        case 20: crtxdiv = 10; break;
        case 10: crtxdiv = 10; break;
        case  2: crtxdiv =  4; break;
        case  4: crtxdiv = 10; break;
        default: crtxdiv =  2; break;
        }
      if(crtxdiv==60)
        crtxdiv = 20;
      if(rflag) draw_ecg_grid(gg, upd, 500, 280, 4);
      ecg_display(gg, upd, thep, crtoffs, crtxdiv, crtyscale, crtyshift);
      break;
    case 'v':
      crt_td_org = crtoffs*50;
      if(crt_td_org<50000)
	crt_td_org = 50000;
      cleardwin(gg, lod, LOWERED_DWIN);
      display_td(gg, lod, thep, crt_td_org);
      break;
      case 'k':
    case 'j':
      switch(td_res) {
      case PIX4SEC1:
        cbt = 600000;
	cblen = 1250000;
        break;
      case PIX2SEC1:
        cbt = 1200000;
	cblen = 2500000;
        break;
      case PIX1SEC1:
        cbt = 2400000;
	cblen = 5000000;
        break;
      case PIX1SEC10:
        cbt = 24000000;
	cblen = 50000000;
        break;
      case PIX1MIN1:
      case PIX1MIN4:
        cbt = 144000000;
        break;
      default:
        cbt = 0;
        break;
      }
      if(c=='k') {
        cleardwin(gg, lod, LOWERED_DWIN);
        if(crt_td_org>cbt+50000)
          crt_td_org -= cbt;
        else
          crt_td_org = 50000;
        display_td(gg, lod, thep, crt_td_org);
        break;
      }
      if(c=='j') {
        cleardwin(gg, lod, LOWERED_DWIN);
        if(crt_td_org<rec_file_duration(thep) - 2 * cbt)
          crt_td_org += cbt;
        else
          crt_td_org = rec_file_duration(thep) - cbt - 100000;
        display_td(gg, lod, thep, crt_td_org);
        break;
      }
      break;  /* this can't be reached */
    case 'K':
    case 'J':
      switch(td_res) {
      case PIX4SEC1:
        cbt = 1200000;
        break;
      case PIX2SEC1:
        cbt = 2400000;
        break;
      case PIX1SEC1:
        cbt = 4800000;
        break;
      case PIX1SEC10:
        cbt = 48000000;
        break;
      case PIX1MIN1:
      case PIX1MIN4:
        cbt = 288000000;
        break;
      default:
        cbt = 0;
        break;
      }
      if(c=='K') {
        cleardwin(gg, lod, LOWERED_DWIN);
        if(crt_td_org>cbt+50000)
          crt_td_org -= cbt;
        else
          crt_td_org = 50000;
        display_td(gg, lod, thep, crt_td_org);
        break;
      }
      if(c=='J') {
        cleardwin(gg, lod, LOWERED_DWIN);
        if(crt_td_org<rec_file_duration(thep) - 2 * cbt)
          crt_td_org += cbt;
        else
          crt_td_org = rec_file_duration(thep) - cbt - 100000;
        display_td(gg, lod, thep, crt_td_org);
        break;
      }
      break;  /* this can't be reached */
    case 'y':
      cleardwin(gg, lod, LOWERED_DWIN);
      if(td_u_base > 10)
        td_u_base -= 10;
      else
        td_u_base = 0;
      display_td(gg, lod, thep, crt_td_org);
      break;
    case 'b':
      cleardwin(gg, lod, LOWERED_DWIN);
      td_u_base += 10;
      display_td(gg, lod, thep, crt_td_org);
      break;
    case 'R':
      cleardwin(gg, lod, LOWERED_DWIN);
      td_u_base = 0;
      td_d_base = 0;
      td_u_incr = 20;
      td_d_incr = 10;
      display_td(gg, lod, thep, crt_td_org);
      break;
    case 'I':
    case 'i':
      cleardwin(gg, lod, LOWERED_DWIN);
      switch(td_u_incr) {
      case 10:
        td_u_incr = 5;
        break;
      case 5:
      case 2:
        td_u_incr = 2;
        break;
      case 20:
        td_u_incr = 10;
        break;
      case 40:
        td_u_incr = 20;
        break;
      default:
        td_u_incr = 20;
        break;
      }
      display_td(gg, lod, thep, crt_td_org);
      break;
    case 'M':
    case 'm':
      cleardwin(gg, lod, LOWERED_DWIN);
      switch(td_u_incr) {
      case 10:
        td_u_incr = 20;
        break;
      case 5:
        td_u_incr = 10;
        break;
      case 2:
        td_u_incr = 5;
        break;
      case 20:
      case 40:
        td_u_incr = 40;
        break;
      default:
        td_u_incr = 20;
        break;
      }
      display_td(gg, lod, thep, crt_td_org);
      break;
    case 'P':
      switch(td_res) {
      case PIX4SEC1:
        td_res = PIX2SEC1;
        break;
      default:
        td_res = PIX2SEC1;
        break;
      }
      cleardwin(gg, lod, LOWERED_DWIN);
      display_td(gg, lod, thep, crt_td_org);
      break;
    case 'W':
      switch(td_res) {
      case PIX2SEC1:
        td_res = PIX4SEC1;
        break;
      default:
        td_res = PIX4SEC1;
        break;
      }
      cleardwin(gg, lod, LOWERED_DWIN);
      display_td(gg, lod, thep, crt_td_org);
      break;
    case '&':
      switch(td_res) {
      case PIX4SEC1:
        cbt = 2500;
        break;
      case PIX2SEC1:
        cbt = 5000;
        break;
      case PIX1SEC1:
        cbt = 10000;
        break;
      case PIX1SEC10:
        cbt = 100000;
        break;
      case PIX1MIN1:
        cbt = 600000;
        break;
      case PIX1MIN4:
        cbt = 2400000;
        break;
      default:
        cbt = 5000;
        break;
      }
      crtoffs = (td_start + (long)(xspotshot - 30) * cbt) / 50;  /* crtoffs is in samples */
      cleardwin(gg, upd, LOWERED_DWIN);
      if(rflag) draw_ecg_grid(gg, upd, 500, 280, 4);
#ifdef MAINTRACE
      fprintf(ecgnosislog, "SPOTSHOT sets crtoffs=%ld\n", crtoffs);
      fflush(ecgnosislog);
#endif
      ecg_display(gg, upd, thep, crtoffs, crtxdiv, crtyscale, crtyshift);
      break;
    default:
      break;
    }
  sprintf(timebuf, "%02ld:%02ld:%02ld", crtoffs / 720000,
          (crtoffs % 720000) / 12000, (crtoffs % 12000) / 200);
  gset_tp_contents(gg, umctrld, 'g', timebuf);
  }
}


/*--
  \end{code}

  \endoffile

  --*/

