/*--

\FileTitle{
{\tt glikalys.c} --- signal analysis for the ECGNOSIS program
}
\CopyrightNotice{

    Copyright (c) 1991--1996 Alexandru Dan Corlan MD (http://dan.corlan.net)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

}

\begin{code}

--*/

#include <stdio.h>
#include <math.h>

#include "gliks.h"

extern FILE *ecgnosislog;

/*--
  \end{code}

  \subsection{QRS Detection}

  \begin{code}
  --*/


short thethr;                                                   /* [VAR] */ 

void qrs_detect_init(PERFILE *p, short *strip, short stril) /* [FUN] */ 

{
  thethr = asses_qrs_threshold(strip, stril);
}

long ecg_sample_size(ECG *e) { /* [FUN] */ 
  return 50;
}


long next_qrs(PERFILE *p, short *strip, long stril, short x) /* [FUN] */ 

{
if(x==-1) {
  qrs_detect_init(p, strip, stril);
  x = 0;
}
if(x==0)
  return _next_qrs(strip, stril, &thethr);
if(x>0) {
  thethr = x;
  return _next_qrs(strip, stril, &thethr);
}
return -1;
}

long min_complex_size(PERFILE *p) /* [FUN] */ 

{
return 2000;
}



static inline short _qrs_discrim(short *strip)                   /* [FUN] */ 

{
  short r, rr;
  
  r = strip[3] - strip[1];
  if (r<0)
    r = -r;
  rr = strip[4] - 2*strip[2] + strip[0];
  if (rr<0)
    rr = -rr;
  rr = 11 * rr + 13 * r;
  return rr;
}

short qrs_discrim(short *s)

{
  return _qrs_discrim(s);
}

#define THR_HIST_SIZE 10                                        /* [DEF] */ 

short thr_history[THR_HIST_SIZE];                               /* [VAR] */ 
short thr_h_i = 0;                                              /* [VAR] */ 

short asses_qrs_threshold(short *strip, short stril)            /* [FUN] */ 

{
  short i;
  short ma, lo;

  ma = 0;
  for(i=0; i<stril-4; i++) {
    lo = _qrs_discrim(strip + i);
    if(lo>300) continue;
    if(ma<lo)
      ma = lo;
  }
  for(i=0; i<THR_HIST_SIZE; i++)
    thr_history[i] = ma/2;
  thr_h_i = 0;
  return ma/2;
}


/* the is the old value of the threshold and will be adjusted by the 
 * function call */

static short lastrr = 100;                                      /* [VAR] */ 

short _next_qrs(short *strip, short stril, short *thr)          /* [FUN] */

{
  short i, lo;
  short j, ac;
  short fact, lthr;
  short its, nor;

  nor = (7 * lastrr + 280) / 10 - 60;
  for(i=0; i<stril-8; i++) {
    lo = _qrs_discrim(strip + i);
    fact = 10;
    if(i>nor)
      fact = 7;
    lthr = *thr * fact / 10;
    if(lo>=lthr) {
      its = 0;
      for(j=0; j<30; j++)
        if(_qrs_discrim(strip+i+j)>=lthr)
          its ++;
      if(its>3) { /* we have a complex */
        for(j=0; j<20; j++)
          if((ac=_qrs_discrim(strip+i+j))>lo) 
            lo = ac;
/*
#ifdef MAINTRACE
  fprintf(ecgnosislog, "          lthr = %d\n", lthr);
  fflush(ecgnosislog);
#endif
*/
        i += j;
        if (lo < 10 * (*thr)) {
          lthr = 0;
          for(j=0; j<THR_HIST_SIZE; j++)
            lthr += thr_history[j];
          *thr = (lthr + (lo * 5) / 10) / (THR_HIST_SIZE + 1);
          thr_history[thr_h_i++] = *thr;
          if(thr_h_i>=THR_HIST_SIZE)
            thr_h_i = 0;
        }
        return lastrr = i;
      }
      i+=8;
      continue;
    }
  }
  return -1;   /* no answer */
}



/*--
  \end{code}

  \subsection{ST detection}

  \begin{code}
  --*/



/*--
  \end{code}

  \funparagraph{preceding\_baseline}

  \begin{code}
  --*/


#define MAX_NO_LEVELS 100

short preceding_baseline(short *strip, short stril, 		/* [FUN] */ 
		    short qrspos, 				/* [FUN] */ 
		    short samplesize) 				/* [FUN] */ 

{
  short lbefore;   /* from where to start before qrs position */
  short i;
  short *str;
  short min, max, med;
  short r, mrv;

  static short sfreq[MAX_NO_LEVELS];

  lbefore = 3000 / samplesize;  /* 0.3s before the start of the complex */
  max = -30000;
  min = 30000;
  str = strip+qrspos-lbefore;
  for(i=0; i<lbefore; i++) {
    if(str[i]<min)
      min = str[i];
    if(str[i]>max)
      max = str[i];
  }
  med = (min+max) / 2;
/*  fprintf(ecgnosislog, "MAX=%d MIN=%d lbefore=%d qrspos=%d ", max, min,
	  lbefore, qrspos);*/
  if(max-min > MAX_NO_LEVELS) {
    max = med + MAX_NO_LEVELS/2 - 1;
    min = med - MAX_NO_LEVELS/2 + 1;
  }
  for(i=0; i<max-min; i++)
    sfreq[i] = 0;
  for(i=0; i<lbefore; i++) 
    if(str[i]>min && str[i]<max)
      sfreq[str[i] - min]++;
  r = -1;
  mrv = 0;
  for(i=0; i<max-min; i++)
    if(sfreq[i]>mrv) {
      mrv = sfreq[i];
      r = min+i;
    }
/*  fprintf(ecgnosislog, "Baseline at %d ", r);*/
  return r;  /* BUT it could be -1, without any error */
             /* This is tricky design here */
}


/*--
  \end{code}

  \funparagraph{locate\_j\_point}

  \ImplementationNotes{
  The maximum value of the first derivative for the complex is computed
  (for 0.10s before the detection point and for 0.10s after the detection
  point). The first point for which the derivative becomes at least 
  3 times smaller (in absolute magnitude) is considered to be the
  J~point. 
  }

  \begin{code}
--*/

static short _last_qrs_len;

short locate_j_point(short *strip, short stril, 		/* [FUN] */ 
		     short qrspos, 				/* [FUN] */ 
		     short samplesize) 				/* [FUN] */ 

{
  short from, to;
  short i, s;
  short d, maxd;
  short jflag, maxpos;

  from = qrspos - 1000 / samplesize;  /* 0.1s before and  0.1s after
                                         after the detection point */
  to = qrspos + 1000 / samplesize;
  maxd = 0;
  maxpos = qrspos;
  for(i=from; i<to-2; i++) {
    d = strip[i]-strip[i+2];
    if(d<0)
      d = -d;
    if(d>maxd) {
      maxd = d;
      maxpos = i;
    }
  }
  to = qrspos + 3000/samplesize;   /* region in which J must be detected */
  jflag = 0;
/*  fprintf(ecgnosislog, "from=%d, to=%d, maxd=%d, ", from, to, maxd);*/
  for(i=maxpos; i<to; i++) {
    d = strip[i]-strip[i+2];
    if(d<0)
      d = -d;
    if(d>(maxd/5))
      jflag = 0;
    else
      jflag ++;
    if(jflag>2)
      break;
  }
  to = qrspos - 3000/samplesize;
  for(s=maxpos; s>to; s--) {
    d = strip[s]-strip[s+2];
    if(d<0)
      d = -d;
    if(d>maxd/5)
      jflag = 0;
    else
      jflag ++;
    if(jflag>2)
      break;
  }
  _last_qrs_len = i-s-3;
  if(jflag<2) /* NOT FOUND => noise */
    return -1;
/*  fprintf(ecgnosislog, "J point at %d.\n", i - qrspos);*/
  return i - qrspos;
}

short last_qrs_len()

{
  return _last_qrs_len;
}

/* The st would be the difference between the baseline and
 * the position situated at a given distance from the J~point
 */



/*--
  \end{code}

  \subsection{Classification of complexes}

  \begin{code}
  --*/


/********* THE COMPLEX CLASSIFICATION PACKAGE *****************/

/*

Two gliks are assumed to be part of a class (the same class)
if, after o process of ``superimposing'' the sum of quadratic differences
between the gliks is lower than a given threshold. 

By superimposing we consider the folowing process: assuming that the
gliks are similar, a moment of syncronisation (usually the maximum 
value of both gliks) is found. The ``amplitude'' of each of the gliks
is assesed, by making the algebric difference between the maximum and
the minimum values. The ``matching glik'' is then scaled in an attempt
to match the ``model glik''. The SQD (sum of quadratic differences) is then
computed while sliding the matching and the model, slightly, one with
respect to the other, and the minimum value of those sums taken into
account. {\it OBSERVATION: we know that the QRS complex is far more
stable in time than the T wave and the ST segment. We probably should
only match the QRS complexes for minimal differences, otherwise, with
a ``large T'' QRSes of match and model may become
unsyncronised}.

*/

/* #include <graph.h> */

GLIKMODEL gmc;   /* cursor --- the current complex */           /* [VAR] */ 

/* The gmsample function takes as argument a |glikmodel|, which it fills
with data from the argument |strip|; the rest of the information in the
glikmodel is also computed (amplitude, position of the maximum and of the
minimum, etc.).

The end of the complex raises the most important problems. For the beginning,
it is taken to start from 0.3s before the current QRS and 0.3s
before the next QRS.

ADAPTING to ECGNOSIS V2.0

The QRS complex is considered to start 0.15s before the reference
point and to end 0.15s after the reference point.

The reference point must be situated inside the QRS,
so that with some detection algorithms a correction must
be used when calling this function.

*/

void gmsample(GLIKMODEL *gm,                                    /* [FUN] */ 
              short *strip, short stril,                        /* [FUN] */ 
              short pos)                                        /* [FUN] */ 

/* pos must be at least 60 */

{
short i;
short m;
short mi;
short maxlim;

for (i=0; i<60; i++)  /* magic 60 = 2 * 0.15s, assumed < GLIKMODELSIZE */
  gm->gm[i] = strip[pos-30+i];  /* magic 30 = 0.15s at 200Hz sampling rate */

gm->gml = i;
m = -30000; /* can't be smaller */
mi = 30000;
for(i=0; i<gm->gml; i++) 
  {
  if(mi>gm->gm[i])
    {
    mi = gm->gm[i];
    gm->minpos = i;
    }
  if(m<gm->gm[i])
    {
    m = gm->gm[i];
    gm->maxpos = i;
    }
  }
gm->amplitude = gm->gm[gm->maxpos] - gm->gm[gm->minpos];
gm->gtype = UNKN;
gm->ngliks = 1;
return ;
}

#define S_COEFICIENT 1
#define D_COEFICIENT 2

long der_sqd(GLIKMODEL *mod, GLIKMODEL *mat)

{
  long matq;
  short minmat, minmod;
  long r;
  short i, j;
  long s, d;
  short from, to;

  matq = (mod->amplitude * (long) 100) / mat->amplitude;
  
  if(mod->minpos<mat->minpos) {
    from = mod->minpos;
    if(from>15)
      from = 15;
    to = 60 - mat->minpos; /* magic 60 = 2 * 0.15s */
    if(to>10)
      to = 15;
  }
  else {
    from = mat->minpos;
    if(from>15)
      from = 15;
    to = 60 - mod->minpos;
    if(to>15)
      to = 15;
  }
  r = 0;

  minmod = mod->gm[mod->minpos];
  minmat = mat->gm[mat->minpos];

  for(i=mod->minpos - from; i<mod->minpos+to-2; i++) {
    j = i - mod->minpos + mat->minpos;
    s = (mod->gm[i] - minmod) * 100 - (mat->gm[j] - minmat) * matq; /* BUG */
    if(abs(s)>10000)
      s = 10000;  /* to avoid unnecessary overflows */
    s = s*s;
    d = (mod->gm[i] - mod->gm[i+2]) * 100 -
      (mat->gm[j] - mat->gm[j+2]) * matq;
    if(abs(s)>10000)
      d = 10000;  /* to avoid unnecessary overflows */
    d = d*d;
    r += (S_COEFICIENT * s + D_COEFICIENT * d) / 30000;
  }
  return r;
}

short ww[60] = {
  1, 2, 3, 4, 5, 6, 7, 8, 9, 10,
  10, 10, 10, 10, 10, 10, 10, 10,
  10, 9, 8, 7, 6, 5, 4, 3, 2, 1
  };

long der_corel(GLIKMODEL *mod, GLIKMODEL *mat)

{
  long matq;
  short minmat, minmod;
  long r;
  short i, j;
  long s, d;
  short from, to;
  long xm, ym;
  long nmr, nm1, nm2;
  double intm;

  matq = (mod->amplitude * (long) 100) / mat->amplitude;
  
  if(mod->minpos<mat->minpos) {
    from = mod->minpos;
    if(from>15)
      from = 15;
    to = 60 - mat->minpos; /* magic 60 = 2 * 0.15s */
    if(to>10)
      to = 15;
  }
  else {
    from = mat->minpos;
    if(from>15)
      from = 15;
    to = 60 - mod->minpos;
    if(to>15)
      to = 15;
  }
  r = 0;

  minmod = mod->gm[mod->minpos];
  minmat = mat->gm[mat->minpos];

  xm = ym = 0;
  for(i=mod->minpos - from; i<mod->minpos+to-2; i++) {
    j = i - mod->minpos + mat->minpos;
    xm += mod->gm[i];
    ym += mat->gm[j];
  }
  xm /= to+from-2;
  ym /= to+from-2;

  nmr = nm1 = nm2 = 0;
  for(i=mod->minpos - from; i<mod->minpos+to-2; i++) {
    j = i - mod->minpos + mat->minpos;
    nmr += (mod->gm[i] - xm) * (mat->gm[j] - ym);
    nm1 += (mod->gm[i] - xm) * (mod->gm[i] - xm);
    nm2 += (mat->gm[j] - ym) * (mat->gm[j] - ym);
  }

  fprintf(ecgnosislog, "from = %d, to = %d\n", from, to);
  fflush(ecgnosislog);
  fprintf(ecgnosislog, "nmr=%d, nm1=%d, nm2=%d\n", nmr, nm1, nm2);
  fflush(ecgnosislog);
  intm = nmr;
  intm *= nmr;
  intm /= nm1;
  intm /= nm2;
  fprintf(ecgnosislog, "Intm = %10.3f\n", intm);
  fflush(ecgnosislog);

  r = (long)(100.0 * intm);
  return r;
}


void clear_morphologies(PERFILE *p)

{
  p->gmofill = 0;
}

short morphology(PERFILE *p, short *strip, short stril,
		 short qrspos, /* corrected to be inside a QRS */
		 long threshold
		 )

{
  short i;
  long s;

  return 1;
  threshold = 80;
  if(p->gmofill>=NUM_GMOS-1)
    return -1;
  gmsample(p->gmo + p->gmofill, strip, stril, qrspos);

#ifdef GENERATE_MATLAB_FILE
  fprintf(ecgnosislog, "MATLAB: ");
  for(i=p->gmo[p->gmofill].minpos - 20; i<p->gmo[p->gmofill].minpos + 20; i++)
    fprintf(ecgnosislog, " %4d", p->gmo[p->gmofill].gm[i]);
  fprintf(ecgnosislog, "\n");
#endif
  
  for(i=2; i<p->gmofill; i++) {
    s = der_corel(p->gmo+i, p->gmo + p->gmofill);

    fprintf(ecgnosislog,"Morphology difference = %d\n", s);
    fflush(ecgnosislog);

    if(s>threshold && i>0) {
      p->gmo[i].ngliks ++;
      return i;
    }
  }
  fprintf(ecgnosislog, "New morhology #%d\n", i);
  fflush(ecgnosislog);
  p->gmo[i].supclass = i;
  p->gmofill ++;
  return i;
}

#ifdef NEVER_DEFINED_BY_ANYBODY

/*-- 
\end{code}

% \section{OBSOLETE CODE, MAYBE NOT TO BE USED ANYMORE}
% 
% 
% The following function takes two glimodels, named ``model'' and
% ``match''. It tries to see if the match matches the model, that is it
% computes the SQD. The smallest length of the two items is taken into
% account.
% 
% OBSERVATION: a future implementation may attempt to see if the first
% and the last parts of the current glik are so different that they must
% be ommited; such an approach would outrule parts from other nearby
% complexes that are not similar to each-other.
% 
% This function has two side effects: it gives |sqd_length| the smallest
% of the two lengths (of |mod| and |mat|), and gives |sqd_amp| the value
% of the amplification used with |mat| to compensate for various scaling
% effects (for example given by respiration).
% 
% --*/
% 
% short sqd_length; /* exported for later use */ /* [VAR] */ double
% sqd_amp; /* amplification of |mat| to match model */ /* [VAR] */ short
% sqd_shift; /* shift of the |mat| to match the model */ /* [VAR] */
% 
% long _sqd(GLIKMODEL *mod, GLIKMODEL *mat, short shf) /* [FUN] */
% 
% { short i, l; short delta; long sqdr, sqdrf; double ampfact;
% 
%   l = mod->gml; if (mat->gml<l) l = mat->gml; sqd_length = l;
% 
%   /* now, mat must be shifted to match the model */
% 
%   ampfact = mod->amplitude; ampfact = ampfact / mat->amplitude;
% sqd_amp = ampfact; mod->gm[mod->minpos] - mat->gm[mat->minpos];
% 
%   /* how much must the mat be displaced in time to match the model? */
% 
%   delta = mod->maxpos - mat->maxpos; if(delta>0) { /* i.e. model is
% later */ for(sqdr = 0, i = 0; i<l-delta; i++) { sqdrf =
% (mod->gm[delta+i] - (short)((mat->gm[i] - mat->gm[mat->minpos]) *
% ampfact + mod->gm[mod->minpos])) - shf; sqdr += sqdrf*sqdrf; } } else
% { for(sqdr = 0, i = 0; i<l-delta; i++) { sqdrf = (mod->gm[i] -
% (short)((mat->gm[i-delta] - mat->gm[mat->minpos]) * ampfact +
% mod->gm[mod->minpos])) - shf; sqdr += sqdrf*sqdrf; } } return sqdr; }
% 
% long sqd(GLIKMODEL *gmo, GLIKMODEL *gmm) /* [FUN] */
% 
% /* Takes the verical shift for which the difference * between the
% traces is minimal */
% 
% { short shf; long osqd, isqd;
% 
% shf = 0; osqd = _sqd(gmo, gmm, 0); isqd = _sqd(gmo, gmm, shf = 1);
% if(isqd < osqd) { while(isqd < osqd) { shf ++; if(shf>=12) break; osqd
% = isqd; isqd = _sqd(gmo, gmm, shf); } sqd_shift = shf-1; return osqd;
% } isqd = _sqd(gmo, gmm, shf = -1); if(isqd < osqd) { while(isqd <
% osqd) { shf --; if(shf<=-12) break; osqd = isqd; isqd = _sqd(gmo, gmm,
% shf); } sqd_shift = shf+1; return osqd; } sqd_shift = 0; return osqd;
% }
% 
% /* The following function takes a model glik --- the result of the
% superimposing of many gliks (their number in it's component |ngliks|)
% and a matching glik (assumed to be matching by means of a decision
% involving |sqd| and a threshold) and ``adds'' the model matching glik
% to the model glik.  The number of gliks that previously composed the
% model glik ``give weight'' to this model. */
% 
% void addtomodel(GLIKMODEL *gmod, GLIKMODEL *gplus, /* [FUN] */ double
% plusamp) /* [FUN] */
% 
% { short l, i, delta; double v;
% 
% /* printf(" %10.2f ", plusamp);*/ l = gmod->gml; if(l>gplus->gml) l =
% gplus->gml; delta = gmod->maxpos - gplus->maxpos; if(delta>0) /* gmod
% is ``later'' */ { for(i=0; i<l; i++) { v = gmod->gm[i] -
% gmod->gm[gmod->minpos]; v = v * gmod->ngliks; v = v +
% ((gplus->gm[i+delta] - gplus->gm[gplus->minpos]) * plusamp); v = v /
% (gmod->ngliks+1); gmod->gm[i] = v + gmod->gm[gmod->minpos]; }
% gmod->ngliks ++; } else { for(i=-delta; i<l; i++) { v = gmod->gm[i] -
% gmod->gm[gmod->minpos]; v = v * gmod->ngliks; v = v +
% ((gplus->gm[i-delta] - gplus->gm[gplus->minpos]) * plusamp); v = v /
% (gmod->ngliks+1); gmod->gm[i] = v + gmod->gm[gmod->minpos]; }
% gmod->ngliks ++; } }
% 
% 
% /* The following function does a classification. It takes a vector of
% models and a fill pointer which initially is 0 and is called
% |gmovecfill|.
% 
% THIS function knows the internals of the PERFILE data structure, in
% particular the gmo, gmofill and the NUM_GMOS functions.
% 
% */
% 
% #define GMOVECSIZE 50 /* [DEF] */ #define SQD_THR 5000 /* [DEF] */
% 
% void gmclassinit(PERFILE *p) /* [FUN] */
% 
% { p->gmofill = 0; }
% 
% short disdiv = 2; 						/*
% [VAR] */
% 
% /* ms_display_gm(GLIKMODEL *gm, short xoffset, short yoffset, double
% scale, short shf)
% 
% { short i;
% 
% _moveto(xoffset, 30+yoffset+shf); for(i=0; i<gm->gml; i++) {
% _lineto(i+xoffset, ((gm->gm[i]-gm->gm[gm->minpos])*scale/disdiv)+
% 30+yoffset+shf); } } */
% 
% 
% 
% short gmclass(PERFILE *p, /* [FUN] */ short *strip, short stril, /*
% [FUN] */ short qrspos, short nextqrs, short prevqrs) /* [FUN] */
% 
% { short i; long s;
% 
%   if(p->gmofill>NUM_GMOS || qrspos<1000 || qrspos>15000) return -1;
% gmsample(&(p->gmo[p->gmofill]), strip, stril, qrspos, prevqrs,
% nextqrs); for(i=0; i<p->gmofill; i++) { s = sqd(p->gmo+i,
% p->gmo+p->gmofill); if(s<SQD_THR) { /* BAD! the smallest should be
% selected */ p->gmo[i].ngliks ++; return i; /* the old class */ } }
% p->gmofill ++; return i; }
% 
%
% 

\endoffile

  --*/
#endif




