
/*--

  \FileTitle{
  {\tt glikcess.c} --- Gliks file Access package
  }
  \CopyrightNotice{

    Copyright (c) 1991--1996 Marius Seritan (http://marius.seritan.net)
    Copyright (c) 1991--1996 Alexandru Dan Corlan MD (http://dan.corlan.net)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

  }

  \section{Gliks Access Body}

  \begin{code}
  --*/

#include <stdio.h>
#include <malloc.h>
#include <sys/stat.h>

#if defined(MSDOS) && !defined(GCC)
#include<fcntl.h>
#include<io.h>
#endif

#include "gliks.h"

/* declaration of the program wide log file */
extern FILE * ecgnosislog;

short geserr;

/* debugging macros */
#define DEBUG_READ_STRIPS
#define TRACE

#undef UNIX
#define MSDOS

long filesize(FILE *f) /* [FUN] */ 

{
struct stat fs;

fs.st_size = 0;
fstat(fileno(f), &fs);

#ifdef TRACE
printf("CALL filesize: %ld\n", fs.st_size);
#endif

return fs.st_size;
}

/*

ecg_open opens a file containing ecg records in a given 
compression format, and returns a newly created ECG structure
pointing into that file; it returns NULL and sets |geserr|
to an error code if the job can't be done

*/

/* 

Creating special structures when opening files.

*/

void *delbyt_sp_open(FILE *f, long sz)  /* sz is the number of samples */

{
long vsz;
long jsz;
short *r, lastv;
short i, j, c;

jsz = STRIP_BUFFER_SIZE / 2;
vsz = sz / jsz;
r = malloc(sizeof(short)*vsz);
if(r==NULL)
  {
  geserr = NO_MEMORY;
  return NULL;
  }
j = jsz-1;
i = 1;
r[0] = -getc(f);
lastv = 0;

while(!feof(f) && i<vsz)
  {
  c = getc(f);
  lastv += c;
  j--;
  if(j==0)
    {
    r[i++] = lastv;
    j = jsz;
    }
  }

return (void *)r;
}


ECG *ecg_open(char *filename, short compression)

{
FILE *ef;
ECG *r;
long fisz;
short i;

#ifdef MSDOS
ef = fopen(filename, "rb");
#else
#ifdef UNIX
ef = fopen(filename, "r");
#endif
#endif

if(ef==NULL)
  {
  geserr = CANT_OPEN_FILE;
  return NULL;
  }

fisz = filesize(ef);
r = (ECG *)malloc(sizeof(ECG));
if(r==NULL)
  {
  geserr = NO_MEMORY;
  return NULL;
  }
strncpy(r->ecgfn, filename, FILE_NAME_SIZE);
r->ecgfn[FILE_NAME_SIZE-1] = 0;
r->ecgf = ef;
r->cprtype = compression;
switch(compression)
  {
  case ANTICOMPRESSION:
    r->eruid[0] = 0;
    r->ecgptr = 0;
    r->ecgsize = fisz / 2;
    r->ecgsamplesz = 50;   /* 50 ccs = 5 ms */
    r->nofstrips = 1;
    r->es[0] = (ECGSTRIP *)malloc(sizeof(ECGSTRIP));
    if(r->es[0] == NULL)
      {
      geserr = NO_MEMORY;
      free(r);
      return NULL;
      }
    r->es[0]->stroffs = 0;
    r->es[0]->strfill = 0;
    r->specd = NULL;
    r->es[0]->specd = NULL;
    break;

  case DELTABYTE:
    for(i=0; i<ERUID_SIZE; i++)
      r->eruid[i] = getc(r->ecgf);
    r->ecgptr = 0;
    r->ecgsize = fisz-ERUID_SIZE;
    r->ecgsamplesz = 50;   /* 50 ccs = 5 ms */
    r->es[0] = (ECGSTRIP *)malloc(sizeof(ECGSTRIP));
    if(r->es[0] == NULL)
      {
      geserr = NO_MEMORY;
      free(r);
      return NULL;
      }
    r->es[0]->stroffs = 0;
    r->es[0]->strfill = 0;

    r->specd = NULL;
    r->es[0]->specd = delbyt_sp_open(r->ecgf, r->ecgsize);
    if(r->es[0]->specd==NULL)
      {
      free(r->es);
      free(r);
      return NULL;
      /* geserr is already set */
      }
    break;
  
  case AUTOMATA_COMPRESSION:
  /* got to fix this in the transmission program */
#undef  ERUID_SIZE
#define ERUID_SIZE 0
    for(i=0; i<ERUID_SIZE; i++)
      r->eruid[i] = getc(r->ecgf);
    r->ecgptr = 0;
    r->ecgsamplesz = 50;
    r->es[0] = (ECGSTRIP *)malloc(sizeof(ECGSTRIP));
    r->es[1] = (ECGSTRIP *)malloc(sizeof(ECGSTRIP));
    if( (r->es[0]==NULL) || (r->es[1]==NULL) )
      {
      geserr = NO_MEMORY;
      free(r);
      return NULL;
      }
    r->es[0]->stroffs = 0;
    r->es[0]->strfill = 0;
    if( init_compressed_file( r ) ) {
      free( r );
      return NULL;
    }
    r->ecgsize /= 2;  /* correction of some misunderstanding */
    break;
  default:
    free(r);
    geserr = BAD_COMPRESSION_TYPE;
    return NULL;    
  }
return r;
}

/* The ecg_seek strategy: if the seek is performed outside
 * the current buffer, then it is assumed that a read
 * is to follow (or a set of them) and thus the position
 * where the seek is performed will be the first byte
 * in the newly read buffer
 */

short ecg_seek(ECG *e, long pos)

/* returns 0 if failure, 1 if success */

{
e->ecgptr = pos;

/* the situation in which the required pointer is outside 
 * the current block
 */

if(e->es[0]->stroffs>pos || (e->es[0]->stroffs+e->es[0]->strfill)<=pos)
  switch(e->cprtype)
    {
    case ANTICOMPRESSION:
      e->es[0]->stroffs = pos;
      e->es[0]->strfill = 0;  /* read when and if necessary */
      break;               /* done */
    case DELTABYTE:
      e->es[0]->stroffs = pos;
      e->es[0]->strfill = 0;
      break;
    case AUTOMATA_COMPRESSION:
      e->es[0]->stroffs = pos;
      e->es[0]->strfill = 0;  /* signal there is no valid data in buffer */
      break;
    default:
      geserr = BAD_COMPRESSION_TYPE;
      return 0;
      break; /* ! ... */
    }

return 1;
}

short ecg_close(ECG *e)

/* returns 0 if failure, 1 if success */

{
switch(e->cprtype)
  {
  case ANTICOMPRESSION:
    break;
  case DELTABYTE:
    free(e->es[0]->specd);
    free(e->es[0]);
    break;
  case AUTOMATA_COMPRESSION:
    free(e->es[0]);
    free(e->es[1]);
    break;
  default:
    geserr = BAD_COMPRESSION_TYPE;
    return 0;
  }
fclose(e->ecgf);
free(e);
return 1;
}

/* The read function starts with functions that have a similar
 * specification: read the block of STRIP_BUFFER_SIZE samples
 * starting with the e->ecgptr.
 */

void antcom_readblock(ECG *e)

{
  /* this is easy */

  long rd;
  long k;

  fseek(e->ecgf, e->ecgptr*2, 0);
  rd = fread(&(e->es[0]->strbuf), 2, STRIP_BUFFER_SIZE, e->ecgf);
  for(k=0; k<rd; k++)
    e->es[0]->strbuf[k] = 128 - e->es[0]->strbuf[k];
  for(k=0; k<4; k++)
    e->es[0]->strbuf[k] = (e->es[0]->strbuf[k+3] + e->es[0]->strbuf[k+2] + e->es[0]->strbuf[k+1]
      + e->es[0]->strbuf[k]) / 4;
  for(k=3; k<rd; k++)
    e->es[0]->strbuf[k] = (e->es[0]->strbuf[k-3] + e->es[0]->strbuf[k-2] + e->es[0]->strbuf[k-1]
      + e->es[0]->strbuf[k]) / 4;
  e->es[0]->stroffs = e->ecgptr;
  e->es[0]->strfill = rd;
  return ;  /* nothing else */
}

void delbyt_readblock(ECG *e)

{
  long lastv;
  short cv, c, i;

  /* compute the |specd| position of the last known value */

  lastv = e->ecgptr * 2;
  lastv = lastv / STRIP_BUFFER_SIZE;

  fseek(e->ecgf, ERUID_SIZE + lastv * STRIP_BUFFER_SIZE/2, 0);
  cv = ((short *)(e->es[0]->specd))[lastv];
  for(i=lastv * STRIP_BUFFER_SIZE / 2; i < e->ecgptr; i++)
    {
    c = getc(e->ecgf);
    if(c==EOF)
      break;
    if(c>127)
      c -= 256;
    cv += c;
    if(c>127)
      c -= 256;
    cv += c;
    }
  fseek(e->ecgf, e->ecgptr+ERUID_SIZE, 0);
  e->es[0]->strfill = 0;
  e->es[0]->stroffs = e->ecgptr;
  for(i=0; i<STRIP_BUFFER_SIZE; i++)
    {
    c = getc(e->ecgf);
    if(c==EOF)
      break;
    if(c>127)
      c -= 256;
    cv += c;
    e->es[0]->strbuf[i] = cv;
    e->es[0]->stroffs ++;
    }
}

short ecg_read(ECG *e, short *buffer, short t, short ns)

{
short roff;  /* offset in the result buffer --- n of s already transfered */
short rrem;  /* reminder */
short i;

rrem = ns;
roff = 0;

do {
  for(i=e->ecgptr - e->es[0]->stroffs; i < e->es[0]->strfill && rrem>0; i++)
    {
      buffer[roff] = e->es[t]->strbuf[i];
      roff++;
      rrem--;
      e->ecgptr++;
    }
  if(rrem>0)
    {
      /* a new block must be read starting from e->ecgptr */
      switch(e->cprtype)
	{
	case ANTICOMPRESSION:
	  antcom_readblock(e);
	  break;
	case DELTABYTE:
	  delbyt_readblock(e);
	  break;
        case AUTOMATA_COMPRESSION:
          if( inflate_read_strips( e ) ) 
            printf( "Error while reading the strips.\n" );
	default:
	  break;   /* Can't happen */
	}
      if(e->es[0]->strfill==0)  /* the end of file have been reached */
	return roff;
    }
}
  while(rrem>0);

return roff;
}

short ecg_read2(ECG *e,
		short *buffer, short t, short *buffer2, short t2,
		short ns)

{
short roff;  /* offset in the result buffer --- n of s already transfered */
short rrem;  /* reminder */
short i;

rrem = ns;
roff = 0;

do {
  switch(e->cprtype) {
  case ANTICOMPRESSION:
  case DELTABYTE:
    for(i=e->ecgptr - e->es[0]->stroffs; i < e->es[0]->strfill && rrem>0; i++)
      {
	if(t==0)
	  buffer[roff] = e->es[t]->strbuf[i];
	else
	  buffer[roff] = 0;
	if(t2==0)
	  buffer2[roff] = e->es[t2]->strbuf[i];
	else
	  buffer2[roff] = 0;
	roff++;
	rrem--;
	e->ecgptr++;
      }
    break;
  case AUTOMATA_COMPRESSION:
    for(i=e->ecgptr - e->es[0]->stroffs; i < e->es[0]->strfill && rrem>0; i++)
      {
	buffer[roff] = e->es[t]->strbuf[i];
	buffer2[roff] = e->es[t2]->strbuf[i];
	roff++;
	rrem--;
	e->ecgptr++;
      }
    break;
  default:
    break;
  }
  if(rrem>0)
    {
      /* a new block must be read starting from e->ecgptr */
      switch(e->cprtype)
	{
	case ANTICOMPRESSION:
	  antcom_readblock(e);
	  break;
	case DELTABYTE:
	  delbyt_readblock(e);
	  break;
        case AUTOMATA_COMPRESSION:
          if( inflate_read_strips( e ) ) 
            printf( "Error while reading the strips.\n" );
	default:
	  break;   /* Can't happen */
	}
      if(e->es[0]->strfill==0)  /* the end of file have been reached */
	return roff;
    }
}
  while(rrem>0);

return roff;
}

long ecg_tell(ECG *e)

  {
    return e->ecgptr;
  }

long ecg_size(ECG *e)  /* returns size in samples of one of the leads */

  {
  return e->ecgsize;
  }

short ecg_noft(ECG *e)

  {
  switch(e->cprtype)
    {
    case ANTICOMPRESSION:
      return 1;
      break;
    case DELTABYTE:
      return 1;
      break;
   case AUTOMATA_COMPRESSION:
     return 2;
     break;
    default:
      geserr = BAD_COMPRESSION_TYPE;
      return 0;
      break;
    }
  }



/*--
  \end{code}
  
  \subsection{ECGNOSIS-2100 Access}
  % The name of the package is expected to be prezent,
  % normally in the proper ``section'' specification above.
  
  \PackagePurpose{  % be short
  }
  \FunctionInterraction{
  % The order in which functions are expected to be called
  % The way to use the package in general
  }
  \Exceptions{
  }
  \GlobalEffects{
  % Global effects on exported or external variables
  }
  \Preconditions{
  % Conditions to be met when the package starts to be used
  }
  \Performance{
  % How it is, not why and what to do
  }
  
  \begin{code}
  --*/

/*--
\end{code}

\dataparagraph{inflate\_block}  % a name for the data structure

\Definition{Holds the private data of the package used in order to
read and decompress files created by ECGNOSIS-2100.}
\SemanticStructure{ 
The structure is used in order to:

\begin{itemize}
\item read in memory a block from the file
\item decompress the block
\item store button information for the accessed blocks.
\end{itemize}
For each of these functions there is a substructure in the global structure.
}

\begin{code}
--*/

/* module process_button_information 
 *    extract button information from current block
 *    the block must be properly initialized
 */

#define BUTTON_BUMP_DIM 128
#define BIT_ARRAY_DIMENSION 8 * 1024 / 4 / 32

struct button_table {
  long * moments;		/* time moments when the button was pressed */
  int dimension;		/* dimension of the above array */
  int fill;			/* how many places occupied */
  long processed[ BIT_ARRAY_DIMENSION ]; /* a bit for each block */
};

#define INFL_EOF 32000

#define BLKSIZE 0x1000

#define DELTA_OFFSET 10
struct block_structure {
  long       ctimer;    /* compression timer at the start of the block */
  short      cend;      /* lsb part of the compression timer at end */
  short      buttonad;  /* start of information for the button */
  unsigned char ch1;       /* initial value for the first channel */
  unsigned char ch2;       /* initial value for the second channel */
  char       cdata[BLKSIZE - DELTA_OFFSET];  /* compression data */
  };

/* macros to access configuration information */
#define CMAX(i)        (automata_cfg[ (i) * 4 + 0 ])
#define CMIN(i)        (automata_cfg[ (i) * 4 + 1 ])
#define CNBB(i)        (automata_cfg[ (i) * 4 + 2 ])
#define CCODEN(i)      (automata_cfg[ (i) * 4 + 3 ])

/* configuration data for the automata  */
extern signed char automata_cfg[];
/* number of states of the automata */
#define  CNBST 5

#define word32 unsigned long
#define word16 unsigned short

struct inflate_block {
  int      automata_state;        /* actual state of the automata */
  unsigned short automata_buffer;       /* bits holding buffer */
  char     ab_len;                /* nb of valid bits in the prev buffer */
  char     meta;                  /* counter for metastable behaviour */

  unsigned position;              /* position in samples */
  unsigned fill;                  /* position in bytes */
  short true_size;             /* number of samples stored in the block */
  unsigned block_index;           /* the number of the actual block */
  unsigned char channels[2];      /* actual value(s) on the two channels */
  struct block_structure block;   /* the contents of the actual block */
  struct button_table button;	/* store button information */
};



/*--
\end{code}

\macparagraph{\_INFL\_BLOCK\_STRUC\_} 

\MacroSemantic{Cast the private data passed in the ECG structure to
the internal structure manipulated by the package.}

\begin{code}
--*/
#define _INFL_BLOCK_STRUC_(e) ((struct inflate_block*)(e->specd))



/*--
\end{code}

\macparagraph{\_BLOCK\_STRUC\_}  

\MacroSemantic{Access the private structure used to read blocks
from the data file into memory.}

\begin{code}
--*/
#define _BLOCK_STRUC_(e) &(_INFL_BLOCK_STRUC_(e)->block)



/*--
\end{code}

\macparagraph{access\_infl\_block}

\MacroSemantic{Accesors applied to the inflate\_block structure, in order
to obtain the INDEX of the current block in the file, the index of the
first sample stored in the file (START), the dimension of the block in samples
(LENGTH), the actual position in the block (POSITION), a pointer to
a vector of dimension 2 holding the values of the samples (VALUES) }

\begin{code}
--*/

#define INFL_BLOCK_INDEX(t)     ((t)->block_index)
#define INFL_BLOCK_START(t)     ((t)->block.ctimer)
#define INFL_BLOCK_LENGTH(t)    ((t)->true_size)
#define INFL_BLOCK_POSITION(t)  ((t)->position)
#define INFL_BLOCK_VALUES(t)    ((t)->channels)



/*--
\end{code}

\macparagraph{access\_ecg} 

\MacroSemantic{Accessors applied to an ECG structure, see the previous
paragraph for further explanations.}

\begin{code}
--*/

#define BLOCK_INDEX(e)          (_INFL_BLOCK_STRUC_(e)->block_index)
#define BLOCK_START(e)          (_INFL_BLOCK_STRUC_(e)->block.ctimer)
#define BLOCK_LENGTH(e)         (_INFL_BLOCK_STRUC_(e)->true_size)
#define BLOCK_POSITION(e)       (_INFL_BLOCK_STRUC_(e)->position)
#define BLOCK_VALUES(e)         (_INFL_BLOCK_STRUC_(e)->channels)


    
        
/*--
  \end{code}
  
  \funparagraph{b2l_endian4}
  \PurposeReturn{Transforms the argument from big to little endian and
  returns it. This function is to be called on long values.}

  \begin{code}
  --*/   
word32 b2l_endian4( word32 this ) {
  word32 temp;
  char * ctemp, * cthis;
  
  cthis = (char*)&this;
  ctemp = (char*)&temp;
  ctemp[0] = cthis[3];
  ctemp[1] = cthis[2];
  ctemp[2] = cthis[1];
  ctemp[3] = cthis[0];
  return (temp);
}

/*--
  \end{code}
  
  \funparagraph{b2l_endian2}
  \PurposeReturn{Transforms the argument from big to little endian and
  returns it. This function is to be called on short values.}

  \begin{code}
--*/


word16 b2l_endian2( word16 this ) {
  word16 temp;
  ((char*)&temp)[0] = ((char*)&this)[1];
  ((char*)&temp)[1] = ((char*)&this)[0];
  return temp;
 }
     


/*--
\end{code}

\subsection{button} 

\PackagePurpose{Gathers together button information.}

\begin{code}
--*/


void maybe_bump_button_table( struct button_table * btn ) 
{
  if( btn->fill < btn->dimension ) return;
  while( btn->fill >= btn->dimension ) btn->dimension+= BUTTON_BUMP_DIM;
#ifdef DEBUG_BUTTON
  printf( "Reallocating button table to %d.\n", btn->dimension );
#endif	  
  if( btn->moments != NULL )
    btn->moments = realloc( btn->moments, btn->dimension * sizeof(long) );
  else
    btn->moments = malloc( btn->dimension * sizeof(long) );
}
/* local function(s) */


/* functions to speedup button processing time */


/*--
\end{code}

\funparagraph{button\_processed}
\PurposeReturn{This is a predicate, returns 1 if the block was already
processed.}

\begin{code}
--*/

static long button_processed_conversion[] ={
    1    , 1<<1, 1<<2, 1<<3, 1<<4, 1<<5, 1<<6, 1<<7
  , 1<<8, 1<<9, 1<<10, 1<<11, 1<<12, 1<<13, 1<<14, 1<<15
  , 1<<16, 1<<17, 1<<18, 1<<19, 1<<20, 1<<21, 1<<22, 1<<23
  , 1<<24, 1<<25, 1<<26, 1<<27, 1<<28, 1<<29, 1<<30, 1<<31
  };
int button_processed( struct button_table * btn, int ndx) {
  return ( btn->processed[ ndx / 32] & button_processed_conversion[ndx % 32] );
}


/*--
\end{code}

\funparagraph{button\_mark\_processed}
\PurposeReturn{ Marks the current block as being processed in the
button\_table structure.}

\begin{code}
--*/

int button_mark_processed( struct button_table * btn, int ndx ) {
  btn->processed[ ndx / 32 ] |= button_processed_conversion[ndx % 32];
}


/*--
\end{code}

\funparagraph{button\_table\_search}
\PurposeReturn{ 
The position in the sorted table in which to insert
the given moment.
}
\Exceptions{ }
\GlobalEffects{ }
\Preconditions{ }
\Performance{ }

\begin{code}
--*/
int button_table_search( struct button_table * btn, int moment ) {
  int position;

  for( position=0; position<btn->fill; position++ ) {
    if( btn->moments[position] == moment ) {
      printf( "Warning: trying to introduce twice the same event.\n" );
      return position;
    }
    if(  btn->moments[position] > moment ) return position;
  }
  return position;
}


/*--
\end{code}

\funparagraph{extract\_moment}
\PurposeReturn{ Get the nth event moment of the block, first event has index 0.
 }
\Preconditions{ The block containing the given event must have been accessed.}

\begin{code}
--*/

long extract_moment( struct block_structure * block, int nth ) {
/* *don't touch* the arithmetic operations, it is detrimental to your health*/
  unsigned short moment;

  moment = ((unsigned char *)block)[BLKSIZE - 2 * nth];
  moment = b2l_endian2( moment );
  moment -= (unsigned short)(block->ctimer & 0xffff);
  return( block->ctimer + (unsigned long)moment );
}


/*--
\end{code}

\funparagraph{button\_process\_information}
\PurposeReturn{Used for the side effects. Called for each block read into
memory to extract button information stored in the given block.}

\begin{code}
--*/

void button_process_information( struct button_table * btn
				, ECG * ecg ) {
  struct block_structure * block;
  long but_ad;			/* index in the block */
  long first_moment;		/* the moment of the first button */
  int nb_events;		/* # of times the button was pressed */
  int block_index;		/* # of the actual block */
  int position;			/* position where to insert information */
  
  if( button_processed( btn, BLOCK_INDEX(ecg)) ) return;
  button_mark_processed( btn, BLOCK_INDEX(ecg) );

  /* extract the relevant information */
  block = _BLOCK_STRUC_(ecg);
  nb_events = (BLKSIZE-block->buttonad)/2;
  if( nb_events == 0 ) return;
  fprintf( ecgnosislog, " %d events detected in block %d.\n", nb_events
	  , BLOCK_INDEX(ecg));
  fflush( ecgnosislog );
  first_moment = extract_moment(block, 0);

  /* look if the moment is already stored in the structure */
  /* linear search is used at the moment, I will switch later 
   * to a binary algoritm
   */
  position = button_table_search( btn, first_moment );
  /* allocate memory for the new events */
  btn->fill += nb_events;
  maybe_bump_button_table( btn );
    
  { 
    int i,j;
    /* move events */
    for(i=btn->fill-1,j=btn->fill-nb_events-1;j>=position;i--,j--)
      btn->moments[i] = btn->moments[j];
    /* insert new events */
    for( i=0; i<nb_events; i++ ) {
      long time_moment;

      time_moment = extract_moment( block, i );
      fprintf( ecgnosislog, "Event nb. %d at time %ld.\n"
	      , position+i, time_moment );
      btn->moments[position+i] = time_moment;
    }
  }
}



/*--
\end{code}

\subsection{e2100 decompression} 
\PackagePurpose{Access files compressed by the holter.}

\begin{code}
--*/


/*--
\end{code}

\macparagraph{debugging\_macros} 

\begin{code}
--*/


/* #define DEBUG_INITIALIZATION */
/* #define RETURN_DELTA_VALUE */
/* #define DEBUG_ONLY_FIRST_BLOCK */
/* #define DEBUG_FREAD */
/* #define DEBUG_GET_BITS */
/* #define DEBUG_GET_INFL */
/* #define DEBUG_SEEK */
/* #define DEBUG_BUTTON */

/* the next defines are only for the first version */
#ifdef ERUID_BUFFER_SIZE
#undef ERUID_BUFFER_SIZE
#endif
#define ERUID_BUFFER_SIZE 0



/*--
\end{code}

\funparagraph{simple\_block\_read}
\PurposeReturn{ 0 if ok, -1 if some error occured. The block index is read
from the file and placed in the buffer.}

\begin{code}
--*/

static int simple_block_read( FILE * f, int index, void * buffer ) {
  /* reads the block `index' from file
   * the first block has index 0
   * return 0 on succes or -1 on failure
   */
  if( fseek( f, index * BLKSIZE + ERUID_BUFFER_SIZE, 0 ) ) {
#ifdef DEBUG_FREAD
    perror("seek");
#endif
    return -1;
  }
  {int c;
   if( (c=fread( buffer, 1, BLKSIZE, f)) != BLKSIZE ) {
#ifdef DEBUG_FREAD
     printf( "fread returned %d\n", c );
     perror( "fread" );
#endif
     return -1;
   }}
  return 0;
}


/*--
\end{code}

\subsection{Compressed file access} 

\begin{code}
--*/



/*--
\end{code}

\funparagraph{init\_inflate\_automata}
\PurposeReturn{This is the constructor. Initialize automata related
fields in the argument.}
\Exceptions{}
\GlobalEffects{}
\Preconditions{}
\Performance{}

\begin{code}
--*/

static void init_inflate_automata( struct inflate_block * this ) {
  /* hardware asumptions 
   *       short variables are stored less significative byte first
   */
  this -> automata_state = 0;
  this -> ab_len = 16;
  ((char*)&(this -> automata_buffer))[1] = this->block.cdata[0];
  ((char*)&(this -> automata_buffer))[0] = this->block.cdata[1];
  this -> fill = 2;
}


/*--
\end{code}

\funparagraph{seek\_start\_current\_block}
\PurposeReturn{Seek to the start of an already initialized block.}
\Exceptions{}
\GlobalEffects{}
\Preconditions{}
\Performance{}

\begin{code}
--*/

static void seek_start_current_block( ECG*  ecg) {
  struct inflate_block * this;
  
  this = _INFL_BLOCK_STRUC_(ecg);
  init_inflate_automata( this );
  this -> meta = 0;
  /* init block quantities */
  this -> position = 0;
  /* initialize the values on the two channels */
  this -> channels[0] = this->block.ch1;
  this -> channels[1] = this->block.ch2;
#ifdef DEBUG_INITIALIZATION
  printf("values on the two channels %d %d.\n"
	 , this->channels[0], this->channels[1] );
#endif
}



/*--
\end{code}

\funparagraph{init\_current\_block}
\PurposeReturn{Bring to the proper form a block read from the file.}
\Exceptions{}
\GlobalEffects{}
\Preconditions{}
\Performance{}

\begin{code}
--*/

static void init_current_block( ECG * ecg ) {
  struct inflate_block * this;
  
  seek_start_current_block( ecg );
  this = _INFL_BLOCK_STRUC_(ecg);
  this->block.ctimer   = b2l_endian4(this->block.ctimer) ;
#ifdef DEBUG_INITIALIZATION
  printf( "\nblock start: %ld", this->block.ctimer );
#endif
#ifdef DEBUG_INITIALIZATION
  printf( " end: [%4x -> %4x] ", this->block.cend
	 , b2l_endian2(this->block.cend) );
#endif
  this->block.cend     = b2l_endian2(this->block.cend);
  this->block.buttonad = b2l_endian2(this->block.buttonad);
  this->block.buttonad &= 0x0fff;
  this->block.buttonad += this->block.buttonad ? 0: 0x1000;
#ifdef DEBUG_BUTTON
  printf( " [ %4x ] ", this->block.buttonad );
#endif
  this -> true_size = (this->block.cend - (word16)(this->block.ctimer))
    + 2; /* HOLTER BUG ? */
  this->block.ctimer += 2 * BLOCK_INDEX(ecg); /* HOLTER BUG? */
  
#ifdef DEBUG_INITIALIZATION
  printf( "length: %u[%x].\n", this->true_size, this->true_size);
  printf( "position in the compressed file: %ld\n", ftell( ecg->ecgf) );
#endif
}


/*--
\end{code}

\funparagraph{read\_next\_block}
\PurposeReturn{Read the next block in the buffer passed as an argument.}
\Exceptions{}
\GlobalEffects{}
\Preconditions{}
\Performance{}

\begin{code}
--*/

static int read_next_block( ECG * ecg) {
   int index;

   index = BLOCK_INDEX(ecg);
   index  ++;
   if( simple_block_read( ecg->ecgf, index, _BLOCK_STRUC_(ecg)) ) {
     BLOCK_INDEX(ecg) = -1;
     return -1;
   }
   BLOCK_INDEX(ecg) = index;
   init_current_block( ecg );
   (void)button_process_information(&(_INFL_BLOCK_STRUC_(ecg)->button),  ecg );
   return 0;
 }


/*--
\end{code}

\funparagraph{read\_previous\_block}
\PurposeReturn{Read the previous block in the buffer passed as
an argument.}
\Exceptions{}
\GlobalEffects{}
\Preconditions{}
\Performance{}

\begin{code}
--*/

 static int read_previous_block( ECG * ecg) {
   int index;

   index = BLOCK_INDEX(ecg);
   if( index )
     index--;
   else return -1;
   if( simple_block_read( ecg->ecgf, index
			    , _BLOCK_STRUC_(ecg)) )
     return -1;
   BLOCK_INDEX(ecg) = index;
   init_current_block( ecg );
   return 0;
 }

 long filesize( FILE *);


/*--
\end{code}

\funparagraph{init\_compressed\_file}
\PurposeReturn{Opens a compressed file and initializes the
apropriate fields in the argument.}
\Exceptions{}
\GlobalEffects{}
\Preconditions{}
\Performance{}

\begin{code}
--*/


int init_compressed_file( ECG * ecg ) 
{	
  /* those are GLIKSEE initialization  actions, not fully implemented
   * returns 0 if ok or -1 if some error occured
   */
  int last_block;
  struct inflate_block * priv;

  /* allocating and initializing my structure */
  priv = (struct inflate_block *) malloc( sizeof(struct inflate_block));
  ecg -> specd = priv;
  if( priv == NULL ) {
    geserr = NO_MEMORY;
    return -1;
  }

  /* initialize button related information */
  {int i; for(i=0;i<BIT_ARRAY_DIMENSION;i++) 
     priv->button.processed[i]=0;}
  priv->button.dimension=0;
  priv->button.fill=0;
  priv->button.moments=0;

  /* read last block */
  last_block = (filesize( ecg->ecgf ) - 1) / BLKSIZE;
  BLOCK_INDEX(ecg) = last_block  - 1 ;
#ifdef DEBUG_INITIALIZATION
  printf( "Reading the last block.\n" );  /* dbg */
#endif
  if( read_next_block( ecg ) ) {
    /* the last block was truncated, read the previous one */
#ifdef DEBUG_INITIALIZATION
    printf( "Truncated, trying the previous one.\n" ); /* dbg */
#endif
    BLOCK_INDEX(ecg) = last_block - 2;
    if( read_next_block(ecg) ) {       /* there is something wrong for sure */
      geserr = COMPRESSION_FILE_ERROR;  /* maybe the file is too short */
      return -1;
    }
  }
#ifdef DEBUG_INITIALIZATION
  printf( "Ok, I have read a valid block.\n " );      /* dbg */
#endif
   /* compute the size of the file in samples */
  ecg -> ecgsize = BLOCK_START(ecg) + BLOCK_LENGTH(ecg);
   /* initializing the index of the block */
  BLOCK_INDEX(ecg) = -1;              /* of course this is not a valid value */
  /*  but now everything should be fine */
  if( read_next_block( ecg ) ) {
    geserr = COMPRESSION_FILE_ERROR;
    return -1;
  }
  return 0;
}




/*--
\end{code}

\subsubsection{Serialization} 

\PackagePurpose{Access the data in a block as a continous stream of bits.}
\FunctionInterraction{ Use function \functiontag{get\_bits} to obtain
the required number of bits.}
\Exceptions{
}
\GlobalEffects{}
\Preconditions{}
\Performance{Good, a lot of efforts were spent on the package.}

\begin{code}
--*/


/*--
\end{code}

\dataparagraph{Bit manipulation data}  % a name for the data structure

\Definition{}
\SemanticStructure{ }
\ConsistencyRules{}

\begin{code}
--*/


static unsigned char bits_clear[] = {
  0x00,0x01,0x03,0x07,0x0f,0x1f,0x3f,0x7f,0xff
};

static unsigned short bits_select[] = {
  0x00, 0x01, 0x02, 0x04, 0x08, 0x10, 0x20, 0x40, 0x80
    , 0x0100, 0x0200, 0x0400, 0x0800, 0x1000, 0x2000, 0x4000, 0x8000
};


/*--
\end{code}

\funparagraph{get\_bits}
\PurposeReturn{Returns the requested number of bits from the given buffer.}
\Exceptions{}
\GlobalEffects{}
\Preconditions{}
\Performance{}

\begin{code}
--*/



/*--
\end{code}

\funparagraph{get\_bits --- implementation}
\ImplementationNotes{ 
The internal function \functiontag{\_get\_bits} is the general function
that can be applied no matter which is the state of the block. The macro
\macrotag{get\_bits} is a lot faster for some simple but frequent cases.
}
\Portability{}
\Changes{}

\begin{code}
--*/

#define USE_FASTER_GET_BITS

#ifdef USE_FASTER_GET_BITS
#define get_bits(this, nb) ((nb==1) && (this->ab_len) ? \
    ((this->automata_buffer&bits_select[this->ab_len--])!=0) :\
     _get_bits(this,nb))
#else
#define get_bits(this, nb) _get_bits(this,nb)
#endif

static char _get_bits( struct inflate_block * this, char nb ) {
  /* get nb bits from the buffer, nb<=8 !!! */
  char ret;      /* location used to build the requested data */
  
#ifdef DEBUG_GET_BITS
  printf("buffer: %2x  len: %2d req: %2d  ", this->automata_buffer
	 , this->ab_len, nb);
#endif
  if( this->ab_len < 8 ) {
    /* move the remaining data from the lower byte of
     * automata_buffer to the upper byte
     */
    ((char*)&(this->automata_buffer))[1]=((char*)&(this->automata_buffer))[0];
    /* fill in the lower byte of automata_buffer */
    ((char*)&(this -> automata_buffer))[0]=this -> block.cdata[ this-> fill ];
     this -> fill ++;
    /*
       if( this->fill > 4086 ) {
       printf( "\n!!!! get_bits: runaway.!!!\n\n" );
       exit( 1 );
       }
       */
    this->ab_len += 8;
  };
  {
    this->ab_len -= nb;
    ret = this -> automata_buffer  >> this->ab_len;
    ret = ret & bits_clear[nb];
  };
#ifdef DEBUG_GET_BITS
  printf("res: %3u   ", ret);
#endif
  return ret;
}

/*--
\end{code}

\subsection{Inflate automata} 

\PackagePurpose{Implements the automata used for the decompression
of files generated by E2100.}
\FunctionInterraction{}
\Exceptions{}
\GlobalEffects{}
\Preconditions{}
\Performance{}

\begin{code}
--*/


/*--
\end{code}

\dataparagraph{Decompression automata configuration}
\Definition{This array stores the configuration of the decompression
automata. An array format was used as this is the way the information
is represented in the holter. For each state of the automata there are
four elements in the array: the maximum and the minimum limit for the values
that can be safely sent in the state, the number of bits to codify them
and the escape sequence that must be sent is the automata is moving
to some upper state.}
\SemanticStructure{ 
}
\ConsistencyRules{
}

\begin{code}
--*/

signed char automata_cfg[] = {
   0,    0,  1,   1,
   1,   -1,  2,   2,
   2,   -2,  3,   4,
  15,  -15,  5,  16,
  63,  -63,  7,  64,
  128,-128,  8, 129   /* the last value cannot appear in the data */
  };


/*--
\end{code}

\macparagraph{SIGN\_EXTEND}  % the name of the macro or group of macros

\MacroSemantic{ Sign extend val as a value on nb_bits.
}

\begin{code}
--*/
static unsigned int bits_decod[] = {0,1,2,4,8,16,32,64,128};
static unsigned int  bits_expand[] = {0,0,0xfffffffc,0xfffffff8
					,0xfffffff0,0xffffffe0
                                     ,0xffffffc0,0xffffff80
				     ,0xffffff00};

#define SIGN_EXTEND(val,nb_bits) (( val & bits_decod[nb_bits] ) ? \
                                   bits_expand[nb_bits] | val: val)


/*--
\end{code}

\funparagraph{buffer\_get\_inflated\_c}
\PurposeReturn{The next inflated character from the buffer. You must be
sure before calling this function that there is a next character stored
in compressed format in the buffer.}
\Exceptions{}
\GlobalEffects{}
\Preconditions{}
\Performance{}

\begin{code}
--*/

static int buffer_get_inflated_c( struct inflate_block * this ) {
  /* inflate data from the buffer
   * return each time the next decodified value
   */
  int temp;                          /* storage for the actual code */
  /* implementing metastable behaviour */
  if( this -> automata_state )
    if( this -> meta ) this -> meta--;
    else this -> automata_state--;

  temp = get_bits( this, CNBB( this -> automata_state ) );
  /* state determination */
  while( temp == CCODEN( this -> automata_state ) ) {
    this -> automata_state ++;
    temp = get_bits( this, CNBB( this -> automata_state ) );
    if( this -> automata_state == CNBST ) {
      this -> meta = 0;
      return( SIGN_EXTEND(temp, CNBB(this->automata_state) ));
    }
    this -> meta = 1;
  }
  return (SIGN_EXTEND(temp, CNBB(this->automata_state)));
}



/*--
\end{code}

\funparagraph{switch\_to\_next\_block}
\PurposeReturn{ 1 if error, 0 if ok }
\Exceptions{}
\GlobalEffects{}
\Preconditions{}
\Performance{}

\begin{code}
--*/

static int switch_to_next_block(ECG * ecg ) {
    unsigned char ch1, ch2;
    ch1 = BLOCK_VALUES(ecg)[0];
    ch2 = BLOCK_VALUES(ecg)[1];
#ifdef DEBUG_GET_INFL
    printf("Reached eob(lock) [%u==%u], getting next one.\n"
               , BLOCK_POSITION(ecg), BLOCK_LENGTH(ecg));
#endif
    if( read_next_block( ecg ) ) {
#ifdef DEBUG_GET_INFL
      printf("No more blocks available, eof.\n" );
#endif
      return 1;
    }
    if( (ch1!=BLOCK_VALUES(ecg)[0]) || (ch2!=BLOCK_VALUES(ecg)[1]) )
      printf("\007Syncronization error %d!=%d or %d!=%d.\n"
                    , ch1, BLOCK_VALUES(ecg)[0]
                    , ch2, BLOCK_VALUES(ecg)[1]);
#ifdef DEBUG_ONLY_FIRST_BLOCK
      return 1;  /* getting only the first block in debugging purposes*/
#endif
  return 0;
  }


/*--
\end{code}

\funparagraph{get\_inflated\_c}
\PurposeReturn{Get an inflated value from the file or INFL_EOF if error.}
\Exceptions{ }
\GlobalEffects{ }
\Preconditions{ }
\Performance{ }

\begin{code}
--*/

int get_inflated_c( ECG * ecg ) { /* [FUN] */
  if( ( BLOCK_POSITION( ecg ) == BLOCK_LENGTH( ecg ) )
      && switch_to_next_block( ecg )) return INFL_EOF;
  { int delta, which_one;
    delta = buffer_get_inflated_c( _INFL_BLOCK_STRUC_(ecg) );
    which_one = BLOCK_POSITION(ecg) & 1;  /* which channel is updated ? */
    BLOCK_POSITION(ecg) ++;     /* the position in the ecg file is computed
                                   from this field */
#ifdef DEBUG_GET_INFL
    printf("Delta value read: %d\n", delta);
#endif
    { int true_value;
      true_value = delta + BLOCK_VALUES(ecg)[which_one];
      true_value &= 0xff;
      BLOCK_VALUES(ecg)[ which_one ] = true_value;
#ifdef RETURN_DELTA_VALUE
      return(  delta );
#else
      return( true_value );
#endif
    }
  }
}


/*--
\end{code}

\funparagraph{inflate\_get\_values}
\PurposeReturn{Decompresses two values from the current buffer. This is an
alternate and faster implementation of buffer_get_inflated_c.}
\Exceptions{}
\GlobalEffects{}
\Preconditions{}
\Performance{}

\begin{code}
--*/

static int inflate_get_values( struct inflate_block * this
			                      , int * ret1
			                      , int * ret2
			      ) {
  int temp; 
  int to_do;			/* how many values still to compute */

  to_do = 2;
work_work:
  /* implementing metastable behaviour */
  if( this -> automata_state )
    if( this -> meta ) this -> meta--;
    else this -> automata_state--;
  
  
  temp = get_bits( this, CNBB( this -> automata_state ) );
  /* state determination */
  while( temp == CCODEN( this -> automata_state ) ) {
    this -> automata_state ++;
    temp = get_bits( this, CNBB( this -> automata_state ) );
    if( this -> automata_state == CNBST ) {
      this -> meta = 0;
      switch( to_do ) {
      case 1: *ret2 = SIGN_EXTEND(temp, CNBB(this->automata_state) );
	return;
      case 2: *ret1 = SIGN_EXTEND(temp, CNBB(this->automata_state) );
	to_do = 1;
	goto work_work;
      };
    }
    this -> meta = 1;
  }
  switch( to_do ) {
  case 1: *ret2 = SIGN_EXTEND(temp, CNBB(this->automata_state) );
    return;
  case 2: *ret1 = SIGN_EXTEND(temp, CNBB(this->automata_state) );
    to_do = 1;
    goto work_work;
  };
}


/*--
\end{code}

\funparagraph{inflate\_get\_from\_block}
% The name of the function is expected to be prezent
% in the sectioning element above

\PurposeReturn{Reads the requested number of characters from the
block and places them in the two buffers.}
\Exceptions{ }
\GlobalEffects{ }
\Preconditions{ }
\Performance{ }

\begin{code}
--*/

int inflate_get_from_block( struct inflate_block * this
                           , short * dest1
                           , short * dest2
                           , int length) {
  /* preconditions
   *     BLOCK_VALUES(0,1) must hold proper values
   *     block boundaries must  not be crossed during processing
   * postcondition
   *     BLOCK_VALUES(0,1) hold proper values
   */
  int i                /* temporary variable */
       , nb_values     /* real number of values extracted in one call */
       , strip_fill    /* number of values stored in the strip */
       ;
  
  if( length == 0) return 0;     /* patological behaviour */
  /* get the first values in the strip */
  dest1[0] = INFL_BLOCK_VALUES(this)[0];
  dest2[0] = INFL_BLOCK_VALUES(this)[1];

  /* complete the rest of the strip */
  for( strip_fill = 1; strip_fill <= length ; strip_fill++ ) {
   int temp1, temp2;

   inflate_get_values( this, &temp1, &temp2);
   /* Integrate the signal */
   temp1 += dest1[strip_fill-1];
   temp2 += dest2[strip_fill-1];
   /* save the samples */
   dest1[strip_fill] = temp1 & 0xff;
   dest2[strip_fill] = temp2 & 0xff;
 };
 /* restore the state of the block */
 INFL_BLOCK_VALUES(this)[0] = dest1[length];
 INFL_BLOCK_VALUES(this)[1] = dest2[length];
 return length;
}


/*--
\end{code}

\funparagraph{inflate\_read}
% The name of the function is expected to be prezent
% in the sectioning element above

\PurposeReturn{  % be short
Reads data from the compressed file in the two buffers in memory.
Returns the true number of values read.
}
\Exceptions{
}
\GlobalEffects{ }
\Preconditions{ }
\Performance{ }

\begin{code}
--*/


int inflate_read( ECG * ecg
                 , short * dest1
                 , short * dest2
                 , int length
                 ) {
  int strip_get                 /* number of values to read from act. block */
      , nb_values               /* number of values read from the act. block */
      , strip_fill              /* dimension of the valid data in the strip */
    ;
  struct inflate_block * this;

  this = _INFL_BLOCK_STRUC_(ecg);
  strip_fill = 0;
  for(  strip_get = length; strip_get; /* in the loop */) {
    if( (INFL_BLOCK_LENGTH(this) - INFL_BLOCK_POSITION(this)) <= strip_get * 2 ) {
      nb_values = inflate_get_from_block( this
             , dest1 + strip_fill
             , dest2 + strip_fill
             , ((int)INFL_BLOCK_LENGTH(this) 
                - (int)INFL_BLOCK_POSITION(this))/2);
      strip_get -= nb_values;
      strip_fill += nb_values;
      /* switch to the next block */
     if( switch_to_next_block( ecg ))
       goto proper_exit;
      } else {
      nb_values = inflate_get_from_block( this
                             , dest1 + strip_fill
                             , dest2 + strip_fill
                             , strip_get );
      INFL_BLOCK_POSITION( this ) += nb_values * 2;
      return length;
    }
  }
proper_exit:
/*
  fprintf( stderr, "inflate_get_from_block ---> error.\n" );
*/
  BLOCK_INDEX(ecg) = -1; /* invalidate actual block */
  return (length-strip_get);
}



/*--
\end{code}

\funparagraph{inflate\_read\_strips}  % The name of the function(s)
\ImplementationNotes{ }
\Portability{ }
\Changes{ }

\begin{code}
--*/


int inflate_read_strips( ECG * ecg ) {
  /* read a strip from the decompressed file in the strip
   * most of the dificulties arise from the fact that
   * the state of the decompression module must be kept consistent
   * preconditions
   *    the current block must be in a valid state
   * postconditions
   *    if no errors occur the current block is in a valid state
   *    if errors occur the current block is invalidated
   */
  int i            /* temporary variable */
       ,code       /* actual value  to be decompressed */
       ,nb_values  /* number of values resulting from actual decompression */
       ;

  inflate_seek_function( ecg, ecg->ecgptr * 2 );
  ecg -> es[0] -> strfill = 0;
  ecg -> es[0] -> stroffs = ecg->ecgptr;
  nb_values = inflate_read( ecg
               , ecg->es[0]->strbuf
               , ecg->es[1]->strbuf
               , STRIP_BUFFER_SIZE);
  ecg -> es[0] -> strfill += nb_values;
  return 0;
}

/*--
\end{code}

\funparagraph{inflate\_seek\_function}
\PurposeReturn{ }
\Exceptions{ }
\GlobalEffects{ }
\Preconditions{ }
\Performance{ }

\begin{code}
--*/

int inflate_seek_function( ECG * ecg, long position ) { /* [FUN] */
  if( BLOCK_INDEX(ecg) == -1 )   /* if the current block is invalid */
    read_next_block( ecg );
  /* locate the right buffer */
  while( position < BLOCK_START(ecg) ) {
    if( read_previous_block( ecg ) ) {
#ifdef DEBUG_SEEK
      printf("seek: trying to go before the first block.\n" );
      printf("  block start %ld, position %ld\n"
                , BLOCK_START(ecg), position);
#endif
      break;
    }
  }
  while( position >= (BLOCK_START(ecg) + BLOCK_LENGTH(ecg) )) {
    if( read_next_block( ecg ) )
      return -1;
  }
#ifdef DEBUG_SEEK
  printf( "block %d start %ld end %ld position %ld.\n"
                , BLOCK_INDEX(ecg)
                , BLOCK_START(ecg)
                , BLOCK_START(ecg)+BLOCK_LENGTH(ecg)
                , position );
#endif
  /* make position relative to this block start */
  position -= BLOCK_START(ecg);
  /* avoid to initialize the block if this is not necessary
   * (i.e. the block has not changed
   * and the requested position has not been reached)
   */
  if( position <= BLOCK_POSITION(ecg) )
    seek_start_current_block( ecg );
  /* seek into the block to the requested position */
#ifdef DEBUG_SEEK
  printf("seek: into the block to %ld .... \n", position );
#endif
  while( position > BLOCK_POSITION(ecg) ) {
    if( get_inflated_c( ecg ) == INFL_EOF ) {
#ifdef DEBUG_SEEK
      printf( "Block ended at %ld instead of %d.\n"
                , BLOCK_POSITION(ecg)
                , BLOCK_LENGTH(ecg));
#endif
      return -1;   /* i.e. error */
    }
  }
  if( position != BLOCK_POSITION(ecg) ) {
    printf( "seek error.\n" );
  }
  return 0;
}


     

/*--
  \end{code}

  \endoffile

  --*/
