
/*== ================================================================
 * \FileTitle      {glikintv.h---Interval superimposition and query package}
 * \Copyright
 *                  {  Copyright (c) 1994 Alexandru Dan CORLAN  }
 * \FileAuthor
 *                  {  Alexandru Dan CORLAN  }
 * \AuthorAffiliation
 *                  {  Electrocardiognosis SRL, 
 *                     28 Argentina str, 71206, Bucharest 1, ROMANIA  }
 * \RevisionHistory
 *
 *     $Log: glikintv.h,v $
 *     Revision 1.1  1996/09/05 12:48:10  dcorlan
 *     Apparently many files changed and some added (the later, for
 *     event processing). This version compiles with the 2.0 wiport
 *     (at the stage where M.Se. gives up its maintenance) and with
 *     the gcc-2.7.2.
 *
 *
 *== ================================================================
 *
 * \FileContents
 */




/*-- ----------------------------------------------------------------
 * \end{code}
 *
 * \section                {glikintv}
 *
 * \PackagePurpose
 *      Compute the current value of a parameter if its values
 *      on potentially overlapping domains is given. The intervals
 *      must have STRICTLY POSITIVE boundaries.
 * \FunctionInterraction
 * \Exceptions
 * \GlobalEffects                                % on global variables
 * \Preconditions
 * \Performance                                  % how it is, not why
 *
 * \begin{code}
 */

/*-- ----------------------------------------------------------------
 * \end{code}
 *
 * \dataparagraph                {Interval maps}
 *
 * \Definition
 *      The value of a parameter at every moment in time
 * \SemanticStructure
 *      Two variants are offered: one for float parameters (FINTVS) and one
 *      for (long) integer parameters (IINTVS). The moments when the parameter
 *      value changes are represented as a vector with a fill pointer
 *      of moment/new value pairs (FLENT, ILENT).

 * \ConsistencyRules
 *      \itemb Moments must be in ascending order. 
 *      \itemb \variabletag{fill} must be shorter than size,
 *        and at least 1 (if the parameter does not change forever, but it must
 *        have some value) in which case \variabletag{at} is 0.
 *      \itemb \variabletag{size} must match the allocated length
 *        of \variabletag{vv}.
 * 
 * \begin{code}
 */


typedef struct flent {
  double v;
  long at;
} FLENT;

typedef struct ilent {
  long v;
  long at;
} ILENT;

typedef struct fintvs {
  short size;
  short fill;
  FLENT *vv;
} FINTVS;

typedef struct iintvs {
  short size;
  short fill;
  ILENT *vv;
} IINTVS;

#define INIT_VS_SIZE       16
#define VS_SIZE_INCREMENT   8



/*-- ----------------------------------------------------------------
 * \end{code}
 *
 * \macparagraph                {Error signaling}
 *
 * \MacroSemantic
 *      Raise an error of the specified type
 *
 * \begin{code}
 */

#define memory_error(x) exit(x)


/*-- ----------------------------------------------------------------
 * \end{code}
 *
 * \funparagraph                {new\_fintvs, new\_iintvs}
 * 
 * \PurposeReturn
 * \Exceptions
 * \GlobalEffects
 * \Preconditions
 * \Performance                                  % How it is, not why
 *
 * \begin{code}
 */

FINTVS *new_fintvs(double defa);
IINTVS *new_iintvs(long defa);

/*-- ----------------------------------------------------------------
 * \end{code}
 *
 * \funparagraph                {ins\_f\_intv, ins\_i\_intv}
 * 
 * \PurposeReturn
 *      Change \variabletag{intvs} to contain starting and including
 *      \variabletag{from} and ending but excluding \variabletag{to}
 *      the value \variabletag{val}. \variabletag{from} and \variabletag{to}
 *      must be STRICTLY POSITIVE and \variabletag{from} strictly smaller
 *      than \variabletag{to}.
 * \Exceptions
 * \GlobalEffects
 * \Preconditions
 * \Performance                                  % How it is, not why
 *
 * \begin{code}
 */

void ins_f_intv(FINTVS *intvs, long from, long to, double val);
void ins_i_intv(IINTVS *intvs, long from, long to, long val);

