
/*--
  \chapter{Gliks package specification}

  \FileTitle{
  {\tt gliks.h} --- Specification of the GLIKS recognision package
  }
  \CopyrightNotice{

    Copyright (c) 1991--1996 Alexandru Dan Corlan MD (http://dan.corlan.net)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

  }


  \PackagePurpose{
  This package contains the functions that access the disk representation of
  the ECG (and other signals) recordings. 
  }

\section*{Introduction and overview}

This file contains all the data types that known outside the modules
of the Electrocardiognosis program, all the constants that
parameterise the execution of the program, but does not contain
prototype of each function in the program (which are stored in the
header files of each module in part).

All the modules in Electrocardiognosis include this file (the one
generated by FunnelWeb from this file) --- and also headers of other
modules for the prototypes and the variable definitions, normally
after including this file ({\tt ecgnosis.h}).

\subsection*{Requirements}

This file assumes that {\tt stdio.h} and {\tt isstime.h} are included by
the using module before itself.

The primary analysis subsystem uses the {\tt isstime} module but,
despite the Lisp-like representation of the edits done to the file,
works without the ILFIVE library (which is, by the way, included by
most of the applications).

\subsection*{History}

This file was created in December 4, 1993, by extracting all relevant
definitions from the modules written until that moment. 

It was rewritten in December 21 using FunnelWeb (choosen in the same
day to be the documentation system for Electrocardiognosis).

It was rewritten again in July 1994 use the newly defined
{\tt taw} system for literate programming. We have reasons to believe
that this time it will not change any more.

\subsection*{Overview}

In principle, there are many kinds of ECGNOSIS applications. The
``core'' of all of them is represented by the primary analysis of
the ECG record files. This notion comprises the most direct,
``lexical'' description of the ECG recording: the placement of the
fast complexes, identification of the other waves of the cardiac
electric activity and all the possible measurements. The most
important characteristic of this process is that doctors don't have
any doubts or controversies regarding the results of this step of
analyses. For example, one can't be sure if a given complex is a
ventricular or supraventricular ectopic beat, or even if it is an
ectopic beat at all. But everybody agrees it is --- say --- an
``rS'' fast complex having a duration of 0.07s. People may argue over
the fact that a 0.1mV displacement of the ST segment above the
baseline is pathological, but not on the fact that it can be observed
on a given recording.

This file is the header of the subsystem consisted of all the modules
that actually perform the primary analysis. It contains the data
structure that is processed by the whole subsystem.

\subsubsection*{Result of the primary analysis}

The goal of the primary analysis process is to produce an description
of the recording that can be made to be in complete and detailed
agreement with the opinion of the user (doctor).  Procedures of this core
are going to make an as accurate as possible automatic analysis, but
the core permits also to the physician to adjust any automatic result
to his opinion.

The primary analysis results come in the form of return values from
the functions in the {\tt glikeval} module, which roughly correspond to
answers to questions like:

\begin{itemize}
\item How many QRS complexes are there in the recording;
\item How many QRS morphologies can be found in the recording;
\item Which is the width of the 155'th QRS in the recording,
\end{itemize}

and so on.

\subsubsection*{The object of the primary analysis}

The primary analysis subsystem processes two kinds of files:

\begin{itemize}
\item{RCF} recording files that contain the data taken from the
ecg recorder;
\item{SIF} supplementary information files that contain additional
descriptions of the recording (where it was performed, details about
the patient, parameters found to be the best in analysing the
recording, adjustments to the results of the primary analysis as
stated by the physician). 
\end{itemize}

There are normally one recording file and one supplementary
information file. A patient may, of course, have many recordings, but
this subsystem deals only with one recording at a time.

\subsubsection*{Main components of the primary analysis}

The conditions in which this process takes place are the following: 

\begin{itemize}
\item{1.} there exist many types of recording files; 
these files differ in the way samples are ordered,
the distance (in time) between samples, the amplification of the
signal, the number of bits per sample, the accompanying information 
and, of course, the compression format;
\item{2.} conversion from a format to another using an external
utility program is rarely feasible due to the very large size of the
recording files;
\item{3.} any conclusion drawn in an automatic fashion can be
wrong (together with those that rely on it) and must thus be
adjustable by the user.
\end{itemize}

As a consequence, both the type-specific record file access and the 
editing facility are included in a transparent manner.

The primary analysis consists of:

\begin{enumerate}
\item access and decompression of the recording files, done by the
{\tt glikcess} module; this module contains submodules for each of the
file-types processed by the system;

\item detection of the fast complexes and organisation of the
recording in ``units of analysis'' called ``complexes'' that span from
one QRS complex to the next one --- the {\tt glikalys} module;

\item classification of the complexes by morphology 
--- the {\tt gliklass} module;

\item detection of the other waves in each complex,
identification of the ``lexical morphology'' and measurement of the
various distances inside a complex --- the {\tt glikmes} module;

\item management of the edits made by the user to the recording
--- the {\tt glikef} module;

\item building and management of the data structure of the primary
analysis subsystem, implementation of the virtual memory mechanisms
involved --- the {\tt glikrec} module;

\item simple interface to the client application --- the
{\tt glikeval} module.
\end{enumerate}

\begin{figure}

%% Created as ecgstruc.fig -> ecgstruc.ltx

\setlength{\unitlength}{0.012500in}%
%
\begingroup\makeatletter
% extract first six characters in \fmtname
\def\x#1#2#3#4#5#6#7\relax{\def\x{#1#2#3#4#5#6}}%
\expandafter\x\fmtname xxxxxx\relax \def\y{splain}%
\ifx\x\y   % LaTeX or SliTeX?
\gdef\SetFigFont#1#2#3{%
  \ifnum #1<17\tiny\else \ifnum #1<20\small\else
  \ifnum #1<24\normalsize\else \ifnum #1<29\large\else
  \ifnum #1<34\Large\else \ifnum #1<41\LARGE\else
     \huge\fi\fi\fi\fi\fi\fi
  \csname #3\endcsname}%
\else
\gdef\SetFigFont#1#2#3{\begingroup
  \count@#1\relax \ifnum 25<\count@\count@25\fi
  \def\x{\endgroup\@setsize\SetFigFont{#2pt}}%
  \expandafter\x
    \csname \romannumeral\the\count@ pt\expandafter\endcsname
    \csname @\romannumeral\the\count@ pt\endcsname
  \csname #3\endcsname}%
\fi
\endgroup
\begin{picture}(320,285)(160,515)
\thicklines
\put(330,515){\framebox(150,145){}}
\put(390,635){\makebox(0,0)[b]{\smash{\SetFigFont{12}{14.4}{bf}Gliks}}}
\put(160,515){\framebox(80,145){}}
\put(195,635){\makebox(0,0)[b]{\smash{\SetFigFont{12}{14.4}{bf}Wiport}}}
\put(245,515){\framebox(80,145){}}
\put(280,635){\makebox(0,0)[b]{\smash{\SetFigFont{12}{14.4}{bf}Ilfive}}}
\put(335,580){\dashbox{4}(140,45){}}
\put(400,520){\dashbox{4}(75,55){}}
\put(335,520){\dashbox{4}(60,55){}}
\put(350,525){\dashbox{4}(40,20){}}
\put(245,670){\dashbox{4}(230,35){}}
\multiput(240,670)(0.00000,7.27273){6}{\line( 0, 1){  3.636}}
\multiput(240,710)(8.18182,0.00000){6}{\line( 1, 0){  4.091}}
\multiput(285,710)(0.00000,7.14286){4}{\line( 0, 1){  3.571}}
\multiput(285,735)(-8.06452,0.00000){16}{\line(-1, 0){  4.032}}
\put(160,735){\line( 1, 0){  5}}
\multiput(165,735)(0.00000,-7.64706){9}{\line( 0,-1){  3.824}}
\multiput(165,670)(7.89474,0.00000){10}{\line( 1, 0){  3.947}}
\put(240,670){\line( 0, 1){  0}}
\put(290,710){\dashbox{4}(185,25){}}
\put(165,740){\dashbox{4}(310,25){}}
\put(160,665){\framebox(320,135){}}
\put(400,610){\makebox(0,0)[b]{\smash{\SetFigFont{12}{14.4}{rm}Record}}}
\put(435,560){\makebox(0,0)[b]{\smash{\SetFigFont{12}{14.4}{rm}Analysis}}}
\put(365,560){\makebox(0,0)[b]{\smash{\SetFigFont{12}{14.4}{rm}Access}}}
\put(370,530){\makebox(0,0)[b]{\smash{\SetFigFont{12}{14.4}{rm}2100}}}
\put(315,780){\makebox(0,0)[b]{\smash{\SetFigFont{12}{14.4}{bf}Electrocardiognosis}}}
\put(310,750){\makebox(0,0)[b]{\smash{\SetFigFont{12}{14.4}{rm}Main}}}
\put(375,720){\makebox(0,0)[b]{\smash{\SetFigFont{12}{14.4}{rm}Interpreter}}}
\put(350,685){\makebox(0,0)[b]{\smash{\SetFigFont{12}{14.4}{rm}Diagrams, Names, Events}}}
\put(220,720){\makebox(0,0)[b]{\smash{\SetFigFont{12}{14.4}{rm}Viewer}}}
\end{picture}

\bigskip
\caption{Figure~1: Interractions between the ECGNOSIS application modules}
\label{OVL:STRUCT}
\end{figure}

\bigskip\bigskip
In figure~\ref{OVL:STRUCT}
the relations of interdependence between modules are
depicted. The $T_{1--3}$ modules are lexically stored inside
{\tt glikcess} but represent drivers for various types of files. 

The most important distinction in the above structure is that between the
application and the nucleus (the interface of {\tt glikeval}). The most
usefull implementation would be to have {\tt glikeval} as a backend server
running as a sepparate process. It should perform the primary analysis for
the whole recording (or recordings if there are more than one) and answer
to the requests from the user; the parts of the analysis that are
necessary for the requests should get priority over the rest, but 
otherwise analysis should continue during user idle times.



\section{Access specification}


\PackagePurpose{
This package provides access to ECG and other signal recordings
represented in many formats. Access is done in a similar manner irrespective
of the format.
}

\dataparagraph{ECG}

\Definition{
This data structure describes a recording, being similar to the 
\typetag{FILE} type in the standard C library.
}

\subparagraph{Dimensional constants}

\begin{code}
--*/


#define STRIP_BUFFER_SIZE 4096
#define FILE_NAME_SIZE    64
#define ERUID_SIZE        64          /* Don't touch this */
#define ERUID_BUFFER_SIZE 68          /* Nor touch this */

/*--
\end{code}

\subparagraph{Recording representation types}

This section describes the types of ECG recording files that can be
read using the glikcess functions. New types are added by
modifying the module.

The types involved are:

\begin{description}
\item[{\tt ANTICOMPRESSION}] the first type of compression, used by an
old experiment done a few years ago for which there still exist a good
number of records --- 16 bit integers for the 5ms samples
--- there was always a single strip, size given by
the size of the data file.

\item[{\tt DELTABYTE}] which is a delta encoding, using 8 bits per
sample, of the {\tt ANTICOMPRESSION} type.
\end{description}

\begin{code}
--*/

#define ANTICOMPRESSION     0
#define DELTABYTE           1
#define DELTADUAL           2
#define AUTOMATA_COMPRESSION 3

/*--
\end{code}

\subparagraph{The ECG struct}

The following structure is normally created by the {\tt ecg\_open}
function, and contains all the information that can be extracted from
the recorded file itself, together with the buffering mechanism for
reading the file (the buffering mechanism is of course specific to
each format of the file read).

\begin{code}
--*/

typedef struct ecgstrip {      /* size is unique for all files */
  long stroffs;                /* offset in the file where the buffer
				  is positioned*/
  short strfill;               /* number of bytes
				  occupied in the buffer */
  short strbuf[STRIP_BUFFER_SIZE];
  void *specd;
} ECGSTRIP;



typedef struct ecgrecord {
  char eruid[ERUID_BUFFER_SIZE];
  char ecgfn[FILE_NAME_SIZE];  /* the name of the file */
  FILE *ecgf;                  /* the opened file */
  short cprtype;               /* type of compression */
  long  ecgptr;                /* current position in the record 
				* --- as returned by ecg_tell() */
  long  ecgsize;               /* size of the ecg record */
  long  ecgsamplesz;           /* samplerate in microseconds */
  
  short nofstrips;             /* number of items 
				* in the following vector */
  ECGSTRIP *es[2];                /* one for each strip */
  
  /* The following strucutres are specific to the different
   * types of compression employed  --- and are private to the
   * glikcess module. */
  
  void *specd;
} ECG;


/*--
  \end{code}

\dataparagraph{Access error returns}

  \Definition{
  The means by which the Gliks.Acess functions return exceptions.
  }

  \begin{code}
  --*/

extern short geserr;

#define CANT_OPEN_FILE 1
#define NO_MEMORY 2
#define BAD_COMPRESSION_TYPE 3
#define COMPRESSION_FILE_ERROR 4



/*--
  \end{code}

  \funparagraph{ecg\_open}
  \PurposeReturn{
  Opens a recording file of any type and returns a newly
  allocated data structure  of type \typetag{ECG} pointing to the
  opened file.
  }

  \begin{code}
  --*/

extern ECG *ecg_open(char *filename, short compression);

/*--
  \end{code}

  \subsection{ECGNOSIS-2100 access subpackage}

  \PackagePurpose{
  Provides the decompression functions for the data format
  produced by the ECGNOSIS-2100 recorder.
  }

  \begin{code}
  --*/


/*--
  \end{code}
  
  \section{Recording specification}

  \PackagePurpose{
This subpackage integrates the function of the whole Gliks
package and provides access to recordings and
pattern recognision and measurement results from the
recordings.
  }

 \dataparagraph{Gliks.Recording error conditions}

 \Definition{ % What the data structure is designed to represent
 These variables (\variabletag{glikrecerr} and \variabletag{glikerrmsg})
 are set, if Gliks.Recording functions find exceptional
 circumstances, to the following constants.
 }
 
 \begin{code}
 --*/

extern  short glikrecerr;
extern char *glikerrmsg;

/* #define NO_MEMORY 2 --- also defined as a glikcess error */
#define NO_SUCH_RECORD_FILE 100
#define BAD_SEEK 101
#define READ_PAST_EOF 102
#define BAD_ARGUMENT 103
#define MEMORY_IS_FULL 104
#define CANT_READ 105
#define CANT_ADVANCE 106
#define NO_SUCH_COMPLEX 107
#define TOO_LITTLE_ALLOCATED_MEMORY 108
#define INTRACTABLE_SEG_TYPE 109
#define NO_SUCH_PARAMETER 110
#define INTERNAL_ERROR 111



/*--
  \end{code}

  \dataparagraph{Recording file representation} 

  \begin{code}
  --*/


#define GLIKMODELSIZE 400

#define UNKN  0
#define ARTF  1
#define SV    2
#define BBB   3
#define WPW   4
#define VEN   5
#define ATIP  6

typedef struct glikmodel {
short gm[GLIKMODELSIZE];     /* a representation of the model */
short gml;                   /* length of the model */
short ngliks;                /* number of gliks matching this model */
short maxpos, minpos;        /* positions of the maximum and minimum */
short amplitude;
short supclass;              /* higher level classification */
short gtype;                 /* ARTF, UNKN, SV, BBB, WPW, VEN */
} GLIKMODEL;


typedef struct stepone {
long pl;     /* position of the detection point, from the
                beginning of the recording, in ccs */
short morph; /* morphology class */
short reff;  /* distance between the detection point and the reference
                point for the fast complex */
short st1;
short st2;
short j;
short qrsl;
short bl1;
short bl2;
} STEPONE;


typedef struct steptwo {
short narea; /* negative value for the area diagram */
short parea; /* positive */

/* The following are relative to the QRS detection point */
short fcs;   /* start of the fast complex */
short fce;   /* end of the fast complex */
short fcr;   /* reference of the fast complex
the same as |reff| in |STEPONE| */
short ps;    /* start of the p wave */
short pe;
short pr;

short ts;    /* for the T wave */
short te;
short tr;

short us;    /* for the U wave */
short ue;
short ur;    /* others may come later */
} STEPTWO;

#define PN_SIZE 64         /* maximum size of the patient name */
#define PI_SIZE 16         /* maximum size of the patient identity */
#define FO_SIZE 16         /* maximum size of the case id */
#define MAX_LEADS 2        /* maximum number of leads */
#define MAX_SEGS 2000
#define NUM_GMOS 100

typedef struct segbody {
long segbegin;            /* begining of the segment, in ccs from */
                          /* the start of the recording */
long segend;              /* end of the segment, in ccs from the start */
                          /* of the recording */

short leadsz[MAX_LEADS];  /* lead buffer size */
short leadfill[MAX_LEADS];/* lead buffer fill pointers */
short *lead[MAX_LEADS];   /* first buffers */

short itabsz;             /* size of the QRS detection-position buffer */
short itabfill;           /* fill pointer in the itab */
STEPONE *itab;            /* the first table of QRS detection points */
long itabn;               /* count of the first entry in the list of all complexes in the recording */
short stabsz;             /* size of the metrics table */
short stabfill;           /* fill pointer in the metrics table */
STEPTWO *stab;            /* the metrics table itself */
} SEGBODY;


#define LTT_I    1
#define LTT_S    2
#define LTT_L    3
#define LTT_NONE 0

#define MAX_NO_EVENTS 2000                                      /* [DEF] */ 

typedef struct perfile {
  ECG *e;
  char pn[PN_SIZE];       /* person name */
  char pi[PI_SIZE];       /* person identification */
/*  ISSTIME birth;          /* [approximate date of] birth */
  char sex;               /* either 'M' or 'F' */
/*  ISSTIME rstart;         /* start of the recording */
/*  ISSTIME rend;           /* end of the recording */

/* \dots other such items may follow here */

  short nleads;           /* actual number of leads */
  char *leadnm[MAX_LEADS]; /* their names */

                         /* a set of current parameters for this recording */
  short crt_qrs_lead;      /* the recommended lead for QRS detection */
  short st_detect_offs;    /* distance in samples from the J~point
                          * where the ST is detected
                          */

  long sgssz;             /* reference number of samples per segment */
  long sgsfill;           /* number of segments actually in sgs */
  long sgscssz;           /* size of the segment in ccs */
  SEGBODY *sgs[MAX_SEGS];
  
/* the GLIKMODEL */
  GLIKMODEL gmo[NUM_GMOS];
  short gmofill;

/********** VARIABLES FOR VIRTUAL MEMORY ********/

  long totleadsz;
  long totitabsz;
  long totstabsz;
  long totetabsz;

  long leadn;
  long itabn;
  long stabn;
  long etabn;

/* limits established for virtual memory */

  long totlim;      /* the total limit */
  long llim;        /* limits for individual types of data blocks */
  long ilim;        /* not used in the current version of the program */
  long slim;
  
/* the regions of interest in the lead, etab, stab and itab, as declared
* by the last calls to stab or itab requests */

  long lintrs_on;   /* all of the following are in ccs */
  long lintrs_off;  /* a value of -1 means they weren't yet used */
  long iintrs_on;
  long iintrs_off;  
  long sintrs_on;
  long sintrs_off;
  
/* The following tells which of the types of tables is the
* last required */

  short lasttabtype;   /* Which may have one of the values: */

/* Representation of events */

long b_events[MAX_NO_EVENTS];                                   /* [VAR] */ 
short nof_events;                                               /* [VAR] */ 

/* QRS file for intermediate results */

FILE *outqrsf;
FILE *assqrsf;

FILE *outstf;
FILE *assstf;

} PERFILE;

#define SS_NULL 0     /* [DEF] */ 
#define SS_POINT 1    /* [DEF] */ 
#define SS_SKETCH 2   /* [DEF] */ 
#define SS_MORPH 3    /* [DEF] */ 
#define SS_FULL 4     /* [DEF] */ 
#define SS_ERROR 1    /* [DEF] */ 


/*--
  \end{code}

  \macparagraph{Exception processing macros}

  \MacroSemantic{
  }

  \begin{code}
  --*/


#define raise(x) 
#define raise_with_msg(x,m)

/*--
  \end{code}

  \subsection{Segment Management}

  \FunctionInterraction{

Each segment of the recording can be in one of the following
states (the name of the state is in bold, the symbol that
identifies the state is im teletype):

\begin{itemize}
\item{\bf null} {\tt SS\_NULL} initially, when all that is known about it is
the position of the beginning in the actual recording;
\item{\bf point} {\tt SS\_POINT} when the count of the first complex in the
segment is known and stored in the segment representation;
\item{\bf sketch} {\tt SS\_SKETCH} when the {\tt itab} component is allocated, but
only the detection positions of the complexes are stored in the
{\tt itab};
\item{\bf morph} {\tt SS\_MORPH} when complexes are classified and their reference
points, together with their class are known (and stored in the
{\tt itab});
\item{\bf full} {\tt SS\_FULL} when the measures stored in the {\tt stab} are also
known.
\end{itemize}

The states above are mutually exclusive (a segment has one single such
state at a given moment) and complete (a segment must always be in 
such a state, otherwise there is a data consistency error).

A dummy segment status, {\tt SS\_ERROR} is used sometimes to return
a non-existent status.

The goal of the segment states is to make possible the evaluation 
of the {\tt glikeval} primitives for the complexes in the given segment.
Each status makes some of the primitives computable. For the complexes
in segment $n$ to be accesible it is
necessary to have all the previous ones [0..$n-1$] at least in the
point state (so that the counts of the complexes are known).

Apart from these states there also exists, for each segment and
for each lead (derivation) of the recording a ``lead status''.
The lead status tells wheather the recording data themselves
are stored in the segment. The \exceptiontag{SLS\_YES} 
and \exceptiontag{SLS\_NO}, 
together with \exceptiontag{SLS\_ERROR} constants are used for this purpose.
  }


  \begin{code}
  --*/


/*--
  \end{code}

  \funparagraph{ss\_inq}

  \PurposeReturn{
It returns the status of segment \functiontag{s} from 
recording \functiontag{p}. It may also check (in the full implementation) for 
a small number of conditions that must be compatible with the found 
status, in order to detect inconsistencies in the internal data.
  }

  \begin{code}
  --*/


short ss_inq(PERFILE *, short);


/*--
  \end{code}

  \funparagraph{ss\_chg}

  \PurposeReturn{
  This very long and important function is the superior link over all the
  \functiontag{gliklass}, 
  \functiontag{glikalys}, 
  \functiontag{glikmes} and \functiontag{glikef} functions, calling all 
  the important ones and storing their results in the \functiontag{STEPONE} and the
  \functiontag{STEPTWO} data items. This function also does the allocation and the
  desallocation of the structures in \functiontag{PERFILE} and is called directly by the
  functions that implement the memory management strategies.

  This function returns 0 if error and 1 if success.
  }

  \begin{code}
  --*/

short ss_chg(PERFILE *, short, short);


/*--
  \end{code}

  \funparagraph{segat}

  \begin{code}
  --*/

long segat(PERFILE *p, long to); 				/* [FUN] */ 

/*--
  \end{code}

  \subsection{Memory Management}
  \PackagePurpose{
  The memory management package provides a cache to the \functiontag{glikeval}
  functions (that ask for metrics related to one complex or to one moment
  in time). 
  }
  \FunctionInterraction{
  \begin{enumerate}
  \item The \functiontag{set\_total\_size} 
  function is called for a file, identifying
  the largest quantity of memory that can possibly be used by that file;
  
  \item A region of interest, on the recording, is identified using
  one of the three functions:
  
  \begin{enumerate}
  \item \functiontag{itabreq} which tells that the ``itab-level'' information
  in a given domain of time will be used;
  \item \functiontag{stabreq} which tells that the ``stab-level'' information
  in a given domain of time will be used;
  \item \functiontag{leadreq} which tells that the uncompressed ECG data
  in the given time domain will be used.
  \end{enumerate}
  
  \item Intensive calls to the \functiontag{glikeval} primitives can now be
  satisfied; calls can be made either with regard to the regions specified
  in step 2 or to other regions (but efficiency may drop in the later case);
  
  \item Step 2 can now be repeated, and/or step 1 can be called again
  to readjust the quantity of memory in use.
  
  \item Global parameters of the analysis may be changed using
  the \functiontag{set\_ana\_par} function call. 
  This may result in freeing all
  the itab and stab memory, if the results of the previous analyses
  are invalidated by the new parameters.
  \end{enumerate}

  Other primitives establish the strategy of memory use for the 
  given record file and give clues to the memory management about the
  regions in the recording that are likely to be used during the
  next processing steps.
  }

  \funparagraph{free\_one\_segment}

  \PurposeReturn{
  This function takes an ECG file and frees the memory associated with one
  segment other than the last required one. 

  This function returns a number greater than 0 if the segment contents could 
  be reclaimed --- no matter at which level.
  }

  \begin{code}
  --*/

short free_one_segment(PERFILE *p); /* [FUN] */ 



/*--
  \end{code}
  
  \funparagraph{adjust\_memory}

  \PurposeReturn{
  This function computes the total amount of memory used by the
  argument record file and frees as many segments as necessary if
  that amount is larger than the limit (see also 
  \functiontag{set\_total\_size}).
  
  This function returns 1 on success and 0 on failure.
  }

  
  \begin{code}
  --*/

short adjust_memory(PERFILE *p); 				/* [FUN] */ 

/*--
  \end{code}
  
  \funparagraph{set\_total\_size}
  
  \PurposeReturn{
  This function tells the total amount of memory that can be occupied by a 
  given file representation. This only includes the total memory used
  by the ``large'' components such as the lead buffers and the various
  per-complex tables.
  
  It returns a non-zero value if the new memory limit could be 
  successfully established, and zero if not.
  }
  
  \begin{code}
  --*/

short set_total_size(PERFILE *p, long n); /* [FUN] */ 

/*--
  \end{code}
  
  \funparagraph{reqlead}
  
  \PurposeReturn{
  This function reads a strip of data into the argument segments.
  
  This function returns 0 if could not be correctly executed
  (and alse sets \functiontag{glikrecerr} to a code of the exception that
  occured) and 1 if success.
  }
  
  \begin{code}
  --*/

short reqlead(PERFILE *p, short leadno, long segfrom, long segto); /* [FUN] */ 


/*--
  \end{code}

  \funparagraph{reqitab}
  % The name of the function is expected to be prezent
  % in the sectioning element above

  \PurposeReturn{  % be short
  This primitive makes sure that all segments in the given region
  are at least at the \functiontag{SS\_POINT} level.
  
  Returns 0 if failure (and \functiontag{glikrecerr} set to
  an exception identifier) and
  1 if success.
  }

  \begin{code}
  --*/

short reqitab(PERFILE *p, long from, long to); 			/* [FUN] */ 
     
  /*--
    \end{code}
       
  \subsection{Recording\_File}

  \PackagePurpose{

  The following functions perform opening, closing, querying and
  seting various ``system'' parameters of the record files --- such
  as name, size, sample size, etc. The parameters that are not
  queried / set by the functions in this section are those
  involving analysis of the ECG information in these files.
  }

  \funparagraph{rec\_file\_open}

  \PurposeReturn{
  This function first creates a \functiontag{PERFILE} data structure.
  It opens the
  RCF File And Tries To Get The Most Out Of It; It Opens The  SIF
  file and reads the ``global'' information from that file
  (such as the name of the patient). Then it initialises the \functiontag{PERFILE}
  data structure.
  
  If everything is OK, returns the data structure.
  If not, it frees all memory , closes files back and returns \functiontag{NULL}
  --- signifying exception. 

  This function takes two files: a \functiontag{SIF} file (possibly missing ---
  which case a NULL is to be used) and a \functiontag{RCF} one
  together with the compression type of the last one. 
  }
  \Exceptions{
  In case of exception, \functiontag{glikrecerr} is set
  to tell the nature of the exception.
  }

  \begin{code}
  --*/

PERFILE *rec_file_open(char *sif, char *rcf, short ftype); /* [FUN] */ 


/*--
  \end{code}

  \funparagraph{rec\_file\_close}

  \PurposeReturn{
  This function closes the file[s] associated with a \functiontag{PERFILE} structure
  and desallocates all of the dynamic blocks it contains. 
  }

  \begin{code}
  --*/

void rec_file_close(PERFILE *p); 				/* [FUN] */ 

/*--
  \end{code}

  \funparagraph{rec\_file parameters}
  \PurposeReturn{
  Compute various actual parameters of the recording file.

  The rec\_file\_size primitive returns the size of the file in 
  samples. The similar rec\_file\_duration primitive returns the
  size of the file in ccs. The rec\_file\_traces primitve returns
  the number of traces (derivations) in the file.
  rec\_file\_name returns a pointer to the name of the file 
  to which the argument \typetag{PERFILE} is opened. 
  }

  \begin{code}
  --*/

extern long rec_file_size(PERFILE *);
extern long rec_file_duration(PERFILE *);
extern short rec_file_traces(PERFILE *);
extern char *rec_file_name(PERFILE *);


/*--
  \end{code}

  \subsection{Results}

  \PackagePurpose{
  The definitions in this packange are the primary names of the
  results of pattern recognision and measurement on the various
  recordings.
  }

  \begin{code}
  --*/


#define CPQRSREF    1                                           /* [DEF] */ 
#define CPQRSSTART  2                                           /* [DEF] */ 
#define CPQRSEND    3                                           /* [DEF] */ 
#define CPQRSPOS    4                                           /* [DEF] */ 
#define CPQRSREFPOS 5 /* QRSREF+QRSPOS */                       /* [DEF] */ 
#define CPQRSMORPH  6 /* morphology */                          /* [DEF] */
#define CPST1       7                                           /* [DEF] */ 
#define CPST2       8                                           /* [DEF] */ 
#define CPJPOS      9                                           /* [DEF] */ 
#define CPBL1      10                                           /* [DEF] */
#define CPBL2      11                                           /* [DEF] */ 
#define CPQRSLEN   12 						/* [DEF] */ 

/*--
  \end{code}

  \funparagraph{complex\_at}

  \PurposeReturn{
  The \functiontag{complex\_at} function takes a record file and a moment
  (expressed in ccs) and returns the count of the complex that
  contains that moment (starting with 0). 
  
  It returns -1 if
  the moment does not exist or could not be found because of
  some other reason.
  
  }
  \Exceptions{
  }
  \GlobalEffects{
  % Global effects on exported or external variables
  The \functiontag{complex\_at} function sets 
  the \functiontag{last\_complex\_n} and
  the \functiontag{last\_complex\_seg} variables to the return value and to
  the number of the segment in which the complex was found 
  respectively.

  It also sets the \functiontag{last\_complex\_mom} variable to the 
  moment argument of the function.
  
  These [static] variables are provided for the use
  of the \functiontag{next\_cat} function. The calling function must
  take care to avoid calling for example \functiontag{complex\_at} with
  one record file and then \functiontag{next\_cat} with another 
  record file, or with a moment that can be very far away from
  the first moment --- this would not result in erratic
  behaviour but could be very inefficient.
  
  The named static variables may be also used by other functions
  that need to process sequences of complexes fast and have physical
  time moments as arguments.
  }
  \Performance{
  This function is rather expensive in execution time. Operations
  on sequencies of moments should not repeatedly invoke
  \functiontag{complex\_at}, but rather use the \functiontag{next\_cat} function
  below (invoking \functiontag{complex\_at} only for the first time).
  }

  \begin{code}
  --*/

long complex_at(PERFILE *p, long mom); 				/* [FUN] */ 


/*--
  \end{code}

  \funparagraph{next\_cat}

  \PurposeReturn{
  The \functiontag{next\_cat} function returns the count of the complex that
  contains a given moment of time, using the results of the last
  \functiontag{complex\_at} or \functiontag{next\_cat}.
  }
  \Exceptions{
  }
  \GlobalEffects{
  % Global effects on exported or external variables
  The \functiontag{skipped\_complex\_p} variable is set to 1 or more
  if at least one of the complexes is skipped during the process.
  
  The \functiontag{last\_complex\_n}, \functiontag{last\_complex\_seg} and the
  \functiontag{last\_complex\_mom} are also set, in order to permit
  repetitive calling of the \functiontag{next\_cat} function.
  }

  \begin{code}
  --*/

long next_cat(PERFILE *p, long mom); 				/* [FUN] */ 


/*--
  \end{code}

  \funparagraph{complexes\_between}

  \PurposeReturn{
  This function is given a recording and
  two moments (counted in ccs from the
  beginning of the recording) and it returns the number of
  complexes between the two moments.
  
  This is not properly a primitive --- it could in principle
  be rplaced by a difference between the results of two 
  \functiontag{complex\_at} calls, but is included here for the cases
  in which moments beyond the end of the recording must be 
  dealt with.
  }
  \GlobalEffects{
  % Global effects on exported or external variables
  IN CASE THE LAST IS BEYOND END OF THE RECORDING, the number of
  complexes until the end of the recording is returned. For this
  special case, the \functiontag{complexes\_between\_eor} variable is set to 1. If both
  of the argument moments are astray or some other
  error occoured, \functiontag{complexes\_between} returns -1.
  }

  \begin{code}
  --*/

long complexes_between(PERFILE *p, long m0, long m1); 		/* [FUN] */ 


/*--
  \end{code}

  \funparagraph{cpar\_of}

  \PurposeReturn{
  The following function is given a parameter and a complex number
  and it returns the parameter for the complex. The \functiontag{cpars\_of}
  is the vectorial version of \functiontag{cpar\_of}. It resolves
  the problem of \functiontag{cpar\_of} for a set of succesive complexes.
  }
  \Performance{
  For succesive application of this function call \functiontag{cpars\_of}.
  Calling \functiontag{cpar\_of} repeatedly can result in a very inefficient
  execution.
  }

  \begin{code}
  --*/

long cpar_of(PERFILE *p, short par, long n); 			/* [FUN] */ 


/*--
  \end{code}

  \funparagraph{cpar\_at, cpars\_at, cpars\_between}

  \PurposeReturn{
  These functions compute a parameter of a complex that is found
  at a given moment, the parameter for a set of complexes starting
  at a given moment, and the parameter for all the complexes
  found between two given moments.
  
  The \functiontag{buf} argument used by the \functiontag{cpars\_at} and the
  \functiontag{cpars\_between}  contains the place to store the set of
  parameters. It must point to a vector large enough to store
  that result. In case of \functiontag{cpars\_at}, this size is given by
  the user function. In case of \functiontag{cpars\_between} it
  may be found using the \functiontag{complexes\_between} function above.
  
  They return the number of complexes in actually
  written in the buffer. It returns -1 if error
  (and sets \functiontag{glikrecerr}).
  }
  \GlobalEffects{
  \functiontag{cpar\_at} sets the same variables as \functiontag{cpar\_of}
  above (see).
  }

  \begin{code}
  --*/

long cpar_at(PERFILE *p, short par, long pos); 			/* [FUN] */ 
long cpars_at(PERFILE *p, long *buf, short par, long pos, long nsam); /* [FUN] */ 

/*--
  \end{code}

  \funparagraph{get\_samples}
  % The name of the function is expected to be prezent
  % in the sectioning element above

  \PurposeReturn{  % be short
  This function  resamples the data from the input file, found
  between two well established moments in time, 
  to a given new sampling frequency.

  \functiontag{buf} will contain the results; \variabletag{p} 
  is the recording to
  read from; \functiontag{t0} and 
  \variabletag{t1} are the moment between which the
  reading must proceed; \functiontag{newper} is the new sampling
  periuod (the distance in ccs between the samples);
  \functiontag{stripno} contains the number of the stripe to read from.

  \variabletag{t1} must be larger than \variabletag{t0};

  \variabletag{buf} must point to a piece of memory containing at least
  \variabletag{1+(t1-t0)/newper} elements.

  \functiontag{get\_samples} returns the number of samples actually read.
  }

  \begin{code}
  --*/

long get_samples(PERFILE *p, short *buf, short stripno, long t0, long t1, long newper); /* [FUN] */ 


/*--
  \end{code}

  \funparagraph{Morphology processing results}

  \begin{code}
  --*/

short nof_morphologies(PERFILE *p);
short sample_morphology(PERFILE *p, short m, short *result);
short type_of_morph(PERFILE *p, short m);
long  nof_gliks_with_morph(PERFILE *p, short m);
short supclass_of(PERFILE *p, short m);
short set_supclass_of(PERFILE *p, short m, short sc);
short set_type_of_morph(PERFILE *p, short m, short t);

/*--
  \end{code}

  \subsection{Intermediary results file}

  \PackagePurpose{
  This package contains functions that perform the storage of intermediate
  results, such as QRS position detection, on disk. These values are produced
  while performing the first step of the analysis. The resulted file can 
  be ``assigned'' to the PERFILE structure, at a later time, so that
  the first step of the analysis can be shortcircuited.
  }
  \FunctionInterraction{
  The {\tt open\_qrs\_file} function initializes the disk reprezentation of the
  intermediate results, which are written by repeatedly calling 
  {\tt write\_to\_qrs\_file}; it is expected that the regions for which
  the {\tt write} function is called are not overlapping and come
  one in the continuation of the other; the process is terminated by calling 
  {\tt close\_qrs\_file}. At a later moment (eventually in another
  run of the program on the same ECG file) {\tt assign\_qrs\_file} can be
  called to make use of the stored file with a given PERFILE.
  {\tt unassign\_qrs\_file} is called to revert the meaning of the above function.
  While the QRS file is assigned to the PERFILE structure, functions
  used to determine the position of QRS complexes take information
  from the file instead of processing the recording.
  }
  \Exceptions{
  These functions normally return 0 if failure and 1 if success.
  Causes of failure are:
  
  \begin{description}
  \item[QRS\_FILE\_NOT\_CLOSED] if the {\tt open} is called twice without
  closing the file before;
  \item[CANT\_OPEN\_QRS\_FILE\_FOR\_OUT] if the operating system does not
  allow it for some reason;
  \item[DISK\_FULL\_IN\_QRS\_FILE] {\tt write} cannot accomplish it's task;
  \item[WRITE\_QRS\_IN\_NO\_FILE] if {\tt write} is called before open;
  \item[BAD\_ARGUMENTS\_TO\_QRS\_WRITE] if complexes can't be found at the
  given positions;
  \item[CLOSE\_NO\_QRS\_FILE] if there is no QRS file to close;
  \item[NO\_QRS\_FILE\_TO\_ASSIGN] if the QRS file to be 
  assigned cannot be found;
  \item[QRS\_FILE\_ALREADY\_ASSIGNED] if \functiontag{assign} is called
  sucessfully twice in a raw.
  \item[NO\_ASSIGNED\_QRS\_FILE] if {\tt unassign} is called without a
  previous {\tt assign}.
  \end{description}
  
  }
  
  \GlobalEffects{
  The {\tt outqrsf4} and {\tt assqrsf} file pointers in the PERFILE
  data structure reprezent files used as output and as assigned files
  respectively. 
  }
  \Preconditions{
  The PERFILE data structures must already be opened to a correct
  and vigurous ECG recording file. The PERFILE data structure is expected
  to have the \variabletag{outqrsf} and the \variabletag{assqrsf} slots
  initialized to \constanttag{NULL} before starting to use this package.
  }


  \funparagraph{open\_qrs\_file}

  \PurposeReturn{  % be short
  Associates to the given PERFILE data structure a newly opened,
  empty QRS file. Returns 0 if error.
  }
  \Exceptions{
  \exceptiontag{QRS\_FILE\_NOT\_CLOSED},
  \exceptiontag{CANT\_OPEN\_QRS\_FILE\_FOR\_OUT}
  }
  \GlobalEffects{
  The {\tt outqrsf} component of the PERFILE structure is assigned
  to the newly opened file.
  }

\begin{code}
--*/

short open_qrs_file(PERFILE *p, char *qrs_file_name);

/*--
\end{code}

\funparagraph{write\_to\_qrs\_file}

\PurposeReturn{  % be short
Append to the end of the QRS file the positions of the QRS complexes
found between the given moments of time. Positions INCLUDING the first
moment argument and EXCLUDING the last argument are written to the file.
}
\Exceptions{
\exceptiontag{DISK\_FULL\_IN\_QRS\_FILE}, 
\exceptiontag{WRITE\_QRS\_IN\_NO\_FILE};
}
\GlobalEffects{
None.
% Global effects on exported or external variables
}
\Preconditions{
\typetag{PERFILE} argument must have an opened QRS file.
}

\begin{code}
--*/

short write_to_qrs_file(PERFILE *p, long t0, long t1);

/*--
\end{code}

\funparagraph{close\_qrs\_file}

\PurposeReturn{
Close the \concepttag{QRS file} associated to the argument \typetag{PERFILE}.
}
\Exceptions{
\exceptiontag{CLOSE\_NO\_QRS\_FILE}
}
\GlobalEffects{
The \variabletag{outqrsf} component of the of \typetag{PERFILE} argument
is assigned \constanttag{NULL} after the file is closed.
% Global effects on exported or external variables
}
\Preconditions{
\typetag{PERFILE} argument assigned to an opened \concepttag{QRS file}.
% Conditions to be met when the function is called
}

\begin{code}
--*/

short close_qrs_file(PERFILE *p);

/*--
\end{code}

\funparagraph{assign\_qrs\_file}

\PurposeReturn{  % be short
Assigns a previously created QRS file to the current PERFILE
and returns 1 if success and 0 if failure.
}
\Exceptions{
\exceptiontag{NO\_QRS\_FILE\_TO\_ASSIGN}
}
\GlobalEffects{
Makes the \variabletag{assqrsf} component of the argument
\typetag{PERFILE} point
to the newly opened QRS file (if succeeded).
}
\Preconditions{
Existence of the file identified by argument.
}

\begin{code}
--*/

short assign_qrs_file(PERFILE *p, char *qrsfn); 		/* [FUN] */ 

/*--
\end{code}

\funparagraph{unassign\_qrs\_file}

\PurposeReturn{  % be short
This is the reverse of the \filetag{assign\_qrs\_file} function,
in that the assignement is made not to hold anymore.
}
\Exceptions{
\exceptiontag{NO\_ASSIGNED\_QRS\_FILE}
}
\GlobalEffects{
Sets the \variabletag{assqrsf} slot of the argument \typetag{PERFILE} to
\constanttag{NULL}.
% Global effects on exported or external variables
}
\Preconditions{
The \typetag{PERFILE} previously assigned to a QRS file.
% Conditions to be met when the function is called
}

\begin{code}
--*/

short unassign_qrs_file(PERFILE *p);



/*--
  \end{code}

  \subsection{Functions that set-get analysis parameters}

  \funparagraph{Parameter set-get functions}

  \begin{code}
  --*/

void set_crt_qrs_lead(PERFILE *p, short v);
void set_st_detect_position(PERFILE *p, short stpos);
short get_st_detect_position(PERFILE *p);


/*--
\begin{code}

\section{Analysis specification}
\end{code}
--*/

/*--
  \end{code}


  \endoffile

  --*/

