/* 
 * Processing of the .rir (and, for that matter, qst) files.
 *

    Copyright (c) 1996 Alexandru Dan Corlan MD (http://dan.corlan.net)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 */


#include <stdio.h>
#include <malloc.h>
#include <rir.h>

RIR *rir_open(char *fn, short rir_type)

{
  RIR *rv;
  long lastpos;
  long offse;
  long othoffs;

  rv = (RIR *) malloc(sizeof(RIR));
  
  if (rv == NULL) {
    /* exception: memory full? */
    return NULL;
  }
  
  rv->rirf = fopen(fn, "r");
  if (rv->rirf==NULL) {
    /* exception: no such file, fn */
    free(rv);
    return NULL;
  }
  
  if(rir_type == QST_TYPE) {
    if(fseek(rv->rirf, 0, SEEK_END)==-1) {
      free(rv);
      return NULL;
    }
    offse = - 6 - (ftell(rv->rirf) % 6);
    do {
      if(fseek(rv->rirf, offse, SEEK_END)==-1) {
	free(rv);
	return NULL;
      }
      rv->n_entries = 1 + ftell(rv->rirf) / 6;
      if(fread(&lastpos, 4, 1, rv->rirf)!=1) {
	free(rv);
	return NULL;
      }
      offse = offse - 6;
    } while((rv->n_entries > 3) && lastpos==79808488);
    /* compensation for aberant end of unknown cause of the .qst file */
/*    fprintf(stderr, " --lastpos=%d-- ", lastpos);*/
    rv->last_place = lastpos;
    fseek(rv->rirf, 0, SEEK_SET);
    rv->last_offset = 0;
    rv->rir_type = QST_TYPE;
    return rv;
  }
  return NULL;  /* true rir not implemented yet */
}

void rir_close(RIR *rf)

{
  if(rf != NULL) {
    fclose(rf->rirf);
    free(rf);
  }
}

int rir_has(RIR *rf, short meas_type)

{
  /* just for qst */
  switch(meas_type) {
  case RIR_PLACE:
  case RIR_ST_A:
  case RIR_ST_B:
    return 1;
    break;
  default:
    return 0;
    break;
  }
}

long rir_duration(RIR *rf)

{
  return rf->last_place;
}

long rir_meas(RIR *rf, short meas_type)

{
  long rv;

  /* just for qst, in the ECGNOSIS 2100 files */
  switch(meas_type) {
  case RIR_PLACE:
    rv = *((long *)rf->crt);
    break;
  case RIR_ST_A:
    rv = rf->crt[4] * 25;  /* 0.025 uV per level in ECGNOSIS 2100 */
    break;
  case RIR_ST_B:
    rv = rf->crt[5] * 25;  /* 0.025 uV per level in ECGNOSIS 2100 */
    break;
  default:
    break;
  }
  return rv;
}

int rir_read_next_record(RIR *rf)

{
  int rv;

  /* assumes .qst file */
  rv = fread(rf->crt, 1, 6, rf->rirf);
  if(rv<6)
    return -1;  /* end of file */
  return 0;
}

/* End of the rir.c module */
